// RootflyGraphics.h: interface for the RootflyGraphics class.
//
//////////////////////////////////////////////////////////////////////

#include "DataStructures.h"
#include "ImageClientConverter.h"

#if !defined(AFX_ROOTFLYGRAPHICS_H__A56BBC24_46B8_41A2_9CDD_17049F680F3D__INCLUDED_)
#define AFX_ROOTFLYGRAPHICS_H__A56BBC24_46B8_41A2_9CDD_17049F680F3D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class RootflyGraphics  
{
public:
  RootflyGraphics();
  virtual ~RootflyGraphics();

  void Initialize( const ImageClientConverter&, const Settings&, const RootflyData& );

  void FillImageDisplayAreaWithGray(CDC& dc, const CRect &rect) const;

  // !!! really should only need one SetClipRegion function, but need to refactor RootflyDlg to fix this
  void SetClipRegion(CDC& dc, bool zoomed) const;
  void SetClipRegion(CDC& dc, const CRect& rect) const;

  void DrawAllRoots(CDC& dc, int mode, bool zoomed) const;
  void DrawSelectedRoot(CDC& dc, int selected_id, int point_session, int mode, const CPoint& mouse_point, bool xtend_tail, int x_offset) const;
  void DrawNewRoot(CDC& dc, int mode, const RootInstance &ri, const CPoint& mouse_point, const int x_offset) const; // 3x3  -MTB

protected:

  void DrawPolyline( const std::vector<RPoint>& line, COLORREF c, int width, CDC& dc, const int x_offset ) const; // 3x3  -MTB
  void DrawPoint(const RPoint& point, CDC& dc, int offset) const;
  void DrawPoints(const std::vector<RPoint>& line, CDC& dc, int offset) const;
  void DrawDiameter(const RootInstance& inst, COLORREF color, bool draw_point, CDC& dc, const int x_offset ) const; //3x3 -MTB

  bool IsRootInCurrentWindow( const RootInstance &inst ) const;
  COLORREF RootColorToRgb( RootInstance::Color tempcolor ) const;
  COLORREF GetRootColor(const RootInstance& inst, bool selected) const;
  int GetRootLineDrawingWidth( bool selected ) const;
  void DrawRootId(const Root& root, bool selected, int current_window, int session, CDC& dc, const int x_offset) const;
  void DrawSingleRoot(CDC& dc, const Root &root, int window, int session, bool extend_root, bool extend_direction, const CPoint& mouse_point, bool selected, bool show_diam, bool show_points, bool show_id) const;
  void ExtendPolyline( const RootInstance& ri, bool extend_from_front, COLORREF color, int width, CPoint mouse_point, CDC& dc, const x_offset) const; // 3x3 -MTB

private:
  const ImageClientConverter *rfy_dlg_conv;
  const Settings *rfy_dlg_settings;
  const RootflyData *rfy_dlg_data;
  static const int m_point_size;
};

#endif // !defined(AFX_ROOTFLYGRAPHICS_H__A56BBC24_46B8_41A2_9CDD_17049F680F3D__INCLUDED_)
