#include "Test.h"
#include <iostream>  // should no longer be needed once all printf and cout are converted to OutputDebugString -- STB 4/20/10

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


extern bool g_collapse_sessions;


namespace RootflyTester
{
  
void TestExample()
{
  OutputDebugString("Testing example...");
  
  bool error = false;
  if (error)
  {
    OutputDebugString("\nERROR!!!  Unable to load file.\n");
  }
  
  OutputDebugString("done\n");
}

void MainTest()
{
  system("mkdir test");
  OutputDebugString("*****************************************\n");
  OutputDebugString("*** Testing the Rootfly codebase:\n");
  TestExample();
 TestRootInstance();
  TestRoot();
  TestTube(); 
  TestExperiment();  
  TestCurrentImageProperties();
  //  TestSessionCollapser();
  TestRootInstanceLoadSave();
  TestRootLoadSave();
  TestLoadAndSave();
  TestSettings();
  OutputDebugString("*** Finished testing the Rootfly codebase.\n");
  OutputDebugString("*****************************************\n");
}

/////////////////////////////////////////////////////////////////////////////
// Test data structures

void TestRootInstance()
{
  OutputDebugString("Testing RootInstance...");
  
  //test condition code
  const CString test_condition_code = "test_code";
  //vector for testing GetWindowNumbers()
  vector<int> test_window_list;
  //iterator variable for "for" loops
  int i = 0;
  // variable used in GetWindowNumbers() test
  bool match;
  // test radius value for diameter
  double rad = 30;
  //variable for FindLengthInWindow
  double length = 0;
  //variable to hold average from GetAverageDiameter
  double average = 0;
  //variable for testing GetAverageDiameter and FindLengthInWindow
  double conversion = 0.5;
  
  
  //create the root instance, Rpoint pointer, and RDiamter Pointer
  RootInstance root;
  RPoint *point;
  RDiameter *diameter;
  
  //loops through to create 3 points in m_point. 
  //length of these points combined is 20 pixels
  for(i=0;i<3;i++){
    point = root.CreatePoint();
    point->x = i*10;
    point->y = 0;
    point->window = 1;
  }
  
  //create a diameter for the root instance
  diameter = root.CreateDiameter();
  diameter->radius = rad;
  
  
  //*****Testing IsAlive******
  
  //set root to alive
  root.SetAlive();
  if(!root.IsAlive()){
    OutputDebugString("ERROR:  Root alive test failed.\n");
  }
  
  //set root to dead
  root.SetDead();
  if(root.IsAlive()){
    OutputDebugString("ERROR:  Root dead test failed.\n");
  }
  
  
  //******Testing SetSessionNumber******
  
  //set the session number to 1
  root.SetSessionNumber(1);
  if(root.GetSessionNumber() != 1){
    OutputDebugString("ERROR:  Session Number test failed.\n");
  }
  
  
  //******Testing Condition Code******
  //set the condition code to the test value of "test_code"
  root.SetConditionCode(test_condition_code);
  if(root.GetConditionCode() != test_condition_code){
    OutputDebugString("ERROR:  Condition Code test failed.\n");
  }
  
  
  //******Testing IsRootInWindow******
  //NOTE: this tests to make sure that function returns true when supplied
  //a correct window value and returns false when supplied an incorrect window value
  if(!root.IsRootInWindow(1)){
    OutputDebugString("ERROR:  Correct window value for IsRootInWindow test failed.\n");
  }
  if(root.IsRootInWindow(2)){
    OutputDebugString("ERROR:  Incorrect window value for IsRootInWindow test failed.\n");
  }
  
  //******Testing SetColor and GetColor******
  //Setting color to light brown since white is the default.
  root.SetColor(RootInstance::COLOR_LIGHTBROWN);
  if(root.GetColor() != RootInstance::COLOR_LIGHTBROWN){
    OutputDebugString("ERROR: SetColor test failed.\n");
  }
  
  //******Testing GetWindowNumbers******
  root.GetWindowNumbers(&test_window_list);
  for(i=0;i<test_window_list.size();i++){
    point = root.GetPoint(i);
    if(test_window_list[i] == point->window){
      match = true;
    }
    else{
      match = false;
      break;
    }
  }
  if(!match){
    OutputDebugString("ERROR:  GetWindowNumbers test failed.\n");
  }
  
  
  //******Testing FindLengthInWindow******
  //conversion is 0.5 and the length in pixels is 20
  //therfore the length will be 40
  length = root.FindLengthInWindow(1, conversion);
  if(length != 40){
    OutputDebugString("ERROR:   FindLengthInWindow test failed.\n");
  }
  
  
  //******Testing GetAverageDiameter******
  //conversion is 0.5 and the diamter is 60 pixels (2*rad where rad = 30 as defined above).
  //since there is only one diameter per root currently,
  //the average should come out to 120.
  average = root.GetAverageDiameter(conversion);
  if(average != 120){
    OutputDebugString("ERROR:  GetAverageDiameter test failed.\n");
  }
  
  //******Testing DeleteDiameter******
  //deletes the only diameter present which is at index 0.
  if(!root.DeleteDiameter(0)){
    OutputDebugString("ERROR:  DeleteDiameter test failed.\n");
  }
  
  
  OutputDebugString("done\n");
}

void TestRoot()
{
	int i,j;
	const int ROOTID = 1;
	const int ROOTDISPID = 2;
	const int ROOTWINID = 3;
	const int CONVERSION_FACTOR =1;
	struct IndivRootData
	{
		std::vector<double> diam;
		std::vector<double> length;
	} rootsdata;

	printf("Testing Root Class Started\n");
	////////
	//Set up a bunch of roots to use later in the test
	////////

	srand (1); // seed with one so its the same everytime  root[7] Must not be max length!
	//create a bunch of random root instances
	RootInstance root[10];
	for(i=0;i<8;i++)
	{
		int max;
		root[i].SetSessionNumber(i);
		max = 1+(rand()%4);
		for(j=0;j<max;j++)
		{
			root[i].CreatePoint();
			root[i].MovePoint(j,abs(rand()%100),abs(rand()%100));
			root[i].ChangeWindowOfPoint(j,0);

		}
		root[i].CreateDiameter();
		root[i].ResizeDiameter(0,abs(1+rand() % 10)); //abs(1+rand() % 10)
	}

	for(i=0;i<8;i++)
	{
		rootsdata.length.push_back(root[i].FindLengthInWindow(0,CONVERSION_FACTOR));
		rootsdata.diam.push_back(root[i].GetAverageDiameter(CONVERSION_FACTOR));
	}

	////////
	///Done Setting up stuff
	////////


	//create root with ID of ROOTID
	Root TestRoot(ROOTID);
	TestRoot.SetDisplayableID(ROOTDISPID);
	TestRoot.SetPrimaryWindow(ROOTWINID);

	if(TestRoot.GetID() != ROOTID)
	{
		printf("Error in testing Root class:  Checking empty structure Method: GetID()\n");
	}
	if(TestRoot.GetDisplayableID() != ROOTDISPID)
	{
		printf("Error in testing Root class:  Checking empty structure Method: GetDisplayableID()\n");
	}
	if(TestRoot.GetPrimaryWindow() != ROOTWINID)
	{
		printf("Error in testing Root class:  Checking empty structure Method: GetPrimaryWindow()\n");
	}

	//check all bounding methods when list is empty
  /*
  //these checks are asseted to fail
	if(TestRoot.GetLastSessionNumber() != -1)
	{
		printf("Error in testing Root class:  Checking empty structure on method GetLastSessionNumber()\n");
	}
	if(TestRoot.GetMaxLength(CONVERSION_FACTOR) != -1)
	{
		printf("Error in testing Root class:  Checking empty structure on method GetMaxLength()\n");
	}
	if(TestRoot.GetMaxDiameter(CONVERSION_FACTOR) != -1)
	{
		printf("Error in testing Root class:  Checking empty structure on method GetMaxDiameter()\n");
	}
	if(TestRoot.GetFirstDeadSessionNumber() != -1) // These checks are aserted to fail
	{
		printf("Error in testing Root class");
	}
	if(TestRoot.GetFirstSessionNumber() != -1)
	{
		printf("Error in testing Root class");
	}
	if(TestRoot.GetFirstSessionNumber() != -1)
	{
		printf("Error in testing Root class");
	}
	*/
	
	if(TestRoot.GetRootInstance(1) != NULL)
	{
		printf("Error in testing Root class:  Checking empty structure on method GetRootInstance()\n");
	}

	
	//create root with session of 1
	RootInstance* new_root = TestRoot.CreateRootInstance(1);
	//root[1] = *new_root;
	*new_root = root[1];
	if(TestRoot.GetRootInstance(1) != new_root)
	{
		printf("Error in testing Root class:  Checking 1 root case Method GetRootInstance() \n");
	}
	if(TestRoot.GetLastSessionNumber() != 1)
	{
		printf("Error in testing Root class:  Checking 1 root case Method GetLastSessionNumber()\n");
	}
	if(TestRoot.GetFirstSessionNumber() != 1)
	{
		printf("Error in testing Root class:  Checking 1 root case Method GetFirstSessionNumber()\n");
	}
	if(TestRoot.GetMaxLength(CONVERSION_FACTOR) != rootsdata.length[1]) 
	{
		printf("Error in testing Root class:  Checking 1 root case Method GetMaxLength()\n");
	}
	if(TestRoot.GetMaxDiameter(CONVERSION_FACTOR) != rootsdata.diam[1])
	{
		printf("Error in testing Root class:  Checking 1 root case Method GetMaxDiameter\n");
	}

	//Remove root then check if empty
	if(!TestRoot.DeleteRootInstance(1))
	{
		printf("Error in testing Root class DeleteRootInstance() returned false\n");
	}
  /*
  asserted to fail
	if(TestRoot.GetLastSessionNumber() != -1)
	{
		printf("Error in testing Root class: After performing delete a root still exists\n");
	}
  */

	//Create a bunch of roots
	//Notice that they are out of order.
	RootInstance* roottokill;
	std::vector<RootInstance> Collection;
	new_root = TestRoot.CreateRootInstance(7);
	//root[7] = *new_root;
	*new_root = root[7];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(7) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(2);
	//root[2] = *new_root;
	*new_root = root[2];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(2) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(3);
	//root[3] = *new_root;
	*new_root = root[3];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(3) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(4);
	//root[4] = *new_root;
	*new_root = root[4];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(4) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(5);
	//root[5] = *new_root;
	*new_root = root[5];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(5) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(6);
	//root[6] = *new_root;
	*new_root = root[6];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(6) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}
	new_root = TestRoot.CreateRootInstance(1);
	//root[1] = *new_root;
	*new_root = root[1];
	Collection.push_back(*new_root);
	if(TestRoot.GetRootInstance(1) != new_root)
	{
		printf("Error in testing Root class: Creating root failed\n");
	}


	if(TestRoot.GetFirstSessionNumber() != 1)
	{
		printf("Error in testing Root class: GetFirstSessionNumber() failed to return correct value\n");
	}
	if(TestRoot.GetLastSessionNumber() != 7)
	{
		printf("Error in testing Root class: GetLastSessionNumber() failed to return correct value\n");
	}
	if(TestRoot.GetFirstDeadSessionNumber() != -2)
	{
		printf("Error in testing Root class: GetFirstDeadSessionNumber should of returned -2 because no dead roots\n");
	}
	if(TestRoot.GetMaxLength(CONVERSION_FACTOR) != *max_element(rootsdata.length.begin(),rootsdata.length.end()))
	{
		printf("Error in testing Root class: GetMaxLength() failed in returning the correct value\n");
	}
	if(TestRoot.GetMaxDiameter(CONVERSION_FACTOR) != *max_element(rootsdata.diam.begin(),rootsdata.diam.end()))
	{
		printf("Error in testing Root class: GetMaxDiameter() failed to return correct value\n");
	}

	//Checking GetAllRootInstance method
	std::vector<RootInstance> get_roots = *TestRoot.GetAllRootInstances();
	for(std::vector<RootInstance>::iterator it = get_roots.begin(); it < get_roots.end();it++)
	{
		for(std::vector<RootInstance>::iterator CollectionItr = Collection.begin(); CollectionItr < Collection.end();CollectionItr++)
		{
			if(*it == *CollectionItr)
			{
				break;
			}
		}
		if(CollectionItr == Collection.end())
		{
			printf("Error in testing Root class: Error checking that all roots were inserted properly\n");
		}
	}

	//Kill a root then check if it can be found
	roottokill = TestRoot.GetRootInstance(5);
	roottokill->SetDead();
	if(TestRoot.GetFirstDeadSessionNumber() != 5)
	{
		printf("Error in testing Root class:  Getting First Dead Session, Should of been 5 but was %d\n",TestRoot.GetFirstDeadSessionNumber());
	}


	//Check Copy method
	//Get index of root with largest length
	//Copy that root to (index + 1)
	const int session_to_copy = 1;
	double length = rootsdata.length[session_to_copy];
	RootInstance *root_to_replace;
	root_to_replace = TestRoot.GetRootInstance(session_to_copy+1); // copy root to root_to_replace
	TestRoot.CopyRootInstance(session_to_copy,(session_to_copy+1));
	TestRoot.DeleteRootInstance(session_to_copy);
	//Check to see if root at max_index is largest length
	if(length != TestRoot.GetRootInstance(session_to_copy + 1)->FindLengthInWindow(0,CONVERSION_FACTOR))
	{
		printf("Error in testing Root class: Copy() method did not work\n");
	}
	//check that the size has been decreased.
	if(TestRoot.GetAllRootInstances()->size() != 6)
	{
		printf("Error in testing Root class: The size of all root instances is not right after deleting\n");
	}
	for(i=1;i<8;i++)
	{
		RootInstance* temp = TestRoot.GetRootInstance(i);
		if(TestRoot.GetRootInstance(i) != NULL)
		{
			if(root_to_replace == TestRoot.GetRootInstance(i))
			{
				printf("Error in testing Root class: Root that should of been replaced during Copy() testing was found!\n");
			}
		}
		
	}


	//Test ClearAll() method
	TestRoot.ClearAll();
	//check if empty
	if(TestRoot.GetLastSessionNumber() != -1)
	{
		printf("Error in testing Root class: List is not empty after calling ClearAll()");
	}
	printf("Testing Root Class Compleate\n\n");
}

void TestTube()
{
  //Declare Values
  const int NEXTROOTID = 47;
  const CString INITTAG = "initializer tag";
  const CString CHANGETAG = "changed tag";
  
  const int NUMIMAGES = 5;
  const int MULTIPLIER = 3;
  const int IMGDATAWIN[] = { 1, 2, 3, 2, 2 };
  const int IMGDATASSN[] = { 1, 1, 1, 2, 3 };
  
  OutputDebugString("Testing Tube...");
  
  //Create Tube (using initializer constructor) and test if the constructor is valid
  Tube test_tube(INITTAG);
  if(strcmp(INITTAG, test_tube.GetTag()))
  {
    // error
    OutputDebugString("ERROR:  Tube:  constructor tag does not match GetTag\n");
  }
  
  test_tube.SetTubeTag(CHANGETAG);
  if(strcmp(CHANGETAG, test_tube.GetTag()))
  {
    // error
    OutputDebugString("ERROR:  Tube:  changed tag does not match GetTag\n");
  }
  
  // Populate Image Vectors
  for(int i = 0; i < NUMIMAGES; i++)
  {
    test_tube.SetImageExists(true, IMGDATAWIN[i], IMGDATASSN[i]);
    test_tube.SetImageScanned(true, IMGDATAWIN[i], IMGDATASSN[i]);
    test_tube.SetImageAnnotated(true, IMGDATAWIN[i], IMGDATASSN[i]);
  }
  
  /////////////////////////
  // Image Exists Functions
  /////////////////////////
  int j = 0;
  for(i = 0; i < NUMIMAGES; i++)
  {
    j = i * MULTIPLIER % NUMIMAGES;
    
    // Test is image data was successfully added
    if(!test_tube.GetImageExists(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Exists:  image does not exist\n");
    }
    
    // Remove image data and test if it was successfully removed
    test_tube.SetImageExists(false, IMGDATAWIN[j], IMGDATASSN[j]);
    if(test_tube.GetImageExists(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Exists:  image not removed\n");
    }
  }
  
  //////////////////////////
  // Image Scanned Functions
  //////////////////////////
  
  for(i = 0; i < NUMIMAGES; i++)
  {
    j = i * MULTIPLIER % NUMIMAGES;
    
    // Test if image data was successfully added
    if(!test_tube.GetImageScanned(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Scanned:  image does not exist\n");
    }
    
    // Remove image data and test if it was successfully removed
    test_tube.SetImageScanned(false, IMGDATAWIN[j], IMGDATASSN[j]);
    if(test_tube.GetImageScanned(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Scanned:  image not removed\n");
    }
  }
  
  ////////////////////////////
  // Image Annotated Functions
  ////////////////////////////
  
  for(i = 0; i < NUMIMAGES; i++)
  {
    j = i * MULTIPLIER % NUMIMAGES;
    
    // Test if image data was successfully added
    if(!test_tube.GetImageAnnotated(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Annotated:  image does not exist\n");
    }
    
    // Remove image data and test if it was successfully removed
    test_tube.SetImageAnnotated(false, IMGDATAWIN[j], IMGDATASSN[j]);
    if(test_tube.GetImageAnnotated(IMGDATAWIN[j], IMGDATASSN[j]))
    {
      // error
      OutputDebugString("ERROR:  Tube:  Image Annotated:  image not removed\n");
    }
  }
  
  OutputDebugString("done\n");
  
}

void TestExperiment()
{
  OutputDebugString("Testing Experiment...");
  
  Experiment test;
  
  //Check if these functions fail when no tubes are created
  if(test.GetTubeByName("B") != NULL) printf("Error:  GetTubeByName failed\n");
  if(test.GetNTubes() != 0) printf("Error:  GetNTubes failed\n");
  
  // TEST CreateTube()
  test.CreateTube("A");
  test.CreateTube("B");
  test.CreateTube("C");
  if (test.GetNTubes() != 3) printf("Error:  CreateTube is broken");
  
  // TEST GetTubeByName(): for an actual tube and a non-existent tube
  if (test.GetTubeByName("B") == NULL) printf("Error:  GetTubeByName NULL failure\n");
  if (test.GetTubeByName("F") != NULL) printf("Error:  GetTubeByName returned tube not in experiment\n");
  
  // TEST GetFirstTubeName & GetLastTubeName
  if (test.GetFirstTubeName() != "A") printf("Error: GetFirstTubeName failure\n");
  if (test.GetLastTubeName() != "C") printf("Error: GetLastTubeName failure\n");
  
  // TEST GetNextTubeName
  if (test.GetNextTubeName("A") != "B") printf("Error: GetNextTubeName failure A->B\n");
  if (test.GetNextTubeName("B") != "C") printf("Error: GetNextTubeName failure B->C\n");
  if (test.GetNextTubeName("C") != "C") printf("Error: GetNextTubeName failure C->C\n");
  if (test.GetNextTubeName("F") != "C") printf("Error: GetNextTubeName failure F->C\n");
  
  //TEST GetPrevTubeName
  if (test.GetPrevTubeName("A") != "A") printf("Error: GetPrevTubeName failure A->A\n");
  if (test.GetPrevTubeName("B") != "A") printf("Error: GetPrevTubeName failure B->A\n");
  if (test.GetPrevTubeName("C") != "B") printf("Error: GetPrevTubeName failure C->B\n");
  if (test.GetPrevTubeName("F") != "A") printf("Error: GetPrevTubeName failure F->A\n");
  
  // TEST DeleteTube(): Check if the tube is deleted
  test.DeleteTube("C");
  if (test.GetTubeByName("C") != NULL) printf("Error:  DeleteTube failed\n");
  
  // TEST ClearAll(): Test if the tubes are gone
  test.ClearAll();
  if (test.GetNTubes() != 0) printf("Error:  ClearAll failed\n");
  
  OutputDebugString("done\n");
}

/////
//
//  ** This TestSettings tests all the functionality of the Settings class in DataStructures.cpp.
//
///// 

void TestSettings()
{
  OutputDebugString("Testing settings...");
  Settings my_settings;
  
  //test AddConditionSet and GetConditionSize
  CString a[8] = {"ab","cd","ef","gh","ij","kl","mn","op"};
  vector<CString> v_test_cv(8);
  std::copy(&a[0], &a[sizeof(a) / sizeof(CString)], v_test_cv.begin());
  my_settings.AddConditionSet(v_test_cv);
  //	int test_cv = my_settings.GetConditionSize();
  //	if(my_settings.GetConditionSize() != v_test_cv.size())
  //		OutputDebugString("ERROR:  AddConditionSet or GetConditionSize unsuccessful.\n");
  
  //test GetConditionSet
  if(my_settings.GetConditionSet()!=v_test_cv) 
    OutputDebugString("ERROR:  AddConditionSet or GetConditionSet unsuccessful.\n");
  
  //test AddConditionCode, GetConditionCode
  CString test_cc = "test string";
  my_settings.AddConditionCode(test_cc);
  if(my_settings.GetConditionSet()[v_test_cv.size()] != test_cc) 
    OutputDebugString("ERROR:  Add- or Get- ConditionCode unsuccessful.\n");
  
  //Test Set- and Get- DefaultFileDir() 
  CString my_file_dir = "C:\\Testing_Directory";
  my_settings.SetDefaultFileDir(my_file_dir);
  if(my_file_dir != my_settings.GetDefaultFileDir())
    OutputDebugString("Error:  Get- or Set- Default File Dir unsuccessful. \n");
  
  //test GetVersion()
  CString my_version = my_settings.GetVersion();
  if(my_version != "Rootfly_21") 
    OutputDebugString("ERROR:  GetVersion() unsuccessful. \n");
  
  bool test_status_tb = true;
  bool test_status_fb = false;
  
  bool test_status_t = true;
  bool test_status_f = false;
  
  //test SetShowRootID and RootIdIsEnabled
  my_settings.SetShowRootID(test_status_fb);
  if(my_settings.GetShowRootID() != test_status_fb )
    OutputDebugString("ERROR:  SetShowRootID() or GetShowRootID() unsuccessful. \n");
  
  //test SetShowDeadRoots and DeadRootsIsEnabled
  my_settings.SetShowDeadRoots(test_status_tb);
  if(my_settings.GetShowDeadRoots() != test_status_tb)
    OutputDebugString("ERROR:  SetShowDeadRoots() or GetShowDeadRoots() unsuccessful. \n");
  
  //test SetShowRoots and ShowRootsIsEnabled
  my_settings.SetShowRoots(test_status_fb);
  if(my_settings.GetShowRoots() != test_status_fb) 
    OutputDebugString("ERROR:  SetShowRoots() or GetShowRoots() unsuccessful. \n");
  
  //test SetShowDiameters and ShowDiametersIsEnabled
  my_settings.SetShowDiameters(test_status_tb);
  if(my_settings.GetShowDiameters() != test_status_tb)
    OutputDebugString("ERROR:  SetShowDiameters() or GetShowDiameters() unsuccessful. \n");
  
  //test SetAutoDetectOptions, and GetAutoDetect*
  float di_value = 2;
  float length_val = 2;
  float degree_val = 2;
  
  my_settings.SetAutoDetectOptions(test_status_f, test_status_t, di_value, test_status_f, length_val, test_status_t, degree_val, test_status_f);
  
  if(my_settings.GetAutoDetectDiameterCheck() != test_status_t )
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectDiameterCheck() unsuccessful.\n");
  if(my_settings.GetAutoDetectLengthCheck() != test_status_f) 
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectLengthCheck() unsuccessful.\n");
  if(my_settings.GetAutoDetectVerticalCheck() != test_status_t )
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectVerticalCheck() unsuccessful.\n");
  if(my_settings.GetAutoDetectPostProcessCheck() != test_status_f )
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectPostProcessCheck() unsuccessful.\n");
  if(my_settings.GetAutoDetectDiameterValue() != di_value )
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectDiameterValue() unsuccessful.\n");
  if(my_settings.GetAutoDetectLengthValue() != length_val)
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectLengthValue() unsuccessful.\n");
  if(my_settings.GetAutoDetectDegreeValue() != degree_val)
    OutputDebugString("ERROR:  SetAutoDetectOptions() or GetAutoDetectDegreeValue() unsuccessful.\n");
  
  //test SetRootDrawingColor* and GetRootDrawingColor*
  COLORREF srdc_l = RGB(0,0,0);
  COLORREF srdc_ls = RGB(255,0,0);
  COLORREF srdc_d = RGB(255,255, 0);
  COLORREF srdc_ds = RGB(255, 255, 255);
  
  my_settings.SetRootDrawingColorLive(srdc_l);
  if(my_settings.GetRootDrawingColorLive() != srdc_l )
    OutputDebugString("ERROR:  SetRootDrawingColorLive() or GetRootDrawingColorLive() unsuccessful.\n");
  
  my_settings.SetRootDrawingColorLiveSelected(srdc_ls);
  if(my_settings.GetRootDrawingColorLiveSelected() != srdc_ls )
    OutputDebugString("ERROR:  SetRootDrawingColorLiveSelevted() or GetRootDrawingColorLiveSelected() unsuccessful.\n");
  
  my_settings.SetRootDrawingColorDead(srdc_d);
  if(my_settings.GetRootDrawingColorDead() != srdc_d )
    OutputDebugString("ERROR:  SetRootDrawingColorDead() or GetRootDrawingColorDead() unsuccessful.\n");
  
  my_settings.SetRootDrawingColorDeadSelected(srdc_ds);
  if(my_settings.GetRootDrawingColorDeadSelected() != srdc_ds )
    OutputDebugString("ERROR:  SetRootDrawingColorDeadSelected() or GetRootDrawingColorDeadSelected() unsuccessful.\n");
  
  //test GetDefaultRootDrawingColor*
  COLORREF drdc_l = RGB(255, 0, 0);
  COLORREF drdc_ls = RGB(0,255,0);
  COLORREF drdc_d = RGB(0,0,0);
  COLORREF drdc_ds = RGB(200,200,200);
  
  if(my_settings.GetDefaultRootDrawingColorLive() != drdc_l )
    OutputDebugString("ERROR:  GetDefaultRootDrawingColorLive() unsuccessful.\n");
  if(my_settings.GetDefaultRootDrawingColorLiveSelected() != drdc_ls )
    OutputDebugString("ERROR:  GetDefaultRootDrawingColorLiveSelected() unsuccessful.\n");
  if(my_settings.GetDefaultRootDrawingColorDead() != drdc_d )
    OutputDebugString("ERROR:  GetDefaultRootDrawingColorDead() unsuccessful.\n");
  if(my_settings.GetDefaultRootDrawingColorDeadSelected() != drdc_ds )
    OutputDebugString("ERROR:  GetDefaultRootDrawingColorDeadSelected() unsuccessful.\n");
  
  //test Set and Get DefaultIndex
  my_settings.SetConditionDefaultIndex(2);
  if(my_settings.GetConditionDefaultIndex() != 2 )
    OutputDebugString("ERROR:  Set - or Get- Condition Default Index unsuccessful.\n");
  
  //test Set- and Get- LastWinLocation
  CRect test_rect(2,4,6,8);
  my_settings.SetLastWinLocation(test_rect);
  if(my_settings.GetLastWinLocation() != test_rect)
    OutputDebugString("ERROR:   Set- or Get- LastWinLocation() unsuccessful.\n");
  
  //test Save - and Read- UserPreferences, but cannot REALLY test m_condition_size
  my_settings.SaveUserPreferences();
  my_settings.ReadUserPreferences();
  
  bool sr_test = true;
  if(my_settings.GetVersion() != "Rootfly_21" ) sr_test = false;
  if(my_settings.GetDefaultFileDir() != my_file_dir) sr_test = false;
  if(my_settings.GetShowRootID() != test_status_fb) sr_test = false;
  if(my_settings.GetShowDeadRoots() != test_status_tb) sr_test = false;
  if(my_settings.GetShowDiameters() != test_status_tb ) sr_test = false;
  if(my_settings.GetAutoDetectDiameterCheck() != test_status_t) sr_test = false;
  if(my_settings.GetAutoDetectLengthCheck() != test_status_f ) sr_test = false;
  if(my_settings.GetAutoDetectVerticalCheck() != test_status_t ) sr_test = false;
  if(my_settings.GetAutoDetectDiameterValue() != di_value ) sr_test = false;
  if(my_settings.GetAutoDetectLengthValue() != length_val ) sr_test = false;
  if(my_settings.GetAutoDetectDegreeValue() != degree_val ) sr_test = false;
  if(my_settings.GetRootDrawingColorLive() != srdc_l ) sr_test = false;
  if(my_settings.GetRootDrawingColorLiveSelected() != srdc_ls ) sr_test = false;
  if(my_settings.GetRootDrawingColorDead() != srdc_d ) sr_test = false;
  if(my_settings.GetRootDrawingColorDeadSelected() != srdc_ds ) sr_test = false;
  if(my_settings.GetLastWinLocation() != test_rect) sr_test = false;
  
  // not sure what the purpose of these lines is, but they're not working -- STB 4/29/10
//  const std::vector<CString>& cs = my_settings.GetConditionSet();
//  for (int i=0; i<cs.size(); i++)
//  {
//    if(v_test_cv[i] != cs[i])
//      sr_test=false;
//  }
  
  if(sr_test!=true)
    OutputDebugString("ERROR:  Save- or Read- UserPreferences unsuccessful.\n");
  
  
  OutputDebugString("done.\n");
		
  
}

// returns true if files are bytewise identical, false otherwise
bool CompareFiles(const char* fname1, const char* fname2)
{
  FILE* fp1 = NULL;
  FILE* fp2 = NULL;
  bool done = false;
  
  fp1 = fopen(fname1, "rb");
  fp2 = fopen(fname2, "rb");
  
  if(!fp1 || !fp2) return false;
  
  while (!done)
  {
    char p1, p2;
    fread(&p1, sizeof(char), 1, fp1);
    fread(&p2, sizeof(char), 1, fp2);
    if (p1 != p2)  return false;
    done = feof(fp1) || feof(fp2);
    
  }
  return (feof(fp1) && feof(fp2));
}

/*
*** Test Load() and Save() functions for the following classes:
*** Tube, Experiment, RootInstance, & Root.
*** Also tests OldLoad() in Experiment Class.
*** Each Class will have 3 corresponding tests: 
***		Load, Save, and a verifying load/save combination
*/
void TestLoadAndSave()
{
  OutputDebugString("Testing Load/Save...");
  Tube test_tube;
  Experiment test_experiment;
  Settings *test_settings = new Settings();
  RootInstance test_ri;
  Root test_root;
  
  FILE *test_fp=NULL;
  
  srand(1);
  char file_data[1024];
  for(int i=0; i<1024; i++){
    file_data[i] = (char) rand();
  }
  
  /*-------------Tests for Tube class*/
  
  Tube standard_tube;
  test_tube.SetTubeTag("Test Tube 01");
  standard_tube.SetTubeTag("Test Tube 01");

  FILE * tube_fp= fopen("test/tubeStandard", "wt+");
  standard_tube.Save(tube_fp);
  fclose(tube_fp);

  //null tests


  if(test_tube.Load(test_fp) != -1){
    printf("Error:  Tube::Load null test!\n");
  }
  
  if(test_tube.Save(test_fp) != -1){
    printf("Error:  Tube::Save null test!\n");
  }
  
  if( standard_tube != test_tube){
    printf("Error:  Tubes are not equal!\n");
  }
  
  //combo tests


  test_tube.SetTubeTag("Test Tube 01");
  remove("test/loadSaveTestTube");      
  if( !( test_fp = fopen("test/loadSaveTestTube", "wt+") ) ) {
    printf("Error: File I/O Error\n");
    
  }

  if( test_tube.Save(test_fp)==-1){//test save to empty file
    printf("Error:  Tube::Save\n");
  }

  fclose(test_fp);

  if( !CompareFiles("test/tubeStandard", "test/loadSaveTestTube") ){
    printf("Error:  Tube files not equal\n");
  }

     
 
  if( standard_tube != test_tube){
    printf("Error:  Tubes are not equal!\n");
  }
  
  test_fp=NULL;
  
  
  /*-------------Tests for Experiment class*/
  /*Note: second parameter of Load and Save seem to be just used for g_debug_logfile.Print() statements
  and may not be necessary*/
  /*Question: why does this class require a filename as a first parameter instead of a file pointer? 
  Perhaps this is the highest-level function used for file i/o*/
  
  //	printf("Testing Experiment::Load/Save\n");
  //null tests
  
  remove("test/loadSaveTestExperiment");
  if(	test_experiment.Load("test/loadSaveTestExperiment", test_settings)==0 ){
    printf("Error:  Experiment::Load null test!\n");
  }
  
  if ( test_experiment.Save("test/loadSaveTestExperiment", test_settings)==-1 ){
    printf("Error:  Experiment::Save null test!\n");
  }
  
  //combo tests
  if( !( test_fp = fopen("test/loadSaveTestExperiment", "wt+") ) ) {
    printf("Error: File I/O Error\n");
    return;
  }	
  
  
  //fill file with 1K of random data
  fwrite(file_data,1, sizeof(file_data)*sizeof(char), test_fp);
  fclose(test_fp);
  test_fp = NULL;

  if(	test_experiment.Load("test/loadSaveTestExperiment", test_settings)==0 ){//test load from randomly filled file - should produce error
    printf("Error:  Experiment::Load rand test!\n");
  }
  
  if ( test_experiment.Save("test/loadSaveTestExperiment", test_settings)==-1 ){//test save to randomly filled file
    printf("Error:  Experiment::Save rand test!\n");
  }
  if(	test_experiment.Load("test/loadSaveTestExperiment", test_settings)==0 ){//verify data saved correctly
    printf("Error:  Experiment::Load rand test!\n");
  }
  test_experiment.Save("test/experimentStandard",test_settings);
  
  if( !CompareFiles("test/experimentStandard", "test/loadSaveTestExperiment") ){
    printf("Error:  Experiment files not equal\n");
  }
  
  // system("cmp file1 file2");
  
  remove("test/loadSaveTestExperiment");
  remove("test/experimentStandard");
  
  OutputDebugString("done\n");
}

void TestCurrentImageProperties()
{
  OutputDebugString("Testing CurrentImageProperties...");

  struct CurrentImageProperties test;
  
  // !!! asserts are not sufficient; we need to be able to test in Release mode as well
  assert( test.m_experiment_name.IsEmpty() );
  assert( test.m_current_image_filename.IsEmpty() );
  assert( test.m_session_date.IsEmpty() );
  assert( test.m_operator_initials.IsEmpty() );
  assert( test.m_tube.IsEmpty() );
  assert( test.m_window == -1 );
  assert( test.m_session == -1 );
  OutputDebugString("done\n");  
}



void TestSessionCollapser()
{
  int remapped_index=2;
  
  SessionCollapser test_collapser;
  
  Experiment test_experiment;//creating test experiment
  
  ApplicationData test_app_data;//creating test application data
  
  test_collapser.Init(&test_experiment,&test_app_data);//initializing members m_exp and m_app
  
  //remapped_index=test_collapser.GetRemappedIndex(3);
  //@@@this test throws an assertion failure because unmapped_index>2
  
  //remapped_index=test_collapser.GetRemappedIndex(-3);
  //@@@this test throws an assertion failure because unmapped_index<-2
  
  
  if(remapped_index!=test_collapser.GetRemappedIndex(remapped_index))
  {
    assert(0);
  }
  else
  {
    printf("The remapped index is %d\n",remapped_index);
  }
  
  //test_collapser.RemapSessionIndices();//testing with g_collapse_sessions=false
  
  g_collapse_sessions=true;//changing the value of g_collapse_session to true
  
  test_collapser.RemapSessionIndices();//testing with g_collapse_sessions=true
  //@@code crashes at if(tube->GetImageExists(window, j)) which calls 
  //@@Tube::GetImageExists(int window, int session) const and the line where it crashes
  //@@for(int i = 0; i < m_image_exists.size(); i++)
}


//using namespace std;

//Test root instance load and save
//This functions checks save-load and save-load-save
void TestRootInstanceLoadSave()
{
  OutputDebugString("Testing TestRootInstanceLoadSave...");
  enum {TEST_POINT_SIZE = 23, TEST_DIAMETER_SIZE=37};
  const CString test_name[2] = {"test/testrils1.txt", "test/testrils2.txt"};
  time_t current_time = time(NULL);
  cout << "seed time: " << current_time << endl;
  srand(current_time);
  RootInstance ri[2];
  FILE *fp;
  
  ri[0].SetSessionNumber(67);
  CString code = "qwerty";
  ri[0].SetConditionCode(code);
  ri[0].SetColor(RootInstance::COLOR_DARKBROWN);
  int i;
  for(i = 0; i < TEST_POINT_SIZE; i++)
  {
    ri[0].CreatePoint();
    ri[0].MovePoint(i,TEST_POINT_SIZE - i, i);
    ri[0].ChangeWindowOfPoint(i, rand()%(4*TEST_POINT_SIZE));
  }
  for(i = 0; i < TEST_DIAMETER_SIZE; i++)
  {
    ri[0].CreateDiameter();
    ri[0].MoveDiameter(i, TEST_DIAMETER_SIZE - i, i);
    ri[0].ResizeDiameter(i,rand()/RAND_MAX);
    ri[0].ChangeWindowOfDiameter(i,rand()%(4*TEST_DIAMETER_SIZE));
  }
  
  fp = fopen(test_name[0], "w");
  if(!fp)return;
  ri[0].Save(fp);
  fclose(fp);
  fp = fopen(test_name[0], "r");
  if(!fp)return;
  ri[1].Load(fp);
  fclose(fp);
  if(ri[0] != ri[1])
    OutputDebugString("The Root Instances are not equal");
  assert(ri[0] == ri[1]);
  fp = fopen(test_name[1], "w");
  if(!fp)return;
  ri[1].Save(fp);
  fclose(fp);
  //external test: diff test_name[0] and test_name[1]
  OutputDebugString("done\n");
}

//this function tests save-load and save-load-save
void TestRootLoadSave()
{
  OutputDebugString("Testing TestRootLoadSave...");
  enum { TEST_SESSION_SIZE = 43 };
  const CString test_name[2] = {"test/testrls1.txt", "test/testrls2.txt"};
  const CString code = "abcdef";
  Root R(4);
  Root B;
  RootInstance *ri;
  int i;
  FILE *fp;
  R.SetPrimaryWindow(9);
  R.SetDisplayableID(7); //fails if not run
  
  for(i = 0; i < TEST_SESSION_SIZE; i++)
  {
    ri = R.CreateRootInstance(i);
    ri->SetConditionCode(code);
  }
  
  fp = fopen(test_name[0], "w");
  if(!fp)return;
  R.Save(fp);
  
  fclose(fp);
  fp = fopen(test_name[0], "r");
  if(!fp)return;
  B.Load(fp);
  
  fclose(fp);
  if(R != B)
    OutputDebugString("The Roots are not equal");
  assert(R == B);
  fp = fopen(test_name[1], "w");
  if(!fp)return;
  B.Save(fp);
  fclose(fp);
  //external test: diff test_name[0] and test_name[1]
  OutputDebugString("done\n");
}

void CurrentImageProperties()
{  
  const CString current_image_filename = "test.jpg";
  const CString session_date = "Jan 24 2010";
  const CString operator_initials = "MB";
  const CString tube = "TubeString";
  const int windownum = 4;
  const int sessionnum = 3;
  
  struct CurrentImageProperties test_instance;
  test_instance.m_current_image_filename = current_image_filename;
  test_instance.m_session_date = session_date;
  test_instance.m_operator_initials = operator_initials;
  test_instance.m_tube = tube;
  test_instance.m_window = windownum;
  test_instance.m_session = sessionnum;
  
  if(test_instance.m_window != -1)
  {
    printf("Error in CurrentImageProperites: reading initial values");
  }
  if(test_instance.m_session != -1)
  {
    printf("Error in CurrentImageProperites: reading initial values");
  }
  
  
  
  if(test_instance.m_current_image_filename != current_image_filename)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }
  if(test_instance.m_session_date != session_date)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }
  if(test_instance.m_operator_initials != operator_initials)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }
  if(test_instance.m_tube != tube)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }
  if(test_instance.m_window != windownum)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }
  if(test_instance.m_session != sessionnum)
  {
    printf("Error in CurrentImageProperties: reading back Strings set");
  }  
}

};// namespace RootflyTester

