/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootflyDlg.cpp : implementation file
//
#include "stdafx.h"
#include "Rootfly.h"
#include "RootflyDlg.h" 
#include "DebugDlg.h" 
#include "AnnotateDlg.h"
#include "Image.h"		  
#include <math.h>
#include <vector>  
#include <iterator>
#include <string> 
#include <sstream> 
#include <direct.h>
#include <afxext.h>    
#include "ImageFileFormatDlg.h"            // file format dialog
#include "MMConversionDlg.h"          // calibration dialog
#include "ManageConditionCodesDlg.h"  // dialog for editing condition codes
#include "CopySessionDlg.h"           // copy session dialog
#include "ColorDlg.h"              // color dialog
#include "FolderDlg.h"             // file browse dialog
#include "ProgressDlg.h"
#include "AutoDetectDlg.h"
#include "RootDetect.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DEFAULT_FILENAME "new.rfy" 

//#define USE_AUTOSAVE  // commented due to bug; see comment in OnTimer

using namespace::std;

DebugLogFile g_debug_logfile;
bool g_collapse_sessions = false; //whether to collapse session view or not
bool g_write_to_debug_log = true;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About
/**@class CAboutDlg
@ brief Bring up the version and copyright information about Rootfly 
*/
class CAboutDlg : public CDialog
{
  
public:
  
  CAboutDlg();
  
  // Dialog Data
  //{{AFX_DATA(CAboutDlg)
  enum { IDD = IDD_ABOUTBOX };
  //}}AFX_DATA
  
  // ClassWizard generated virtual function overrides
  //{{AFX_VIRTUAL(CAboutDlg)
protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
  //}}AFX_VIRTUAL
  
  // Implementation
protected:
  //{{AFX_MSG(CAboutDlg)
  //}}AFX_MSG
  DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
  //{{AFX_DATA_INIT(CAboutDlg)
  //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  
  //{{AFX_DATA_MAP(CAboutDlg)
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
//{{AFX_MSG_MAP(CAboutDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRootflyDlg dialog
CRootflyDlg::CRootflyDlg(CWnd* pParent /*=NULL*/)
: CDialog(CRootflyDlg::IDD, pParent)
{
  m_controller.C_ReadUserPreferences();
  
  m_rfy_data = m_controller.GetRootflyData();
  //	m_set_data = m_rfy_data->GetSettings();
  //	m_exp_data = m_rfy_data->GetExperiment();
  //	m_app_data = m_rfy_data->GetApplicationData();
  //	m_graphics.Initialize( m_converter, *m_set_data, *m_rfy_data );
  m_graphics.Initialize( m_converter, m_rfy_data->GetSettings(), *m_rfy_data );
  m_shiftx = 0;
  m_shifty = 0;
  m_zoomin = 1;
  
  //{{AFX_DATA_INIT(CRootflyDlg)
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  
  
  
}

CRootflyDlg::~CRootflyDlg()
{
}

void CRootflyDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CRootflyDlg)	
  //	DDX_Control(pDX, IDC_BUTTON_ZoomIn, m_button1);
  DDX_Control(pDX, IDC_CHECK_Zoom_In, m_check_zoom_in);
  DDX_Control(pDX, IDC_CHECK_Zoom_Out, m_check_zoom_out);
  DDX_Control(pDX, IDC_BUTTON_ZoomOut, m_button_zoomout);
  DDX_Control(pDX, IDC_BUTTON_ZoomIn, m_button_zoomin);
  DDX_Control(pDX, IDC_BUTTON_DOWNZOOM, m_button_downzoom);
  DDX_Control(pDX, IDC_BUTTON_LEFTZOOM, m_button_leftzoom);
  DDX_Control(pDX, IDC_BUTTON_RIGHTZOOM, m_button_rightzoom);
  DDX_Control(pDX, IDC_BUTTON_UPZOOM, m_button_upzoom);
  //	DDX_Control(pDX, IDC_BUTTON_Auto_Detect, m_button_auto_detect);
  DDX_Control(pDX, IDC_BUTTON_Annotate, m_button_annotate);
  DDX_Control(pDX, IDC_STATIC_status, m_status_text);
  DDX_Control(pDX, IDC_BUTTON_Move_All_Roots, m_button_move_all_roots);
  DDX_Control(pDX, IDC_COMBO_COLOR, m_combo_color);
  DDX_Control(pDX, IDC_STATIC_MM, m_mm_text);
  DDX_Control(pDX, IDC_SESSION_DATE_5, m_session_date5);
  DDX_Control(pDX, IDC_SESSION_DATE_4, m_session_date4);
  DDX_Control(pDX, IDC_SESSION_DATE_3, m_session_date3);
  DDX_Control(pDX, IDC_SESSION_DATE_2, m_session_date2);
  DDX_Control(pDX, IDC_SESSION_DATE_1, m_session_date1);
  DDX_Control(pDX, IDC_STATIC_Root_ID, m_root_id_text); 
  DDX_Control(pDX, IDC_BUTTON_Delete_Root, m_button_delete_root);
  DDX_Control(pDX, IDC_BUTTON_Copy_Session, m_button_copy);
  DDX_Control(pDX, IDC_BUTTON_Extend_Endpoints, m_button_extend_endpoints);
  DDX_Control(pDX, IDC_BUTTON_New_Root, m_button_new_root);
  DDX_Control(pDX, IDC_STATIC_Color, m_color_text);
  DDX_Control(pDX, IDC_STATIC_Length, m_length_text);
  DDX_Control(pDX, IDC_STATIC_Session, m_session_text);
  DDX_Control(pDX, IDC_STATIC_Custom, m_custom_text);
  DDX_Control(pDX, IDC_STATIC_Diameter, m_diameter_text);
  DDX_Control(pDX, IDC_STATIC_State, m_state_text);
  DDX_Control(pDX, IDC_STATIC_Tube, m_tube_text);
  DDX_Control(pDX, IDC_STATIC_Window, m_window_text);
  DDX_Control(pDX, IDC_STATIC_Properties_Section, m_properties_section);
  DDX_Control(pDX, IDC_STATIC_Session_Section, m_session_section);
  DDX_Control(pDX, IDC_STATIC_Images_Section, m_images_section);
  DDX_Control(pDX, IDC_STATIC_Window_Section, m_window_section); 
  DDX_Control(pDX, IDC_CHECK_Show_Root_IDs, m_checkbox_show_root_ids);
  DDX_Control(pDX, IDC_CHECK_Show_Diameters, m_checkbox_show_diameters);
  DDX_Control(pDX, IDC_CHECK_Show_Dead_Roots, m_checkbox_show_dead_roots);
  DDX_Control(pDX, IDC_CHECK_Show_Roots, m_checkbox_show_roots);
  DDX_Control(pDX, IDC_SCROLLBAR3, m_scrollbar_horizontal);
  DDX_Control(pDX, IDC_SCROLLBAR2, m_scrollbar_vertical);
  DDX_Control(pDX, IDC_WIND_PIC_1, m_Window_Pic_1);
  DDX_Control(pDX, IDC_WIND_PIC_2, m_Window_Pic_2);
  DDX_Control(pDX, IDC_WIND_PIC_3, m_Window_Pic_3);
  DDX_Control(pDX, IDC_WIND_PIC_4, m_Window_Pic_4);
  DDX_Control(pDX, IDC_WIND_PIC_5, m_Window_Pic_5);
  DDX_Control(pDX, IDC_SESS_PIC_1, m_Session_Pic_1);
  DDX_Control(pDX, IDC_SESS_PIC_2, m_Session_Pic_2);
  DDX_Control(pDX, IDC_SESS_PIC_3, m_Session_Pic_3);
  DDX_Control(pDX, IDC_SESS_PIC_4, m_Session_Pic_4);
  DDX_Control(pDX, IDC_SESS_PIC_5, m_Session_Pic_5);
  DDX_Control(pDX, IDC_EDIT_TUBE, m_edit_tube);
  DDX_Control(pDX, IDC_EDIT_WINDOW, m_edit_window);
  DDX_Control(pDX, IDC_SPIN_TUBE, m_spin_tube);
  DDX_Control(pDX, IDC_SPIN_WINDOW, m_spin_window);
  DDX_Control(pDX, IDC_SPIN_SESSION, m_spin_session);
  DDX_Control(pDX, IDC_SPIN_DIAMETER, m_spin_diameter);
  DDX_Control(pDX, IDC_EDIT_ROOTID, m_edit_rootid);	
  DDX_Control(pDX, IDC_EDIT_DIAMETER, m_edit_diameter);
  DDX_Control(pDX, IDC_EDIT_LENGTH, m_edit_length);
  DDX_Control(pDX, IDC_COMBO_STATE, m_combo_state);
  DDX_Control(pDX, IDC_COMBO_CUSTOM, m_combo_custom);
  DDX_Control(pDX, IDC_EDIT_SESSION, m_edit_session); 
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRootflyDlg, CDialog)
//{{AFX_MSG_MAP(CRootflyDlg)
ON_WM_SYSCOMMAND()
ON_WM_PAINT()
ON_WM_QUERYDRAGICON()
ON_BN_CLICKED(IDC_BUTTON_New_Root, OnBUTTONNewRoot)
ON_BN_CLICKED(IDC_BUTTON_Extend_Endpoints, OnBUTTONExtendEndpoints)
ON_BN_CLICKED(IDC_BUTTON_Delete_Root, OnBUTTONDeleteRoot)
ON_COMMAND(ID_FILE_EXIT, OnFileExit)
ON_COMMAND(ID_FILE_NEW, OnFileNew)
ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
ON_COMMAND(ID_FILE_SAVEAS, OnFileSaveAs)
ON_COMMAND(ID_FILE_SAVE, OnFileSave)
ON_COMMAND(ID_EDIT_IMAGEFILEFORMAT, OnEditImageFileFormat)
ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_LBUTTONDBLCLK()
ON_WM_RBUTTONDOWN()
ON_WM_RBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_SIZE()
ON_COMMAND(ID_HELP_CONTENTS, OnHelpContents)
ON_EN_KILLFOCUS(IDC_EDIT_DIAMETER, OnKillfocusEditDiameter)
ON_EN_KILLFOCUS(IDC_EDIT_SESSION, OnKillfocusEditSession)
ON_EN_KILLFOCUS(IDC_EDIT_WINDOW, OnKillfocusEditWindow)
ON_EN_KILLFOCUS(IDC_EDIT_TUBE, OnKillfocusEditTube)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_TUBE, OnDeltaposSpinTube)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_SESSION, OnDeltaposSpinSession)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_WINDOW, OnDeltaposSpinWindow)	
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DIAMETER, OnDeltaposSpinDiameter)
ON_BN_CLICKED(IDC_CHECK_Show_Root_IDs, OnCHECKShowRootIDs)
ON_BN_CLICKED(IDC_CHECK_Show_Dead_Roots, OnCHECKShowDeadRoots)
ON_BN_CLICKED(IDC_CHECK_Show_Roots, OnCHECKShowRoots)
ON_BN_CLICKED(IDC_CHECK_Show_Diameters, OnCHECKShowDiameters)
ON_WM_VSCROLL()
ON_WM_HSCROLL()
ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
ON_WM_CLOSE()
ON_COMMAND(ID_EDIT_CONVERSION, OnEditCalibrate)
ON_CBN_SELCHANGE(IDC_COMBO_COLOR, OnSelchangeComboColor)
ON_CBN_SELCHANGE(IDC_COMBO_CUSTOM, OnSelchangeComboCustom)
ON_CBN_SELCHANGE(IDC_COMBO_STATE, OnSelchangeComboState)
ON_BN_CLICKED(IDC_BUTTON_Move_All_Roots, OnButtonMoveAllRoots)
ON_CBN_DROPDOWN(IDC_COMBO_CUSTOM, OnDropdownComboCustom)
ON_CBN_KILLFOCUS(IDC_COMBO_CUSTOM, OnKillfocusComboCustom)
ON_COMMAND(ID_EDIT_CONDITION, OnCondition)
ON_COMMAND(ID_EDIT_CHANGE_IMG_DIR, OnEditChangeImgDir)
ON_COMMAND(ID_EDIT_COLORS, OnEditColors)
ON_COMMAND(ID_EDIT_SESSION_DATES, OnEditSessionDates)
ON_COMMAND(ID_MENUCONTEXT_DELETEPOINT, OnDeletePoint)
ON_COMMAND(ID_MENUCONTEXT_ADDPOINT, OnAddPoint)
ON_WM_MOUSEWHEEL()
ON_WM_HELPINFO()
ON_COMMAND(ID_DEBUG_RESETMODE, OnDebugResetmode)
ON_COMMAND(ID_DEBUG_DISPLAYROOTDATA, OnDebugDisplayrootdata)
ON_COMMAND(ID_DEBUG_REPAIRDIAMETER, OnDebugRepairdiameter)
ON_COMMAND(ID_DEBUG_SHOWDEBUGLOGFILELOCATION, OnDebugShowdebuglogfilelocation)
ON_COMMAND(ID_NAVIGATE_PROGRESSWINDOW, OnNavigateProgresswindow)
ON_COMMAND(ID_EDIT_ZOOM_OUT, OnEditZoomOut)
ON_COMMAND(ID_EDIT_ZOOM_IN, OnEditZoomIn)
ON_UPDATE_COMMAND_UI(ID_VIEW_RIGHTPANE, OnUpdateViewRightpane)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_AUTODETECT, OnEditAutodetect)
ON_UPDATE_COMMAND_UI(ID_VIEW_LEFTPANE, OnUpdateViewLeftpane)
ON_UPDATE_COMMAND_UI(ID_VIEW_BOTTOMPANE, OnUpdateViewBottompane)
ON_COMMAND(ID_KEYBOARD_LEFTZOOM, OnKeyboardLeftzoom)
ON_COMMAND(ID_KEYBOARD_DOWNZOOM, OnKeyboardDownzoom)
ON_COMMAND(ID_KEYBOARD_UPZOOM, OnKeyboardUpzoom)
ON_COMMAND(ID_KEYBOARD_RIGHTZOOM, OnKeyboardRightzoom)
ON_BN_CLICKED(IDC_BUTTON_UPZOOM, OnButtonUpzoom)
ON_BN_CLICKED(IDC_BUTTON_LEFTZOOM, OnButtonLeftzoom)
ON_BN_CLICKED(IDC_BUTTON_RIGHTZOOM, OnButtonRightzoom)
ON_BN_CLICKED(IDC_BUTTON_DOWNZOOM, OnButtonDownzoom)
ON_BN_CLICKED(IDC_BUTTON_ZoomIn, OnBUTTONZoomIn)
ON_BN_CLICKED(IDC_BUTTON_ZoomOut, OnBUTTONZoomOut)
ON_BN_CLICKED(IDC_BUTTON_Annotate, OnBUTTONAnnotate)
ON_COMMAND(ID_EDIT_AUTODETECTROOTS, OnEditAutodetectroots)
ON_BN_CLICKED(IDC_BUTTON_Copy_Session, OnEditCopysession)
ON_WM_TIMER()
ON_WM_MOUSEWHEEL()
ON_EN_KILLFOCUS(IDC_EDIT_DIAMETER, OnKillfocusEditDiameter)
ON_CBN_EDITCHANGE(IDC_COMBO_STATE, OnSelchangeComboState)
ON_COMMAND(ID_KEYBOARD_LEFTARROW, OnLeftArrow)
ON_COMMAND(ID_KEYBOARD_RIGHTARROW, OnRightArrow)
ON_COMMAND(ID_KEYBOARD_UPARROW, OnUpArrow)
ON_COMMAND(ID_KEYBOARD_DOWNARROW, OnDownArrow)	
ON_COMMAND(ID_EDIT_COPYSESSION, OnEditCopysession)
ON_BN_CLICKED(IDC_CHECK_Show_Roots, OnCHECKShowRoots)
ON_COMMAND(ID_MENUCONTEXT_ANNOTATE, OnBUTTONAnnotate)
ON_COMMAND(ID_MENUCONTEXT_NEWROOT, OnBUTTONNewRoot)
ON_COMMAND(ID_MENUCONTEXT_MOVEALLROOTS, OnButtonMoveAllRoots)
ON_COMMAND(ID_MENUCONTEXT_STATELIVE, OnMENUCONTEXT_SELECT_STATE_live)
ON_COMMAND(ID_MENUCONTEXT_STATEDEAD, OnMENUCONTEXT_SELECT_STATE_dead)
ON_COMMAND(ID_MENUCONTEXT_COLORWHITE, OnMENUCONTEXT_SELECT_COLOR_white)
ON_COMMAND(ID_MENUCONTEXT_COLORLIGHTBROWN, OnMENUCONTEXT_SELECT_COLOR_lightbrown)
ON_COMMAND(ID_MENUCONTEXT_COLORDARKBROWN, OnMENUCONTEXT_SELECT_COLOR_darkbrown)
ON_COMMAND(ID_MENUCONTEXT_COLORBLACK, OnMENUCONTEXT_SELECT_COLOR_black)
ON_COMMAND(ID_MENUCONTEXT_DELETEROOT, OnBUTTONDeleteRoot)
ON_COMMAND(ID_MENUCONTEXT_EXTENDENDPOINT, OnBUTTONExtendEndpoints)
ON_COMMAND(ID_MENUCONTEXT_COPY, OnEditCopysession)
ON_COMMAND(ID_MENUCONTEXT_AUTODETECTROOTS, OnEditAutodetectroots)
ON_WM_GETMINMAXINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRootflyDlg message handlers

void CRootflyDlg::ClearSelection()
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  m_selected_index = -1;
  m_over_root = false;
}



BOOL CRootflyDlg::OnInitDialog()
{
  CDialog::OnInitDialog();
  // IDM_ABOUTBOX must be in the system command range.
  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);
  //DialogGUIstate = 0;  //Dialog size is restored size
  
  CMenu* pSysMenu = GetSystemMenu(FALSE);
  if (pSysMenu != NULL)
  {
    CString strAboutMenu;
    strAboutMenu.LoadString(IDS_ABOUTBOX);
    if (!strAboutMenu.IsEmpty())
    {
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }
  }
  
  SetIcon(m_hIcon, TRUE);			// Set big icon
  SetIcon(m_hIcon, FALSE);		// Set small icon
  
  m_button_zoomin.LoadBitmaps(IDB_ZOOMIN, IDB_ZOOMIN_DN, IDB_ZOOMIN, IDB_ZOOMIN_DIS);
  m_button_zoomout.LoadBitmaps(IDB_ZOOMOUT, IDB_ZOOMOUT_DN, IDB_ZOOMOUT, IDB_ZOOMOUT_DIS);
  m_button_upzoom.LoadBitmaps(IDB_UP, IDB_UP_DN, IDB_UP, IDB_UP_DIS);
  m_button_rightzoom.LoadBitmaps(IDB_RIGHT, IDB_RIGHT_DN, IDB_RIGHT, IDB_RIGHT_DIS);
  m_button_leftzoom.LoadBitmaps(IDB_LEFT, IDB_LEFT_DN, IDB_LEFT, IDB_LEFT_DIS);
  m_button_downzoom.LoadBitmaps(IDB_DOWN, IDB_DOWN_DN, IDB_DOWN, IDB_DOWN_DIS);
  
		
  //STATUSBAR
  Status_bar.Create(this,WS_CHILD | WS_VISIBLE | CBRS_BOTTOM);	
  Status_bar.SetIndicators(StatusBarIndicators,Status_bar.numofindicators);
  Status_bar.GetStatusBarCtrl().SetBkColor(RGB(180,180,180));
  //  Status_bar.GetStatusBarCtrl().SetBkColor(RGB(212,208,200));  // same color as dialog
  //  Status_bar.GetStatusBarCtrl().SetBkColor(RGB(230,230,230));
  
  //CDialogBar not functional
#if 0
  if (!m_test_bar.Create(this, IDD_TESTBAR,
    CBRS_TOP | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY,
    IDD_TESTBAR))
  {
    TRACE0("Failed to create dialog bar m_test_bar\n");
    return -1;		// fail to create
  }
  
  m_test_bar.EnableDocking(CBRS_ALIGN_TOP | CBRS_ALIGN_BOTTOM);
  //  EnableDocking(CBRS_ALIGN_ANY);
  //  DockControlBar(&m_test_bar);
  m_test_bar.MoveWindow(30,30,100,100);
#endif
  
  m_controller.Attach(this);    
  SetMode(MODE_NOFILELOADED);  
  
  
  // These are all of the cursors used in Rootfly
  m_cur_openhand   = AfxGetApp()->LoadCursor(IDC_CUR_OPENHAND);
  m_cur_closedhand = AfxGetApp()->LoadCursor(IDC_CUR_CLOSEDHAND);
  //	m_cur_pen        = AfxGetApp()->LoadCursor(IDC_CUR_PEN);
  m_cur_pen        = AfxGetApp()->LoadStandardCursor(IDC_CROSS);
  m_cur_uparrow    = AfxGetApp()->LoadStandardCursor(IDC_UPARROW);
  m_cur_sizeall    = AfxGetApp()->LoadStandardCursor(IDC_SIZEALL);
  m_cur_sizens     = AfxGetApp()->LoadStandardCursor(IDC_SIZENS);
  m_cur_movepoint   = AfxGetApp()->LoadStandardCursor(IDC_SIZENWSE);
  
  // No roots are selected by default
  ClearSelection();
  
  CRect section_rect;  
  m_images_section.GetWindowRect(&section_rect); 
  ScreenToClient(section_rect);  
  
  // sets up the initial custom box
  std::vector<CString> condition_set;
  condition_set = m_rfy_data->GetSettings().GetConditionSet();
  // if size is zero then there are no codes to load
  // probably first time using program
  if (condition_set.size() == 0)
  {
    InitializeConditionSet();
  }
  else
  {
    for (int i = 0; i < condition_set.size(); i++)
    {
      m_combo_custom.AddString(condition_set.at(i));
    }
  }
  m_combo_custom.SetCurSel(m_rfy_data->GetSettings().GetConditionDefaultIndex());
  
  // read from data structure and set check based off bool
  // if entry not present in registry, is set to on by default
  m_checkbox_show_roots.SetCheck(m_rfy_data->GetSettings().GetShowRoots());
  m_checkbox_show_root_ids.SetCheck(m_rfy_data->GetSettings().GetShowRootID());
  m_checkbox_show_dead_roots.SetCheck(m_rfy_data->GetSettings().GetShowDeadRoots());
  m_checkbox_show_diameters.SetCheck(m_rfy_data->GetSettings().GetShowDiameters());
  
  // ---------------------places toolbar in dialog------------------ //
  {
  /*if (!m_wndToolBar.Create(this)
  || !m_wndToolBar.LoadToolBar(IDR_TOOLBAR1))
  {
  TRACE0("Failed to Create Dialog Toolbar\n");
  EndDialog(IDCANCEL);
  }*/
    
    CRect	rcClientOld; 
    CRect	rcClientNew; 
    GetClientRect(rcClientOld); 
    RepositionBars(AFX_IDW_CONTROLBAR_FIRST,
      AFX_IDW_CONTROLBAR_LAST,0,reposQuery,rcClientNew);
    CPoint ptOffset(rcClientNew.left-rcClientOld.left,
      rcClientNew.top-rcClientOld.top);
    CRect	rcChild;
    CWnd*	pwndChild = GetWindow(GW_CHILD);
    while (pwndChild) 
    {
      pwndChild->GetWindowRect(rcChild); 
      ScreenToClient(rcChild);
      rcChild.OffsetRect(ptOffset);
      pwndChild->MoveWindow(rcChild,FALSE); 
      pwndChild = pwndChild->GetNextWindow();
    }
    CRect	rcWindow;
    GetWindowRect(rcWindow);
    rcWindow.right += rcClientOld.Width() - rcClientNew.Width();
    rcWindow.bottom += rcClientOld.Height() - rcClientNew.Height();
    RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST,0);
  }
  // ---------------------end of toolbar placement------------------ //

  // set initial zoom to 1
  m_check_zoom_in.SetCheck(BST_CHECKED);
  
  CheckForCommandLine();  // Check For Command Line Arguments
  
  m_controller.C_SetModify(false);
  
  if (g_write_to_debug_log) g_debug_logfile.Open("rootfly_logfile.txt");
  
#ifdef USE_AUTOSAVE
  //start timer for autosave
  SetTimer(0, 60000, 0);
  
#endif
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

void CRootflyDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
  if ((nID & 0xFFF0) == IDM_ABOUTBOX)
  {
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  }
  else
  {
    CDialog::OnSysCommand(nID, lParam);  
  }
}

/// DRAWING RELATED METHODS
void CRootflyDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this);     
    SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
    
    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;
    
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {   
    CDialog::OnPaint();
    ReDraw(1, 1);
  }
}

/*ON_INIT,ROOT_ADDED,ROOT_DELETED,ROOT_RESIZED,POINT_ADDED,POINT_REMOVED,POINT_CHANGED,
DIAMETER_ADDED,DIAMETER_REMOVED,DIAMETER_CHANGED,SESSION_CHANGED,TUBE_CHANGED,
EXPERIMENT_CHANGED,CURRENT_IMAGE_CHANGED,SESSION_DATE_CHANGED,OPERATOR_CHANGED,SAVE_STATUS_SAVED,
SAVE_STATUS_UNSAVED,FILENAME_CHANGED,IMAGEDIR_CHANGED,IMAGEFILEFORMAT_CHANGED,WINDOW_CHANGED,
COLOR_CHANGED,STATUS_CHANGED,CONDITION_CODE_ADDED,SHOW_ROOT_ID_CHANGED,SHOW_ROOTS_CHANGED,s_CHANGED, 
SHOW_DIAMETERS_CHANGED,DRAW_SETTINGS_CHANGED,TUBE_ADDED,NEW_FILE_LOADED,FILE_SAVED,
ROOT_INSTANCE_CLEAR,ROOT_CLEAR,TUBE_CLEAR,EXPERIMENT_CLEAR,PREFERENCES_READ,PREFERENCES_SAVED
*/
void CRootflyDlg::OnUpdate(Subject* theChangeSubject, int code_update) 
{   
  CString name,name2;
  char backslash = '\\';
  int index;
  
  switch(code_update)
  {
		case SAVE_STATUS_SAVED:
      name = m_rfy_data->GetApplicationData().GetFileName();
      index = name.ReverseFind(backslash);
      name2.Format("%d", index);
      if (index == -1)
      {
        SetWindowText(name);
      } 
      else
      {
        name = name.Right(name.GetLength()-index-1);
        SetWindowText("Rootfly - [" + name + "]");
      }
      break;
      
    case SAVE_STATUS_UNSAVED:
      name = m_rfy_data->GetApplicationData().GetFileName();
      index = name.ReverseFind(backslash);
      name2.Format("%d", index);
      if (index == -1)
      {
        SetWindowText("Rootfly - [" + name + " *]");
      } 
      else
      {
        name = name.Right(name.GetLength()-index-1);
        SetWindowText("Rootfly - [" + name + "*]");
      }   
      break; 
      
    case FILENAME_CHANGED:
      name = m_rfy_data->GetApplicationData().GetFileName();
      index = name.ReverseFind(backslash);
      name2.Format("%d", index);
      if (index == -1)
      {
        if (name != DEFAULT_FILENAME)
        {
          SetWindowText("Rootfly - [" + name + " *]");
        }
        else
        {
          SetWindowText("Rootfly - [" + name + "]");
        }
      }
      else
      {
        name = name.Right(name.GetLength()-index-1);
        if (name != DEFAULT_FILENAME)
        {
          SetWindowText("Rootfly - [" + name + " *]");
        }
        else
        {
          SetWindowText("Rootfly - [" + name + "]");
        }
      }   
      break; 
    case TUBE_CHANGED:
    case WINDOW_CHANGED:
    case SESSION_CHANGED:
      EnableDisable();  // make sure buttons are grayed out appropriately based upon 'Annotate'
      SetZoomWindows();  // !!! This seems out of place, but if you remove it, the main image won't be displayed -- STB 5/3/10
      break;
      
    default: break;
  }
  
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName( m_rfy_data->GetApplicationData().GetCurrentTubeName() );  // current tube
  int window = m_rfy_data->GetApplicationData().GetCurrentWindow(); //current window
  int session = m_rfy_data->GetApplicationData().GetCurrentSession(); // current session
  CString roots,autodetect;
  tube ? roots.Format("%d",tube->GetNumberOfRootsInWindowSession(window,session)) : "";
  autodetect = (roots != "") ? roots: "0"; 
  autodetect += (roots != "1") ? " roots in window" : " root in window"; 
  autodetect += (tube && tube->GetImageScanned(window,session)) ? " - auto-detect has been run" : " - auto-detect has not been run";
  Status_bar.SetPaneText(2,autodetect);  //STATUSBAR
}

// handles the code to display the properties about the selected root
void CRootflyDlg::UpdateProperties(int selected_root)
{
  // no root selected go back to initial conditions
  if (selected_root == -1)
  {
    m_edit_length.SetWindowText("");
    m_edit_rootid.SetWindowText("");
    m_edit_diameter.SetWindowText("");
    m_combo_color.SetWindowText("");
    m_combo_state.SetWindowText("");
    m_combo_custom.SetWindowText("");
    
    return;
  }
  
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  
  if (!tube)
  {
    return;
  }
  
  const Root *root = tube->GetRoot( selected_root );
  //assert(root);  
  if (!root)
  {
    return;
  }
  
  const RootInstance *inst = root->GetRootInstance(m_selected_root_session);
  if (!inst)
  {
    return;
  }
  
  const std::vector<RPoint> *line = inst->GetAllPoints();
  assert(line);  
  if (!line)
  {
    return;
  }
  CString tempstr;
  
  // displays the root id, checks that the current window is the same as the primary window for the RootInstance, if it is not
  // then it will show the RootInstance as "WindowID(PrimaryWindow
  if (m_rfy_data->GetApplicationData().GetCurrentWindow() != root->GetPrimaryWindow()) 
  {
    tempstr.Format("%d(%d)",root->GetDisplayableID(),root->GetPrimaryWindow());
  }
  else
  {
    tempstr.Format("%d",root->GetDisplayableID());
  }
  
  m_edit_rootid.SetWindowText(tempstr);
  
  // updates the length of the root
  double templength = 0;
  double dx, dy;
  const std::vector<RPoint> *points= inst->GetAllPoints();
  
  for (int j = 1; j < points->size(); j++)
  {
    dx = (points->at(j).x-points->at(j-1).x) / m_rfy_data->GetExperiment().GetMMConversion();
    dy = (points->at(j).y-points->at(j-1).y) / m_rfy_data->GetExperiment().GetMMConversion();
    templength += sqrt(pow(dx,2)+pow(dy,2));
  }
  tempstr.Format("%.2f mm",templength);
  m_edit_length.SetWindowText(tempstr);
  
  // displays the diameter
  const RDiameter *diam = inst->GetDiameter(0);
  double rad = diam->radius / m_rfy_data->GetExperiment().GetMMConversion();
  tempstr.Format("%.2f mm",rad*2);
  m_edit_diameter.SetWindowText(tempstr); 
  
  // displays the state of the root
  if (inst->IsAlive())
  {
    m_combo_state.SetCurSel(1);
  }
  else 
  {
    m_combo_state.SetCurSel(0);
  }
  
  // displays the root color (white, black, dark brown, or light brown)
  int color = inst->GetColor();
  assert(color >= 0 && color <= 3);
  if (color < 0 || color > 3)  
  {
    color = 0;
  }
  m_combo_color.SetCurSel( color );
  
  // custom box
  CString user_code = inst->GetConditionCode();
  // code is no longer valid reset condition back to default
  if (m_combo_custom.SelectString(-1,user_code) == CB_ERR)
  {
    CString temp;
    m_combo_custom.GetLBText(m_rfy_data->GetSettings().GetConditionDefaultIndex(), temp);
    m_controller.C_SetConditionCode(selected_root,m_selected_root_session,temp);
    m_combo_custom.SetCurSel(m_rfy_data->GetSettings().GetConditionDefaultIndex());
  }
} 

// returns the rect (in client coordinates) with associated index and type
// index 0 is the middle rect
// if type is 0 then returns a window rect
// if type is 1 then returns a session rect
CRect CRootflyDlg::GetThumbnailRect(int index,int type)
{
  CRect rect;
  if (type == 0 && GetMenu()->GetMenuState(ID_VIEW_LEFTPANE, MF_BYCOMMAND) == MF_CHECKED)
  {
    switch(index+((NUM_THUMBNAILS-1)/2))
    {
    case 0: 
      m_Window_Pic_1.GetWindowRect(&rect); 
      break;
      
    case 1: 
      m_Window_Pic_2.GetWindowRect(&rect); 
      break;
      
    case 2: 
      m_Window_Pic_3.GetWindowRect(&rect);  
      break;
      
    case 3: 
      m_Window_Pic_4.GetWindowRect(&rect);  
      break;
      
    case 4: 
      m_Window_Pic_5.GetWindowRect(&rect);  
      break;
      
    default:  
      m_Window_Pic_3.GetWindowRect(&rect);
    }
  } 
  else if (type == 1 && GetMenu()->GetMenuState(ID_VIEW_BOTTOMPANE, MF_BYCOMMAND) == MF_CHECKED)
  {
    switch(index+((NUM_THUMBNAILS-1)/2))
    {
    case 0: 
      m_Session_Pic_1.GetWindowRect(&rect); 
      break;
      
    case 1: 
      m_Session_Pic_2.GetWindowRect(&rect); 
      break;
      
    case 2: 
      m_Session_Pic_3.GetWindowRect(&rect); 
      break;
      
    case 3: 
      m_Session_Pic_4.GetWindowRect(&rect); 
      break;
      
    case 4: 
      m_Session_Pic_5.GetWindowRect(&rect); 
      break;
      
    default:  
      m_Session_Pic_3.GetWindowRect(&rect);
    }
  }
  ScreenToClient(rect);
  return rect;
}

// draws the images in the main window
bool CRootflyDlg::DrawImage(blepo::ImgBgr img, CRect rect, int window_offset, int session_offset, CDC& dc) const
{
  //  ImageInfo current_image;
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  CString temp_str;
  
  // Session collapser
  SessionCollapser sess_coll = m_controller.GetRootflyData()->GetSessionCollapser();
  int coll_session_offset = sess_coll.GetRemappedIndex( session_offset );
  
  // checks to see if file is valid
  // draws image if valid, displays message if image is unavailable
  temp_str = iff.GetImageFileName(m_rfy_data->GetExperiment().GetImageDir(),m_rfy_data->GetApplicationData().GetCurrentTubeName(),
    m_rfy_data->GetApplicationData().GetCurrentWindow()+window_offset,m_rfy_data->GetApplicationData().GetCurrentSession()+coll_session_offset);
  if (temp_str != "") 
  {
    blepo::Draw(img, dc, CRect(0, 0, img.Width(), img.Height()), rect); 
    return true;
  }
  else
  {
    CRect section_rect = GetClientRectOfImageDisplayArea();
    rect.top += rect.Height()/2;  // for some reason, DT_VCENTER is not working
    if (GetMode() != MODE_NOFILELOADED) 
    {
      dc.DrawText("No Image Available", rect, DT_CENTER);
    }
    return false;
  }
}

// displays one or three images, depending upon whether in zoomed out mode
void CRootflyDlg::UpdateMainImage(CDC& dc)
{
  if ( GetMode() == MODE_NOFILELOADED) 
  {
    return;
  }
  m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
  //	SetZoomWindows();  // needed in case image aspect ratio has changed
  
  CRect rect, rect1, rect2, rect3 ;
  blepo::ImgBgr img, img1, img3; 
  
  bool anyimageexists = 0;
  
  m_rfy_data->GetApplicationData().GetImageByOffset( 0, 0, &img );
  m_rfy_data->GetApplicationData().GetImageByOffset( -1, 0, &img1 );
  m_rfy_data->GetApplicationData().GetImageByOffset( 1, 0, &img3 );
  
  
  rect.SetRect(0,0,img.Width(),img.Height());
  {
    {
      bool null1 = (img1.Width() == 0 || img1.Height() == 0);
      bool null3 = (img3.Width() == 0 || img3.Height() == 0);
      bool samesize1 = (img.Width () == img1.Width () && img.Height() == img1.Height());
      bool samesize3 = (img.Width () == img3.Width () && img.Height() == img3.Height());
      
      if ( (!null1 && !samesize1) || (!null3 && !samesize3))
      {
        assert(0);  // if images are of different sizes, we might have problems with preserving the aspect ratio in the drawing, although it will not likely cause huge errors -- STB
      }
    }
  }
  
  
  ApplicationData temp = m_rfy_data->GetApplicationData();
  
  //Display Image(s)
  int w = m_rfy_data->GetApplicationData().GetCurrentWindow();
  rect = m_converter.GetClientRect( w );
  
  if (IsZoomOutMode()) //in 3x3 grid
  {
    for ( int i = -1; i <= 1; i++ )
    {
      for ( int j = -1; j <= 1; j++ )
      {
        rect1 = rect;
        rect1.OffsetRect( i * rect.Width(), j * rect.Height() );
        
        temp.GetImageByOffset( j, i, &img );
        
        if( DrawImage(img, rect1, j, i, dc )) anyimageexists = true;
      }
    }

    // Draw boundaries between sessions
    rect1 = rect;
    rect1.DeflateRect( 0, 0, rect1.Width(), 0 );
    rect1.InflateRect( 1, rect1.Height(), 1, rect1.Height() );
    CBrush background( RGB(0,0,255) );
    dc.FillRect(rect1, &background);
    rect1.OffsetRect( rect.Width(), 0 );
    dc.FillRect( rect1, &background );

    if(anyimageexists)
    {
      if(GetMode() == MODE_NOIMAGEDISPLAYED) SetMode(MODE_IMAGEDISPLAYED);
    }
    else SetMode(MODE_NOIMAGEDISPLAYED);
  }
  
  else //not in 3x3 grid
  {     
    if ( GetMode() == MODE_NOIMAGEDISPLAYED )
    {
      if (DrawImage(img, rect, 0, 0, dc)) SetMode(MODE_IMAGEDISPLAYED);        
    }
    else 
    {
      if (!DrawImage(img, rect, 0, 0, dc)) SetMode(MODE_NOIMAGEDISPLAYED);
    }
  }
}

void CRootflyDlg::UpdateThumbnails()
{
  blepo::ImgBgr img;
  CClientDC dc(this);
  ImageInfo current_image;
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  CRect rect;
  int index, current, max, pos;
  CString temp_str;
  CPoint corner;
  
  // used for coloring
  COLORREF COLOR_BLUE =        RGB(0,0,255);
  COLORREF COLOR_BLACK =       RGB(0,0,0);
  COLORREF COLOR_SYS_GRAY =    ::GetSysColor(COLOR_3DFACE);
  CBrush black_brush(COLOR_BLACK);
  CBrush blue_brush(COLOR_BLUE);
  CBrush background_brush(COLOR_SYS_GRAY);
  
  if (GetMode() != MODE_NOFILELOADED)
  {
    // draws window thumbnails then session thumbnails
    for (pos = 0; pos < 2; pos++)
    {
      // draws a set of five thumbnails
      for (index = -((NUM_THUMBNAILS-1)/2); index < ((NUM_THUMBNAILS+1)/2); index++)
      {
        rect = GetThumbnailRect(index,pos);
        
        // tests to see if there is a file for the thumbnail and sets the current and max varibles
        //window thumbnails
        if (pos == 0)
        {
          m_rfy_data->GetApplicationData().GetImageByOffset( index, 0, &img );
          temp_str = iff.GetImageFileName(m_rfy_data->GetExperiment().GetImageDir(), m_rfy_data->GetApplicationData().GetCurrentTubeName(),
            m_rfy_data->GetApplicationData().GetCurrentWindow()+index,m_rfy_data->GetApplicationData().GetCurrentSession());
          current = index  + m_rfy_data->GetApplicationData().GetCurrentWindow();
          max = m_rfy_data->GetExperiment().GetMaxWindows();
        } 
        //session thumbnails
        else
        {
          m_rfy_data->GetApplicationData().GetImageByOffset( 0, index, &img );
          temp_str = iff.GetImageFileName(m_rfy_data->GetExperiment().GetImageDir(), m_rfy_data->GetApplicationData().GetCurrentTubeName(),
            m_rfy_data->GetApplicationData().GetCurrentWindow(), 
            m_rfy_data->GetApplicationData().GetCurrentSession()+ m_rfy_data->GetSessionCollapser().GetRemappedIndex(index));
          current = m_rfy_data->GetSessionCollapser().GetRemappedIndex(index) + m_rfy_data->GetApplicationData().GetCurrentSession();
          max = m_rfy_data->GetExperiment().GetMaxSessions();
        }
        CRect temp(0, 0, img.Width(), img.Height());
        
        // draws the image
        if (temp_str != "")
        {
          blepo::Draw(img, dc, temp, rect);
        }
        else 
        {
          dc.FillRect(rect,&background_brush);
        }
        
        // places index of thumbnail in corner
        if ((current> 0) && (current <= max))
        {
          corner = rect.TopLeft();
          temp_str.Format("%d", current);
          dc.TextOut(corner.x+1, corner.y+1, temp_str);
        }
        
        // frames the rect
        if (index == 0)
        {
          dc.FrameRect(rect,&blue_brush);
          for(int i = 0; i < SELECTED_THUMBNAIL_BORDER_WIDTH; i++)
          {
            rect.InflateRect(1,1);
            dc.FrameRect(rect,&blue_brush);
          }
        }
        else dc.FrameRect(rect,&black_brush); 
        
      }    
    }
    UpdateSessionDateText();
  }
}

// updates the session dates under the thumbnails
void CRootflyDlg::UpdateSessionDateText()
{  
  // updates dates
  Date temp_date;
  CString temp;
  
  temp_date = m_rfy_data->GetExperiment().GetSessionDate(m_rfy_data->GetApplicationData().GetCurrentSession() +m_rfy_data->GetSessionCollapser().GetRemappedIndex(-2));            
  if (temp_date.month != -1) 
  {
    m_session_date1.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  }
  else 
  {
    m_session_date1.SetWindowText("");
  }
  
  temp_date = m_rfy_data->GetExperiment().GetSessionDate(m_rfy_data->GetApplicationData().GetCurrentSession() +m_rfy_data->GetSessionCollapser().GetRemappedIndex(-1));
  if (temp_date.month != -1) 
  {
    m_session_date2.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  }
  else 
  {
    m_session_date2.SetWindowText("");
  }
  
  temp_date = m_rfy_data->GetExperiment().GetSessionDate(m_rfy_data->GetApplicationData().GetCurrentSession());
  if (temp_date.month != -1) 
  {
    m_session_date3.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  }
  else 
  {
    m_session_date3.SetWindowText("");
  }
  
  temp_date = m_rfy_data->GetExperiment().GetSessionDate(m_rfy_data->GetApplicationData().GetCurrentSession() +m_rfy_data->GetSessionCollapser().GetRemappedIndex(+1));
  if (temp_date.month != -1) 
  {
    m_session_date4.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  }
  else 
  {
    m_session_date4.SetWindowText("");
  }
  
  temp_date = m_rfy_data->GetExperiment().GetSessionDate(m_rfy_data->GetApplicationData().GetCurrentSession() +m_rfy_data->GetSessionCollapser().GetRemappedIndex(+2));
  if (temp_date.month != -1) 
  {
    m_session_date5.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  }
  else 
  {
    m_session_date5.SetWindowText("");
  }
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRootflyDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

void CRootflyDlg::OnBUTTONNewRoot() 
{
  if ( GetMode() == MODE_NOIMAGEDISPLAYED ) 
  {
    return;
  }
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  m_current_root_being_drawn.ClearAll();
  m_current_root_being_drawn.SetSessionNumber(m_rfy_data->GetApplicationData().GetCurrentSession());
  m_current_root_being_drawn.SetAlive();
  
  SetMode(MODE_DRAWINGNEWROOT);
  
  EnableDisable();  // CWK Spring 10
  
  ReDraw(1,0);
}

/**
/ @brief: This function fills the member root with the 
/         selected root's information.  Then the mode
/         changes to MODE_EXTENDINGENDPOINTS until
/         right button is clicked to abort, or a valid 
/         point is placed on the drawing area.
*/
void CRootflyDlg::OnBUTTONExtendEndpoints() 
{
  //if ( GetMode() != MODE_ROOTSELECTED )  
  //{
  //	return; 
  //}
  
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);
  if (!tube)  
  {
    return;
  }
  
  const vector<Root> *root = tube->GetAllRoots();
  if (!root) 
  {
    return;
  }
  
  m_current_root_being_drawn.ClearAll();
  
  for ( int i = 0; i <  root->size(); i++ )
  {
    if ( root->at(i).GetID() == m_selected_root_id )
    {
      const RootInstance *inst = root->at(i).GetRootInstance(m_selected_root_session);  
      if (!inst) 
      {
        return;
      }
      if (inst->IsAlive()) 
      {
        m_current_root_being_drawn.SetAlive();
      }
      else 
      {
        m_current_root_being_drawn.SetDead();
      }
      m_current_root_being_drawn.SetSessionNumber(m_selected_root_session);
      m_current_root_being_drawn.SetConditionCode(inst->GetConditionCode());
      m_current_root_being_drawn.SetColor(inst->GetColor());
      
      const vector<RPoint> *points = inst->GetAllPoints();
      assert(points);
      if (!points)
      {
        return;
      }
      
      for ( int j = 0; j < points->size(); j++ )
      {
        RPoint *temp = m_current_root_being_drawn.CreatePoint();        
        temp->x = points->at(j).x;
        temp->y = points->at(j).y;
        temp->window = points->at(j).window;        
      }
      RDiameter *temp_diam = m_current_root_being_drawn.CreateDiameter();
      temp_diam->r_point = inst->GetDiameter(0)->r_point;
      temp_diam->radius = inst->GetDiameter(0)->radius;
    }   
  }
  SetMode(MODE_EXTENDINGENDPOINT); 
}

void CRootflyDlg::OnBUTTONDeleteRoot() 
{
  const Experiment& exp = m_rfy_data->GetExperiment();   
  const Tube *tube = exp.GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if (tube == NULL) 
  {
    return;
  }
  
  if (m_selected_root_id == -1)
  {
    // This should not happen, because 'delete root' button is disabled if no root selected
    AfxMessageBox("Please Select a Root First");
    return;
  }
  const Root *TempRoot = tube->GetRoot(m_selected_root_id);
  
  CString temp_str;
  temp_str.Format("Delete Root %d?", TempRoot->GetDisplayableID());
  
  if (AfxMessageBox(temp_str, MB_YESNO) == IDYES)
  {
    m_controller.C_DeleteRoot(m_selected_root_id);
    m_selected_root_id = -1;
    SetMode(MODE_IMAGEDISPLAYED);
    ReDraw(1,0);
  }
}

/**
* This function is called when the user clicks the Copy... button
*/
//void CRootflyDlg::OnBUTTONCopySession() 
void CRootflyDlg::OnEditCopysession() 
{
  CCopySessionDlg dlg;
  
  int curr_session = m_rfy_data->GetApplicationData().GetCurrentSession();
  int curr_window = m_rfy_data->GetApplicationData().GetCurrentWindow();  
  
  //Set the initial source and destination sessions in the copy session dialog.
  dlg.SetCurrent(curr_session);
  int foo = m_rfy_data->GetExperiment().GetMinSessions();
  if (curr_session <= m_rfy_data->GetExperiment().GetMinSessions())
  { // If this is the first session, then it's not clear what we should display
    bool ret;
    ret = dlg.SetSource(curr_session);
    assert(ret);
    ret = dlg.SetDestination(curr_session+1);
    assert(ret);
  }
  else
  { 
    bool ret;
    ret = dlg.SetSource(curr_session-1);
    assert(ret);
    ret = dlg.SetDestination(curr_session);
    assert(ret);
  }
  
  //	if (curr_session < m_rfy_data->GetExperiment().GetMaxSessions())
  //	{
  //		dlg.SetSource(curr_session);
  //		dlg.SetDestination(curr_session+1);
  //	}
  //	else
  //	{  // If this is the last session, then it's not clear what we should display
  //		dlg.SetSource(curr_session);
  //		dlg.SetDestination(curr_session);
  //	}
  
  // This old approach is not intuitive:
  //  //  If the user is on the first session, set the source to 1 and destination to 2.
  //  //  otherwise, set the destination to the current session, and the source to the session right before it.
  //  if (curr_session > 1)
  //  {
  //    dlg.SetSource(curr_session - 1);
  //    dlg.SetDestination(curr_session);
  //  }
  //  else 
  //  {
  //    dlg.SetSource(1);
  //    dlg.SetDestination(2);
  //  }
  
  int response = dlg.DoModal();
  
  //If the user selected OK in the copy session dialog...
  if (response == IDOK) 
  {
    if (dlg.GetSource() >= dlg.GetDestination()) 
    {
      AfxMessageBox("Destination session must be greater than source session.  Unable to copy.");
      return;
    }
    
    //To copy from all the tubes and all the windows
    CString tubename = m_rfy_data->GetApplicationData().GetCurrentTubeName();
    
    //NOTE:  If copying roots in ALL windows, from a given session to a given session, 
    //        pass -1 as the window parameter to C_CopySession
    CCopySessionDlg::CopyWhat copy_what = dlg.GetCopyWhat();
    if (copy_what == CCopySessionDlg::ALL_ROOTS_IN_ALL_WINDOWS_TUBES) 
    {
      // copy all tubes (this is the option displayed to user, so we had better do it)
      const vector<Tube> *tubes = m_rfy_data->GetExperiment().GetAllTubes();
      m_controller.C_CopySession(dlg.GetSource(), dlg.GetDestination(), tubename, -1, -1);
    } 
    else 
    {
      int which_root = (copy_what == CCopySessionDlg::JUST_SINGLE_ROOT) ? dlg.GetWhichRoot() : -1;
      m_controller.C_CopySession(dlg.GetSource(), dlg.GetDestination(), tubename, curr_window, which_root);
    }
    
    ReDraw(1, 0);
  }
}

/**
/
/    @action: This resizes the individual items in the main window
/     and checks the menu to see if any objects should be hidden
/
*/  
void CRootflyDlg::OnSizeNew(UINT nType, int cx, int cy)
{
  Rootfly_Left_Objects_Hide();
  Rootfly_Bottom_Objects_Hide();
  Rootfly_Right_Objects_Hide();
  
  int htop = 25;
  int hsession = 20;
  int m_sessionSectionPos = 192, m_windowSectionPos = 29-htop;
  
  
  // Calculate relative positions based on m_image_section height and width
  if (628 < (cx-369) ) {
    m_sessionSectionPos = (191 + ((cx-369)/2)) - (628/2);
  }
  if (530 < (cy-176-htop) ) {
    m_windowSectionPos = ((29-htop) + ((cy-176-htop)/2)) - (530/2);
  }
  
  //Window, Images, Session, Properties
  m_window_section.SetWindowPos(&m_window_section,19,m_windowSectionPos,167,530,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_images_section.SetWindowPos(&m_images_section,191,29-htop,cx-369,cy-176-htop,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_section.SetWindowPos(&m_session_section,m_sessionSectionPos,cy-146-htop-hsession,628,141,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_properties_section.SetWindowPos(&m_properties_section,cx-173,135-htop,163,200,SWP_NOZORDER|SWP_SHOWWINDOW);
  // m_status_text.SetWindowPos(&m_status_text,160,12,cx-160-20,20,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  //Tube, Window, Session
  m_spin_tube.SetWindowPos(&m_spin_tube,cx-36,41-htop,16,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_tube.SetWindowPos(&m_edit_tube,cx-108,43-htop,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_tube_text.SetWindowPos(&m_tube_text,cx-167,47-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_spin_window.SetWindowPos(&m_spin_window,cx-36,70-htop,16,23, SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_window.SetWindowPos(&m_edit_window,cx-108,72-htop,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_window_text.SetWindowPos(&m_window_text,cx-167,76-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_spin_session.SetWindowPos(&m_spin_session,cx-36,100-htop,16,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_session.SetWindowPos(&m_edit_session,cx-108,102-htop,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_text.SetWindowPos(&m_session_text,cx-167,103-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Properties Section
  m_root_id_text.SetWindowPos(&m_root_id_text,cx-165,158-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_rootid.SetWindowPos(&m_edit_rootid,cx-108,155-htop,88,22,SWP_NOZORDER|SWP_SHOWWINDOW);   
  m_diameter_text.SetWindowPos(&m_diameter_text,cx-165,188-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_spin_diameter.SetWindowPos(&m_spin_diameter,cx-36,183-htop,16,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_diameter.SetWindowPos(&m_edit_diameter,cx-108,185-htop,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);   
  m_length_text.SetWindowPos(&m_length_text,cx-165,217-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_length.SetWindowPos(&m_edit_length,cx-108,215-htop,88,22,SWP_NOZORDER|SWP_SHOWWINDOW); 
  m_color_text.SetWindowPos(&m_color_text,cx-165,247-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_color.SetWindowPos(&m_combo_color,cx-108,245-htop,88,80,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_state_text.SetWindowPos(&m_state_text,cx-165,277-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_state.SetWindowPos(&m_combo_state,cx-108,275-htop,88,80,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_custom_text.SetWindowPos(&m_custom_text,cx-165,307-htop,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_custom.SetWindowPos(&m_combo_custom,cx-108,305-htop,88,100,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  //Checkboxes
  m_checkbox_show_roots.SetWindowPos(&m_checkbox_show_roots,cx-167,342-htop,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_checkbox_show_dead_roots.SetWindowPos(&m_checkbox_show_dead_roots,cx-147,367-htop,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_checkbox_show_root_ids.SetWindowPos(&m_checkbox_show_root_ids,cx-147,392-htop,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
 	m_checkbox_show_diameters.SetWindowPos(&m_checkbox_show_diameters,cx-147,417-htop,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  //  GetDlgItem(IDC_CHECK_Done_Annotating)->ShowWindow (SW_SHOW);   //temp fix
  
  //Buttons
  m_button_annotate.SetWindowPos(&m_button_annotate, cx-167, 447-htop, 89, 23, SWP_NOZORDER | SWP_SHOWWINDOW);
  m_button_new_root.SetWindowPos(&m_button_new_root,cx-167,477-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_move_all_roots.SetWindowPos(&m_button_extend_endpoints,cx-167,507-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_extend_endpoints.SetWindowPos(&m_button_move_all_roots,cx-167,537-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_delete_root.SetWindowPos(&m_button_delete_root,cx-167,567-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  //	m_button_extend_endpoints.SetWindowPos(&m_button_extend_endpoints,cx-167,477-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  //	m_button_move_all_roots.SetWindowPos(&m_button_move_all_roots,cx-167,507-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  //	m_button_delete_root.SetWindowPos(&m_button_delete_root,cx-167,537-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  //	m_button_copy.SetWindowPos(&m_button_copy,cx-167,567-htop,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);    
  //m_check_zoom_out.SetWindowPos(&m_check_zoom_out,cx-167,587-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //m_check_zoom_in.SetWindowPos(&m_check_zoom_in,cx-167,627-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //  GetDlgItem(IDC_BUTTON_Auto_Detect)->ShowWindow (SW_SHOW); //Temp fix, autodetect group needs to add function for redrawing of button
  m_button_upzoom.SetWindowPos(&m_button_upzoom, cx-136,597-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  m_button_downzoom.SetWindowPos(&m_button_downzoom, cx-136,651-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  m_button_rightzoom.SetWindowPos(&m_button_rightzoom, cx-110,624-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  m_button_leftzoom.SetWindowPos(&m_button_leftzoom, cx-162,624-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  m_button_zoomin.SetWindowPos(&m_button_zoomin, cx-110,597-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  m_button_zoomout.SetWindowPos(&m_button_zoomout, cx-110,651-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_upzoom.SetWindowPos(&m_button_upzoom, cx-133,597-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_downzoom.SetWindowPos(&m_button_downzoom, cx-133,637-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_rightzoom.SetWindowPos(&m_button_rightzoom, cx-110,617-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_leftzoom.SetWindowPos(&m_button_leftzoom, cx-156,617-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_zoomin.SetWindowPos(&m_button_zoomin, cx-110,597-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  //	m_button_zoomout.SetWindowPos(&m_button_zoomout, cx-110,637-htop,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  
  /*	//Window Thumbnails
  m_Window_Pic_1.SetWindowPos(&m_Window_Pic_1,32,53-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_2.SetWindowPos(&m_Window_Pic_2,32,153-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_3.SetWindowPos(&m_Window_Pic_3,32,253-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_4.SetWindowPos(&m_Window_Pic_4,32,353-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_5.SetWindowPos(&m_Window_Pic_5,32,453-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_scrollbar_vertical.SetWindowPos(&m_scrollbar_vertical,160,43-htop,15,509,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
    //Session Thumbnails
    m_Session_Pic_1.SetWindowPos(&m_Session_Pic_1,206,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
    m_Session_Pic_2.SetWindowPos(&m_Session_Pic_2,328,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
    m_Session_Pic_3.SetWindowPos(&m_Session_Pic_3,450,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
    m_Session_Pic_4.SetWindowPos(&m_Session_Pic_4,572,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
    m_Session_Pic_5.SetWindowPos(&m_Session_Pic_5,694,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
    m_scrollbar_horizontal.SetWindowPos(&m_scrollbar_horizontal,198,cy-20-htop-hsession-5,618,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
    
      //Session Dates
      m_session_date1.SetWindowPos(&m_session_date1,236,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
      m_session_date2.SetWindowPos(&m_session_date2,358,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
      m_session_date3.SetWindowPos(&m_session_date3,480,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
      m_session_date4.SetWindowPos(&m_session_date4,602,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
      m_session_date5.SetWindowPos(&m_session_date5,724,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW); 
  */
  
  //Window Thumbnails
  m_Window_Pic_1.SetWindowPos(&m_Window_Pic_1,32,m_windowSectionPos+49-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_2.SetWindowPos(&m_Window_Pic_2,32,m_windowSectionPos+149-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_3.SetWindowPos(&m_Window_Pic_3,32,m_windowSectionPos+249-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_4.SetWindowPos(&m_Window_Pic_4,32,m_windowSectionPos+349-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_5.SetWindowPos(&m_Window_Pic_5,32,m_windowSectionPos+449-htop,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_scrollbar_vertical.SetWindowPos(&m_scrollbar_vertical,160,m_windowSectionPos+39-htop,15,509,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Session Thumbnails
  m_Session_Pic_1.SetWindowPos(&m_Session_Pic_1,m_sessionSectionPos+14,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_2.SetWindowPos(&m_Session_Pic_2,m_sessionSectionPos+136,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_3.SetWindowPos(&m_Session_Pic_3,m_sessionSectionPos+258,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_4.SetWindowPos(&m_Session_Pic_4,m_sessionSectionPos+380,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_5.SetWindowPos(&m_Session_Pic_5,m_sessionSectionPos+502,cy-129-htop-hsession,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_scrollbar_horizontal.SetWindowPos(&m_scrollbar_horizontal,m_sessionSectionPos+6,cy-20-htop-hsession-5,618,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Session Dates
  m_session_date1.SetWindowPos(&m_session_date1,m_sessionSectionPos+36,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date2.SetWindowPos(&m_session_date2,m_sessionSectionPos+158,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date3.SetWindowPos(&m_session_date3,m_sessionSectionPos+280,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date4.SetWindowPos(&m_session_date4,m_sessionSectionPos+402,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date5.SetWindowPos(&m_session_date5,m_sessionSectionPos+524,cy-40-htop-hsession,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  //STATUSBAR
  CRect statrect;
  //GetClientRect(&statrect);
  //int const statusbarpanes = 2;   
  Status_bar.GetClientRect(&statrect);
  //int widths[statusbarpanes] = {cx/3,cx/3,cx/3};
  
  int widrect = statrect.Width();//statrect.Width();
  
  char cwidth[10];
  char ccx[10];
  CString width;
  CString scx;
  itoa(widrect,cwidth,10);
  itoa(cx,ccx,10);
  width = cwidth;
  scx = ccx;
  
  //AfxMessageBox("Status_bar.GetClientRect.Width() = " + width + " cx = "  + scx);
  
  Status_bar.SetWindowPos(0, 0, cy-20+2, cx, 20, SWP_SHOWWINDOW);//SetWindowPos(&Status_bar,cx,720,cx,cy,SBPS_NORMAL);
  Status_bar.SetPaneInfo(0,ID_INDICATOR_STATUS,SBPS_NORMAL,cx/2);
  Status_bar.SetPaneInfo(1,ID_INDICATOR_PIXEL,SBPS_NORMAL,cx/8);
  Status_bar.SetPaneInfo(2,ID_INDICATOR_AUTODETECT,SBPS_NORMAL,(3*cx)/8);
  
  
  UINT m_bottomState = GetMenu()->GetMenuState(ID_VIEW_BOTTOMPANE, MF_BYCOMMAND);
  if(m_bottomState == MF_UNCHECKED)
  {
    int m_sessionSectionBottom;
    RECT m_RectWindow;
    m_session_section.GetWindowRect(&m_RectWindow);
    m_sessionSectionBottom = m_RectWindow.bottom;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 191,29-htop, m_RectWindow.right - m_RectWindow.left, 
      m_sessionSectionBottom - m_RectWindow.top, SWP_NOZORDER|SWP_SHOWWINDOW);
    Rootfly_Bottom_Objects_Hide();
  }
  
  UINT m_rightState = GetMenu()->GetMenuState(ID_VIEW_RIGHTPANE, MF_BYCOMMAND);
  if(m_rightState == MF_UNCHECKED)
  {
    int m_propertiesSectionRight;
    RECT m_RectWindow;
    m_properties_section.GetWindowRect(&m_RectWindow);
    m_propertiesSectionRight = m_RectWindow.right;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 191,29-htop, m_propertiesSectionRight - m_RectWindow.left , m_RectWindow.bottom - m_RectWindow.top, SWP_NOZORDER|SWP_SHOWWINDOW);
    Rootfly_Right_Objects_Hide();
  }
  
  UINT m_leftState = GetMenu()->GetMenuState(ID_VIEW_LEFTPANE, MF_BYCOMMAND);
  if(m_leftState == MF_UNCHECKED)
  {
    int m_windowSectionLeft;
    int m_windowSectionTop;
    int m_imagesSectionLength;
    RECT m_RectWindow;
    m_window_section.GetWindowRect(&m_RectWindow);
    m_windowSectionLeft = m_RectWindow.left;
    m_windowSectionTop = m_RectWindow.top;
    m_images_section.GetClientRect(&m_RectWindow);
    m_imagesSectionLength = m_RectWindow.bottom;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 19,29-htop, m_RectWindow.right - m_windowSectionLeft , m_imagesSectionLength, SWP_NOZORDER|SWP_SHOWWINDOW);
    Rootfly_Left_Objects_Hide();
  }
}

void CRootflyDlg::OnSize(UINT nType, int cx, int cy) 
{
  //  m_window_size = CPoint( cx, cy );
  
  /* working on the resizing problems
  // prevents items from resizeing if size is less than these values
  if (cy < 595) 
  {
		SetWindowPos(this,0,0,cx,595,SWP_NOZORDER|SWP_SHOWWINDOW);
    return;
    } 
    else if (cx < 990) 
    {
    SetWindowPos(this,0,0,,990,cy,SWP_NOZORDER|SWP_SHOWWINDOW);
    return;
}  */
  //Sections
  //Sections
  
  int  a,b,c,d;
  char aa[10],cc[10],bb[10],dd[10],xx[10],yy[10];
  CString MFCString, A, B, C, D, X, Y;
		
  CRect section_rect2;
  m_images_section.GetWindowRect(&section_rect2); 
  
  a = section_rect2.left;
  b = section_rect2.right;
  c = section_rect2.top;
  d = section_rect2.bottom;
	 
  //convert ints to chars
  itoa(a,aa,10);
  itoa(b,bb,10);
  itoa(c,cc,10);
  itoa(d,dd,10);
  itoa(cx,xx,10);
  itoa(cy,yy,10);
  //converts chars to strings
  A = aa;
  B = bb;
  C = cc;
  D = dd;
  X = xx;
  Y = yy;
  //Concat
  MFCString = A + " " +B + " "+ C + " " +D + " " + X+ " " + Y;
  
  //Display Message box
  //AfxMessageBox(MFCString);]
  
  //nType == SIZE_MAXIMIZED
  //nType == SIZE_RESTORED
  OnSizeNew(nType,cx, cy);
  
  //STATUSBAR
  CRect statrect;
  GetClientRect(&statrect);
  
  //Status_bar.SetPaneInfo(0,ID_INDICATOR_STATUS,SBPS_NORMAL,statrect.Width()-500);
  //Status_bar.SetPaneInfo(1,ID_INDICATOR_PIXEL,SBPS_STRETCH,0);
  
  SetZoomWindows();  
  UpdateStatusText();
  
}

CRect CRootflyDlg::GetClientRectOfImageDisplayArea() const
{
  CRect section_rect;
  m_images_section.GetWindowRect(&section_rect);
  
  section_rect.DeflateRect(1,7,2,2);
  ScreenToClient(section_rect); 
  return section_rect;
}

/* 
/  @brief:  This calculates the window position and corresponding
/           boundary rectangles for the drawing areas depending on
/           the zoom state and the current window size.
*/
void CRootflyDlg::SetZoomWindows()
{
  CRect main_rect, top_rect, middle_rect, bottom_rect, section_rect, clip_rect;
  blepo::ImgBgr img;
  double main_img_asp_ratio, new_rect_width, new_rect_height;
  section_rect = GetClientRectOfImageDisplayArea();
  
  // Get current image to determine image width, height, aspect ratio.
  // If the current images is blank (due to a blank session), this will give a
  // height and width of 0, causing nothing to be drawn.  So if the current image is blank,
  // keep going backwards through the sessions looking for a non-blank image.  In the worst case,
  // at least the first session should never be blank.
  int i = 0;
  while( ! m_rfy_data->GetApplicationData().GetImageByOffset( 0, i, &img ) && i >= -1 * m_rfy_data->GetApplicationData().GetCurrentSession() )
    // The check for i >= -1 * CurrentWindow is necessary or the program loops infinitely when no file is loaded
  {
    i--;
  }
  main_img_asp_ratio = ((double) img.Width()) / ((double) img.Height());
  CPoint cpt = section_rect.CenterPoint();
  int w = section_rect.Width(), h = section_rect.Height();
  
  // test code for zooming in
  if (m_check_zoom_in.GetCheck() == BST_CHECKED) 
  {
    if (m_zoomin != 1) 
    { // zoom in on image
      new_rect_height = h * m_zoomin;
      new_rect_width = new_rect_height * main_img_asp_ratio;
      
      if      ((new_rect_height / 2) <= m_shifty)   m_shifty = new_rect_height/2;
      else if ((new_rect_height / 2) <= -m_shifty)  m_shifty = -new_rect_height/2;
      if      ((new_rect_width / 2) <= m_shiftx)    m_shiftx = new_rect_width/2;
      else if ((new_rect_width / 2) <= -m_shiftx )  m_shiftx = -new_rect_width/2;
      
      cpt.x -= m_shiftx;
      cpt.y -= m_shifty;
    }
    else 
    { // display image at full resolution; scale image to client area
      double ww = h * main_img_asp_ratio, hh = w / main_img_asp_ratio;
      new_rect_width  = (ww > w) ? w  : ww;  // set width and height whether image is wide or tall
      new_rect_height = (ww > w) ? hh : h ;
    }
  }
  else if (IsZoomOutMode())
  { // 3x3 display mode
    if ( h * main_img_asp_ratio < w )
    {
      new_rect_height = h / 3;
      new_rect_width = new_rect_height * main_img_asp_ratio;
    }
    else
    {
      new_rect_width = w / 3;
      new_rect_height = new_rect_width / main_img_asp_ratio;
    }
  }
  else
  {
    double ww = h * main_img_asp_ratio, hh = w / main_img_asp_ratio;
    new_rect_width  = (ww > w) ? w  : ww;  // set width and height whether image is wide or tall
    new_rect_height = (ww > w) ? hh : h ;
  }
  main_rect.left   = cpt.x - (int) (new_rect_width  / 2);
  main_rect.top    = cpt.y - (int) (new_rect_height / 2);
  main_rect.right  = cpt.x + (int) (new_rect_width  / 2);
  main_rect.bottom = cpt.y + (int) (new_rect_height / 2);
  
  
  
  m_converter.SetClientRect( main_rect );
  m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );  // may be unnecessary, but will not hurt
  
  m_converter.SetImageSize( CSize( img.Width(), img.Height() ) );
  m_converter.SetClientRect( main_rect );
  m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );  // may be unnecessary, but will not hurt
  
  //ReDraw(1, 0);
}

void CRootflyDlg::OnEditImageFileFormat() 
{	
  ImageFileFormat iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  ImageFileFormatDlg optionsdlg(&iff,m_rfy_data->GetExperiment().GetImageDir(),this);
  int id = optionsdlg.DoModal();
  if (id == IDOK)
  {
    m_controller.C_SetImageFileFormat(iff);
    // Get the first image and get the size of the image to set the image
    // rectangle for the image converter
    CString firstImage = iff.GetFirstImage(m_rfy_data->GetExperiment().GetImageDir());
    CRect rect = iff.GetImageRectangle(firstImage);
    m_converter.SetImageSize( CSize(rect.Width(), rect.Height()) );
  }
}

void CRootflyDlg::OnHelpAbout() 
{
  CDialog dlg(IDD_ABOUTBOX);
  dlg.DoModal();
}

void CRootflyDlg::OnHelpContents() 
{
  int index, return_val;
  CString help_path = AfxGetApp()->m_pszHelpFilePath; 
  
  index = help_path.ReverseFind('\\');
  if(index == -1) return;
  help_path = help_path.Left(index);
  
  while(1)
  {
    //execute help file
    return_val = (int)ShellExecute(NULL, "open","Rootfly.chm", NULL, help_path, SW_SHOWNORMAL);
    if(return_val == ERROR_FILE_NOT_FOUND)
    {
      index = help_path.ReverseFind('\\');
      if(index == -1) break;
      help_path = help_path.Left(index);
    }
    else break;
  }
}

void CRootflyDlg::OnOK() 
{
  if (GetFocus() == &m_edit_tube)
  {
    OnKillfocusEditTube();
  }
  else if (GetFocus() == &m_edit_window)
  {
    OnKillfocusEditWindow();
  }
  else if (GetFocus() == &m_edit_session)
  {
    OnKillfocusEditSession();
  }
  else if (GetFocus() == &m_edit_diameter)
  {
    OnKillfocusEditDiameter();
  }
  else if (GetFocus() == m_combo_custom.GetWindow(GW_CHILD))  // focus is on the child of the combo box
  {
    OnKillfocusComboCustom();
  }
}		

void CRootflyDlg::OnCancel() 
{
  // Perhaps the purpose of this line is so that 'escape' key will not cause program to exit.  I can't remember -- STB
  CDialog::OnCancel();
}

// Set the diameter of the selected root based upon the text typed in the edit box
void CRootflyDlg::OnKillfocusEditDiameter() 
{
  if ( GetMode() == MODE_ROOTSELECTED)
  {
    const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
    if (!tube)  
    {
      return;
    }
    const Root *root = tube->GetRoot(m_selected_root_id);
    if (!root) 
    {
      return;
    }
    const RootInstance *inst = root->GetRootInstance(m_rfy_data->GetApplicationData().GetCurrentSession());
    if (!inst)  
    {
      return;
    }
    const RDiameter *diam = inst->GetDiameter(0);
    if (!diam)  return;
    
    CString temp;
    m_edit_diameter.GetWindowText(temp);
    double new_radius = atof(temp) * m_rfy_data->GetExperiment().GetMMConversion() / 2.0;
    if (new_radius > 0)
    {
      m_controller.C_ResizeDiameter(0, m_selected_root_id, m_selected_root_session, new_radius);
    }
    ReDraw(1,0);  
  }
}

// Set the current session based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditSession() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_session = m_rfy_data->GetApplicationData().GetCurrentSession();
  int min_session = m_rfy_data->GetExperiment().GetMinSessions();
  int max_session = m_rfy_data->GetExperiment().GetMaxSessions();
  int new_session;
  
  // get edit box data - stores as session to send
  m_edit_session.GetWindowText(temp);
  new_session = atoi(temp);
  if (new_session < 1) 
  {
    temp.Format("%d", curr_session);
    m_edit_session.SetWindowText(temp);
    return;
  }
  if (new_session != curr_session)
  {
    // user enters value bigger than range. set new range
    if (new_session > max_session) 
    {
      m_controller.C_SetMaxSessions(new_session); 
      m_spin_session.SetRange(min_session, new_session);
      m_scrollbar_horizontal.SetScrollRange(min_session, new_session);
    }
    // user enters value smaller than range. set new range
    if (new_session < min_session) 
    {
      m_controller.C_SetMinSessions(new_session); 
      m_spin_session.SetRange(new_session, max_session);
      m_scrollbar_horizontal.SetScrollRange(new_session, max_session);
    }
    m_scrollbar_horizontal.SetScrollPos(new_session);  
    m_controller.C_SetSession(new_session);
    temp.Format("%d", new_session);
    m_edit_session.SetWindowText(temp);
    ReDraw(1, 1);   
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

// Set the current window based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditWindow() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int min_window = m_rfy_data->GetExperiment().GetMinWindows();
  int max_window = m_rfy_data->GetExperiment().GetMaxWindows();
  int new_window;	
  
  // get edit box data - stores as window to send
  m_edit_window.GetWindowText(temp);
  new_window = atoi(temp);
  if (new_window < 1) 
  {
    temp.Format("%d", curr_window);
    m_edit_window.SetWindowText(temp);
    return;  
  }
  if (new_window != curr_window)
  {
    // user enters value bigger than range. set new range
    if (new_window > max_window) 
    {
      m_controller.C_SetMaxWindows(new_window); 
      m_spin_window.SetRange(min_window, new_window);
      m_scrollbar_vertical.SetScrollRange(min_window, new_window);
    }
    // user enters value bigger than range. set new range
    if (new_window < min_window) 
    {
      m_controller.C_SetMinWindows(new_window); 
      m_spin_window.SetRange(new_window, max_window);
      m_scrollbar_vertical.SetScrollRange(new_window, max_window);
    }
    m_scrollbar_vertical.SetScrollPos(new_window);
    m_controller.C_SetWindow(new_window);
    m_converter.SetWindow( new_window );
    temp.Format("%d", new_window);
    m_edit_window.SetWindowText(temp);
    ReDraw(1, 1);
  }  
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

CString CRootflyDlg::ConvertTubeName(CString input)
{
  // file_info is used to find length of tube in file format
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  const vector<OrderElement> file_info = iff.GetImageFileFormatInfo().InfoOrder;
  int tube_length;
  int input_length;             // length of string that the user entered
  
  // sets the length of the input
  input_length = input.GetLength();
  
  // find element in file
  for (int i = 0; i < file_info.size(); i++)
  {
    // sets the length of tube_length
    if (file_info.at(i).element == TUBE)
    {
      tube_length = file_info.at(i).length;
    }
  }
  
  if (input_length < tube_length)
  {
    for (int i = 0; i < tube_length-input_length; i++)
    {
      input = "0" + input;
    }
  }
  return input;
}

CString CRootflyDlg::StripPaddedZeros(CString input)
{
  int i;
  CString temp;
  for (i = 0; i < input.GetLength(); i++)
  {
    if (input.GetAt(i) != '0') 
    {
      break;
    }
  }
  temp = input.Right(input.GetLength() - i);
  return temp;
}

// Set the current tube based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditTube() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp, converted_tube;
  
  // get edit box data - stores as tube to send
  m_edit_tube.GetWindowText(temp);
  converted_tube = ConvertTubeName(temp);
  
  if (m_rfy_data->GetApplicationData().GetCurrentTubeName().Compare(temp))
  {
    m_controller.C_SetTube(converted_tube);
    // Displays the tube without padded zeros
    m_edit_tube.SetWindowText(StripPaddedZeros(temp));
    ReDraw(1, 1);
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

// Set the current tube based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinTube(NMHDR* pNMHDR, LRESULT* pResult) 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  TubeName tube_tag;
  CString temp, converted_tube;
  
  m_edit_tube.GetWindowText(temp);
  converted_tube = ConvertTubeName(temp);  
  tube_tag = converted_tube;
  if (pNMUpDown->iDelta > 0)
  {
    for (int i = 0; i < pNMUpDown->iDelta; i++)
    {
      tube_tag = m_rfy_data->GetExperiment().GetPrevTubeName(tube_tag);
//      int tagSize = tube_tag.GetLength();  // get length of tag
//  
//      if (tagSize != 0) // if tag is not empty
//      {
//        char asciiValue = tube_tag.GetAt(tagSize-1);
//    
//        for (int i = tagSize - 1; i >= 0; i--) // loop through tag
//        {
//          asciiValue = tube_tag.GetAt(i);
//          if (asciiValue == '0') // if number equals zero, set to nine and continue loop
//          {
//            tube_tag.SetAt(i, '9');
//          }
//          else  // else decrement ascii value and break out of loop
//          {
//            tube_tag.SetAt(i, --asciiValue);
//            break;
//          }
//        }
//      }
    }
  } 
  else
  {
    for (int i = pNMUpDown->iDelta; i < 0; i++)
    {
      tube_tag = m_rfy_data->GetExperiment().GetNextTubeName(tube_tag);
//      int tagSize = tube_tag.GetLength();  // get length of tag
//  
//      if (tagSize != 0) // if tag is not empty
//      {
//        char asciiValue = tube_tag.GetAt(tagSize-1);
//    
//        for (int i = tagSize - 1; i >= 0; i--) // loop through tag
//        {
//          asciiValue = tube_tag.GetAt(i);
//          if (asciiValue == '9') // if number equals nine, set to zero and continue loop
//          {
//            tube_tag.SetAt(i, '0');
//          }
//          else  // else increment ascii value and break out of loop
//          {
//            tube_tag.SetAt(i, ++asciiValue);
//            break;
//          }
//        }
//      }
    }
  } 
  
  if (tube_tag.Compare(m_rfy_data->GetApplicationData().GetCurrentTubeName()))
  {
    // displays the tube without padded zeros
    m_edit_tube.SetWindowText(StripPaddedZeros(tube_tag));
    m_controller.C_SetTube(tube_tag);
    *pResult = 1;
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

void CRootflyDlg::OnDeltaposSpinDiameter(NMHDR* pNMHDR, LRESULT* pResult) 
{
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  double temp;
  CString tempstr;
  double number = pNMUpDown->iPos - pNMUpDown->iDelta;

//  const Root *root = tube->GetRoot( GetMode() == MODE_ROOTSELECTED ? m_selected_root_id : -1 );
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if (!tube)
  {
    *pResult = 1;
    return;
  }
  const Root *root = tube->GetRoot(m_selected_root_id);
  if (!root)
  {
    *pResult = 1;
    return;
  }
  const RootInstance *inst = root->GetRootInstance(m_selected_root_session);
  if (!inst)
  {
    *pResult = 1;
    return;
  }
  const RDiameter *diam = inst->GetDiameter(0);
  if (!diam)
  {
    *pResult = 1;
    return;
  }
  double rad;

  if(diam->radius + (number / 2) < 0)
  {
    *pResult = 1;
    rad = diam->radius;
    temp = (rad*2);
    tempstr = ("%f mm", temp);
    m_edit_diameter.SetWindowText(tempstr);
  }
  else
  {
    *pResult = 1;
    rad = diam->radius + (number / 2);
    temp = (rad*2);
    if (rad > 0)
    {
      m_controller.C_ResizeDiameter(0, m_selected_root_id, m_selected_root_session, rad);
    }
    tempstr = ("%f mm", temp);
    m_edit_diameter.SetWindowText(tempstr);
  }
  ReDraw(1,0);
}

// Set the current session based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinSession(NMHDR* pNMHDR, LRESULT* pResult) 
{  
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  CString temp;
  int number;
  if(pNMUpDown->iDelta == -1 || pNMUpDown->iDelta == 1) number = pNMUpDown->iPos + m_rfy_data->GetSessionCollapser().GetRemappedIndex(pNMUpDown->iDelta);
  else number =pNMUpDown->iPos + pNMUpDown->iDelta;
  if ((number <= 0) || (number > m_rfy_data->GetExperiment().GetMaxSessions()))
  {	
    *pResult = 1;
    return;
  }
  if (number != m_rfy_data->GetApplicationData().GetCurrentSession())
  {
    *pResult = 1;
    temp.Format("%d", number);
    m_edit_session.SetWindowText(temp);
    m_scrollbar_horizontal.SetScrollPos(number);
    m_controller.C_SetSession(number);
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

// Set the current window based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinWindow(NMHDR* pNMHDR, LRESULT* pResult) 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  CString temp;
  int number =pNMUpDown->iPos + pNMUpDown->iDelta;
  if ((number <= 0) || (number > m_rfy_data->GetExperiment().GetMaxWindows()))
  {
    *pResult = 1;
    return;
  }
  if ((number) != m_rfy_data->GetApplicationData().GetCurrentWindow())
  {
    *pResult = 1;
    temp.Format("%d", number);
    m_edit_window.SetWindowText(temp);
    m_scrollbar_vertical.SetScrollPos(number);
    m_controller.C_SetWindow(number);	
    m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
}

// sets the root color when using the drop down box
void CRootflyDlg::OnSelchangeComboColor() 
{
  RootInstance::Color color;
  
  if (m_selected_root_id != -1)
  {
    color = (RootInstance::Color)m_combo_color.GetCurSel();
    m_controller.C_SetRootColor(m_selected_root_id, m_selected_root_session, color);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_COLOR_white()
{
  if (m_selected_root_id != -1)
  {
    m_combo_color.SetCurSel(0);
    m_controller.C_SetRootColor(m_selected_root_id, m_selected_root_session, RootInstance::COLOR_WHITE);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_COLOR_lightbrown()
{
  if (m_selected_root_id != -1)
  {
    m_combo_color.SetCurSel(1);
    m_controller.C_SetRootColor(m_selected_root_id, m_selected_root_session, RootInstance::COLOR_LIGHTBROWN);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_COLOR_darkbrown()
{
  if (m_selected_root_id != -1)
  {
    m_combo_color.SetCurSel(2);
    m_controller.C_SetRootColor(m_selected_root_id, m_selected_root_session, RootInstance::COLOR_DARKBROWN);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_COLOR_black()
{
  if (m_selected_root_id != -1)
  {
    m_combo_color.SetCurSel(3);
    m_controller.C_SetRootColor(m_selected_root_id, m_selected_root_session, RootInstance::COLOR_BLACK);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_STATE_live()
{
  if (m_selected_root_id != -1)
  {
    m_combo_state.SetCurSel(true);
    m_controller.C_SetRootAlive(m_selected_root_id, m_selected_root_session, true);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnMENUCONTEXT_SELECT_STATE_dead()
{
  if (m_selected_root_id != -1)
  {
    m_combo_state.SetCurSel(false);
    m_controller.C_SetRootAlive(m_selected_root_id, m_selected_root_session, false);
  }
  ReDraw(1, 0);
  EnableDisable();
}


// sets the root state when using the drop down box
void CRootflyDlg::OnSelchangeComboState() 
{
  int status;
  
  if (m_selected_root_id != -1)
  {
    status = m_combo_state.GetCurSel();
    if (status) m_controller.C_SetRootAlive(m_selected_root_id, m_selected_root_session,true);
    else m_controller.C_SetRootAlive(m_selected_root_id, m_selected_root_session,false);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnCHECKShowRootIDs() 
{
  //if (GetMode() == MODE_STARTINGMOVEALL){
  //  m_controller.C_SetShowRootID(false);
  //  return;
  //}
  
  if (m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowRootID(true);
  }
  else
  {
    m_controller.C_SetShowRootID(false);
  }
  
  ReDraw(1, 0);
}

void CRootflyDlg::OnCHECKShowRoots() 
{
  if (m_checkbox_show_roots.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowRoots(true);
  }
  else
  {
    CheckWhetherToDeselectRoot();
    m_controller.C_SetShowRoots(false);
  }
  
  ReDraw(1, 0);
  
}

void CRootflyDlg::OnCHECKShowDeadRoots() 
{
  SetMode(MODE_IMAGEDISPLAYED);
  if (m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowDeadRoots(true);
  }
  else
  {
    CheckWhetherToDeselectRoot();
    m_controller.C_SetShowDeadRoots(false);
  }
  
  ReDraw(1, 0);
}

void CRootflyDlg::OnCHECKShowDiameters() 
{
  if (m_checkbox_show_diameters.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowDiameters(true);
  }
  else
  {
    m_controller.C_SetShowDiameters(false);
  }
  
  ReDraw(1, 0);
}

////////////////////// start condition code handling //////////////////////////

CString CRootflyDlg::GetConditionDefault() const
{
  CString default_code;
  m_combo_custom.GetLBText(m_rfy_data->GetSettings().GetConditionDefaultIndex(), default_code);
  return default_code;
}

void CRootflyDlg::SetConditionDefault(const Root *root)
{
  m_controller.C_SetConditionCode(root->GetID(), m_selected_root_session, GetConditionDefault());
}

// sets the user code when using the drop down box
void CRootflyDlg::OnSelchangeComboCustom() 
{
  CString user_code;
  int index = m_combo_custom.GetCurSel();
  if (index >= 0)
  {
    m_combo_custom.GetLBText(index,user_code);
    m_controller.C_SetConditionCode(m_selected_root_id, m_selected_root_session, user_code,true);
  }
}

// sets the condition code to the selected root
void CRootflyDlg::SetCondition(CString user_code)
{
  int index;
  index = m_combo_custom.FindStringExact(-1,user_code);
  if (index >= 0)
  {
    m_combo_custom.SetCurSel(index);
    m_controller.C_SetConditionCode(m_selected_root_id, m_selected_root_session,user_code);
  } else if (user_code != "")
  {
    m_combo_custom.AddString(user_code);
    m_combo_custom.SelectString(-1,user_code);
    m_controller.C_SetConditionCode(m_selected_root_id, m_selected_root_session,user_code);
    m_controller.C_AddConditionCode(user_code);
  }
}

// sets the user code when drop down is used
void CRootflyDlg::OnDropdownComboCustom() 
{
  CString user_code;  
  m_combo_custom.GetWindowText(user_code);
  SetCondition(user_code);  
  int index = m_combo_custom.GetCurSel();
  m_combo_custom.SetCurSel(index);
}

// selects the correct choice or adds if not found
// in the combocustom box
void CRootflyDlg::OnKillfocusComboCustom() 
{	  
  CString user_code;
  m_combo_custom.GetWindowText(user_code);
  OnSelchangeComboCustom();
  SetCondition(user_code);
}

// reloads the custom combo box with the set passed in
void CRootflyDlg::UpdateCustomCombo(std::vector<CString> set)
{
  m_combo_custom.ResetContent();
  for (int i = 0; i < set.size(); i++)
  {
    m_combo_custom.AddString(set.at(i));
  }
}

void CRootflyDlg::InitializeConditionSet()
{
  // sets up the initial custom box
  std::vector<CString> condition_set;
  condition_set.push_back("NO_CODE");
  m_controller.C_SetConditionDefaultIndex(0);
  m_controller.C_SetConditionSet(condition_set);
  m_combo_custom.AddString("NO_CODE");
}

// brings up dialog to add or remove condition codes
void CRootflyDlg::OnCondition() 
{
  ManageConditionCodesDlg dlg;
  std::vector<CString> codes;
  codes = m_rfy_data->GetSettings().GetConditionSet();
  if (codes.size() == 0)
  {
    InitializeConditionSet();
    codes = m_rfy_data->GetSettings().GetConditionSet();
  }
  dlg.SetConditionCodes(codes);
  dlg.SetDefaultIndex(m_rfy_data->GetSettings().GetConditionDefaultIndex());
  int ret = dlg.DoModal();
  if (ret == IDOK)
  {
    codes = dlg.GetConditionCodes();
    if (codes.size() == 0)
    {
      codes.push_back("NO_CODE");
      m_controller.C_SetConditionDefaultIndex(0);
    } 
    else 
    {
      m_controller.C_SetConditionDefaultIndex(dlg.GetDefaultIndex());
    }
    m_controller.C_SetConditionSet(codes);
    UpdateCustomCombo(codes);
    UpdateProperties( GetMode() == MODE_ROOTSELECTED ? m_selected_root_id : -1);
  }    
}

////////////////////// end condition code handling //////////////////////////

void CRootflyDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
  // ignore anything but the scrollbar for changing the window
  // (note that spin controls also cause OnVScroll to be called for some reason) -- STB 4/21/10
  if (pScrollBar != &m_scrollbar_vertical)  return;

  int min, max;
  m_scrollbar_vertical.GetScrollRange(&min,&max);

  int CurPos = m_scrollbar_vertical.GetScrollPos();
  int OldPos = CurPos;
  
  // Determine the new position of scroll box.
  switch (nSBCode)
  {
    case SB_BOTTOM:    // Scroll to bottom.
      CurPos = max;
      break;    
    case SB_TOP:       // Scroll to top.
      CurPos = min;
      break;    
    case SB_ENDSCROLL:   // End scroll.
      return;    
    case SB_LINEDOWN:      // Scroll down.
      if (CurPos < max)
		CurPos++;     
        break;    
    case SB_LINEUP:   // Scroll up.
      if (CurPos > min)
		CurPos--;
        break;    
    case SB_PAGEDOWN:    // Scroll one page down.
	  if (CurPos < max-((NUM_THUMBNAILS-1)/2))
        CurPos += ((NUM_THUMBNAILS-1)/2);  
      else CurPos = max;
        break; 
    case SB_PAGEUP:      // Scroll one page up
	  if (CurPos > min+((NUM_THUMBNAILS-1)/2))
        CurPos -= ((NUM_THUMBNAILS-1)/2);  
      else 
        CurPos = min;
        break;
    case SB_THUMBPOSITION: // Scroll to absolute position. nPos is the position
      CurPos = nPos;      // of the scroll box at the end of the drag operation.
      break;    
    case SB_THUMBTRACK:   // Drag scroll box to specified position. nPos is the
      CurPos = nPos;     // position that the scroll box has been dragged to.
      break;
  }  
  // returns if the position didn't change
  if (OldPos == CurPos) 
	  return;

  if (GetMode() != MODE_NOIMAGEDISPLAYED) 
	  SetMode(MODE_IMAGEDISPLAYED);
  
  SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  m_scrollbar_vertical.SetScrollPos(CurPos);
  m_controller.C_SetWindow(CurPos);	
  m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );
  CString temp;
  temp.Format("%d", CurPos);
  m_edit_window.SetWindowText(temp); 
  ReDraw(1,1); 
}

void CRootflyDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{  
  // ignore anything but the scrollbar for changing the window
  // (just in case) -- STB 4/21/10
  if (pScrollBar != &m_scrollbar_horizontal)  return;

  int CurPos = m_scrollbar_horizontal.GetScrollPos();
  int OldPos = CurPos;
  int min, max, left, right;
  m_scrollbar_horizontal.GetScrollRange(&min,&max);
  
  // Determine the new position of scroll box.
  switch (nSBCode)
  {
    case SB_LEFT:                       // Scroll to far left.
      CurPos = min;
      break;    
    case SB_RIGHT:                      // Scroll to far right.
      CurPos = max;
      break;    
    case SB_ENDSCROLL:                  // End scroll.
      return;    
    case SB_LINELEFT:                   // Scroll left.
      //if (CurPos > min)      CurPos--;     
      left = m_rfy_data->GetSessionCollapser().GetRemappedIndex(-1);
      if(CurPos + left >= min)
        CurPos += left;
      else 
        CurPos = min;
      break;    
    case SB_LINERIGHT:                  // Scroll right.
      //if (CurPos < max)      CurPos++;
      right = m_rfy_data->GetSessionCollapser().GetRemappedIndex(1);
      if(CurPos + right <= max)
        CurPos += right;
      else CurPos = max;
        break;
    
    case SB_PAGELEFT:                   // Scroll one page left.
    //if (CurPos > min+((NUM_THUMBNAILS-1)/2))        CurPos -= ((NUM_THUMBNAILS-1)/2);  
    //else CurPos = min;
      left = m_rfy_data->GetSessionCollapser().GetRemappedIndex(-2);
      if(CurPos + left >= min) 
		  CurPos += left;
      else 
		  CurPos = min;
      break;    
    
    case SB_PAGERIGHT:                  // Scroll one page right
    //if (CurPos <max-((NUM_THUMBNAILS-1)/2))        CurPos += ((NUM_THUMBNAILS-1)/2);  
    //else CurPos = max;
      right = m_rfy_data->GetSessionCollapser().GetRemappedIndex(2);
      if(CurPos + right <= max) 
        CurPos += right;
      else 
        CurPos = max;
      break;    
    case SB_THUMBPOSITION:     // Scroll to absolute position. nPos is the position
      CurPos = nPos;           // of the scroll box at the end of the drag operation.
      break;    
    case SB_THUMBTRACK:        // Drag scroll box to specified position. nPos is the
      CurPos = nPos;           // position that the scroll box has been dragged to.
      break;
  }  
  // returns if position didn't change
  if (OldPos == CurPos) return;
  
  // Set the new position of the thumb (scroll box).  
  if ( GetMode() != MODE_NOIMAGEDISPLAYED) SetMode(MODE_IMAGEDISPLAYED);
  SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  m_scrollbar_horizontal.SetScrollPos(CurPos);
  m_controller.C_SetSession(CurPos);	
  CString temp;
  temp.Format("%d", CurPos);
  m_edit_session.SetWindowText(temp);
  ReDraw(1, 1);
  
}

void CRootflyDlg::ProgressImageClicked(int window, int tube, int session)
{
  CString new_locationW, new_locationS;
  new_locationW.Format("%d",window);
  new_locationS.Format("%d", session);
	 
  m_edit_window.SetWindowText(new_locationW);      
  m_scrollbar_vertical.SetScrollPos(window);
  m_controller.C_SetWindow(window);;
	 
  m_edit_session.SetWindowText(new_locationS);      
  m_scrollbar_horizontal.SetScrollPos(session);
  m_controller.C_SetSession(session);
  
  CString tube_tag = m_rfy_data->GetExperiment().GetAllTubes()->at(tube).GetTag();
  m_controller.C_SetTube(tube_tag);
  m_edit_tube.SetWindowText(StripPaddedZeros(tube_tag));
  
  ReDraw(1,1);
}

void CRootflyDlg::ThumbnailClicked(int index, int type)
{  
  int pic_clicked, max;
  CString new_location;
  if (type == 0)
  { // user clicked on window rect
    max = m_rfy_data->GetExperiment().GetMaxWindows();
    pic_clicked = m_rfy_data->GetApplicationData().GetCurrentWindow();
  } 
  else 
  { // user clicked on session rect
    max = m_rfy_data->GetExperiment().GetMaxSessions();
    pic_clicked = m_rfy_data->GetApplicationData().GetCurrentSession();
  }
  
  if ((index < 0) && (pic_clicked < abs(index)+1) && type == 0) return;
  else if ((index > 0) && (max-pic_clicked < index) && type == 0) return;  
  if ((m_rfy_data->GetSessionCollapser().GetRemappedIndex(index) < 0) && (pic_clicked < abs(m_rfy_data->GetSessionCollapser().GetRemappedIndex(index))+1) && type == 1) return;
  else if ((m_rfy_data->GetSessionCollapser().GetRemappedIndex(index) > 0) && (max-pic_clicked < m_rfy_data->GetSessionCollapser().GetRemappedIndex(index)) && type == 1) return;  
  
  new_location.Format("%d",pic_clicked+m_rfy_data->GetSessionCollapser().GetRemappedIndex(index));
  if (type == 0)
  {
    m_edit_window.SetWindowText(new_location);      
    m_scrollbar_vertical.SetScrollPos(pic_clicked+index);
    m_controller.C_SetWindow(pic_clicked+index);
  } else 
  {
    m_edit_session.SetWindowText(new_location);      
    m_scrollbar_horizontal.SetScrollPos(pic_clicked+m_rfy_data->GetSessionCollapser().GetRemappedIndex(index));
    m_controller.C_SetSession(pic_clicked+m_rfy_data->GetSessionCollapser().GetRemappedIndex(index));
  }
  
  // reset the Mode so the text in the status bar updates correctly
  if ( GetMode() != MODE_NOIMAGEDISPLAYED )
  {
    SetMode(MODE_IMAGEDISPLAYED);
    m_current_root_being_drawn.ClearAll();
    //    m_selected_root_id = -1;
  }
  
  ReDraw(1,1);
}

bool CRootflyDlg::CheckForThumbnailClick(CPoint point)
{
  if ( GetMode() == MODE_NOFILELOADED ) return false;   
  CRect rect;
  bool clicked = false;
  for (int i = 0; i < 2; i++)
  {
    for (int index = -1*(NUM_THUMBNAILS-1)/2 ; index<(NUM_THUMBNAILS+1)/2 ; index++)
    {
      // i=0 means get window rect
      // i=1 means get session rect
      rect = GetThumbnailRect(index,i);
      if (rect.PtInRect(point))
      {
        ThumbnailClicked(index,i);
        clicked = true;
      }
    }
  }
  return clicked;
}

void CRootflyDlg::OnEditSessionDates()
{
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  if (!iff.DateIsIncluded())
  {
    SessionDatesDlg dlg;
    std::vector<Date> dates;
    dates = m_rfy_data->GetExperiment().GetSessionDates();
    dlg.SetSessionDates(dates);
    int ret = dlg.DoModal();
    if (ret == IDOK)
    {
      dates = dlg.GetSessionDates();
      m_controller.C_SetSessionDates(dates);          
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                                 DRAWING FUNCTIONS                                   //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

/**
@ brief: ReDraw updates only the necessary images
@ param: main_changed - if 1 update main image, if 0 do not update main image
@ param: update_thumbnails - if 1 redraw thumbnails, if 0 do not redraw thumbnails
*/

void CRootflyDlg::ReDraw(int main_changed, int update_thumbnails)
{   
  CClientDC dc(this);  
  CRect section_rect, clip_rect;
  m_images_section.GetWindowRect(&section_rect);
  
  section_rect.DeflateRect(1,7,2,2);
  clip_rect = section_rect;
  ScreenToClient(clip_rect);
  
  CRgn clip_rgn;
  clip_rgn.CreateRectRgnIndirect(&clip_rect);
  CMemDC mdc(&dc);
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName( m_rfy_data->GetApplicationData().GetCurrentTubeName() );  // current tube
  
  if (update_thumbnails) 
  {
    UpdateThumbnails();   
  }
  
  if (main_changed)
  {
    UpdateMainImage(mdc);
  } 
  
  UpdateProperties(m_selected_root_id); //this should be sufficient to ensure the root exists, i.e. root can not be selected if it does not exist
  
  // shows the mm conversion on the window
  CString str;
  str.Format("%5.1f pixels per mm", m_rfy_data->GetExperiment().GetMMConversion());
  
  Status_bar.SetPaneText(1,str);  //STATUSBAR
  //m_mm_text.SetWindowText(str);
  
  m_graphics.SetClipRegion(dc, IsZoomOutMode());
  if (m_check_zoom_in.GetCheck() == BST_CHECKED && GetMode() != MODE_NOFILELOADED)
  {
    // placeholder for new clipping region on zoom in
    m_graphics.SetClipRegion(dc, clip_rect);
  }
  
  //gray out lesser checkboxes if show roots is not checked
  EnableDisable();
  
  //ensure user wants to see all roots before drawing
  if(m_checkbox_show_roots.GetCheck() == BST_CHECKED)
  {
    m_graphics.DrawAllRoots(mdc, GetMode(), IsZoomOutMode());      // draw all the existing roots in the data structure
    
    //3x3 operations find offset for drawing
    int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
    CRect main = IsZoomOutMode() 
      ? m_converter.GetExtendedClientRect(window-1, window+1)
      : m_converter.GetClientRect( window );
    int offset = main.right - main.left;
    
    if(m_current_root_being_drawn.GetSessionNumber() == m_rfy_data->GetApplicationData().GetCurrentSession())
    {
      offset *= 0;
    }
    if(m_current_root_being_drawn.GetSessionNumber() == m_rfy_data->GetApplicationData().GetCurrentSession() - 1)  // 3X3 left
    {
      offset *= -1;
    }
    if(m_current_root_being_drawn.GetSessionNumber() == m_rfy_data->GetApplicationData().GetCurrentSession() + 1)  // 3x3 right
    {
      offset *= 1;
    }
    m_graphics.DrawNewRoot(mdc, GetMode(), m_current_root_being_drawn, m_last_mouse_point, offset);   // draw the temporary new root that is being created
    if ( m_selected_root_id > 0)
    {
      // draw the selected root (or the root under the mouse, i.e., the one that will be selected if the mouse button is pressed)
      m_graphics.DrawSelectedRoot( mdc, m_selected_root_id, m_selected_root_session, GetMode(), m_last_mouse_point, m_extending_from_tail, offset);  
    }
  }
}

// checks to see if the root detector has been run on the current image
// will run if it has not been run, will do nothing if it has been run
void CRootflyDlg::RunAutomaticRootDetector(bool button)
{
  if (GetMode() == MODE_NOIMAGEDISPLAYED || GetMode() == MODE_NOFILELOADED)  
  {
    return;
  }
  
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName( m_rfy_data->GetApplicationData().GetCurrentTubeName() );  // current tube
  int window = m_rfy_data->GetApplicationData().GetCurrentWindow();   // current window
  int session = m_rfy_data->GetApplicationData().GetCurrentSession(); // current session
  int i, j, k, l;
  
  // check whether image has been scanned (i.e., whether automatic root detector has been run)
  if (button || !tube || !tube->GetImageScanned(window,session))
  {
    
    // if any roots exist in the current tube, then do not run detector
    //need to check window and not tube!
    //if (!(tube && tube->AutoDetectGetRootsInWindow(window) > 0))
    //{
    
    // run the automatic root detector here  
    blepo::ImgBgr img;    	
    m_rfy_data->GetApplicationData().GetImageByOffset( 0, 0, &img );
    
    unsigned char *img_data = img.BytePtr();
    std::vector<DetectedRoot> detected_roots;
    double templength = 0;
    double dx, dy;
    
    detected_roots.resize(30);
    for(k = 0; k < detected_roots.size(); k++)
    {
      detected_roots.at(k).points.resize(100);
      detected_roots.at(k).extremity.resize(50);
    }
    
    //the actual autodetect function
    MyDetectRoots(img.Width(), img.Height(), img_data, &detected_roots);
    
    
    if(GetMode() == MODE_NOIMAGEDISPLAYED) return;
    m_selected_root_id = -1;
    m_selected_root_session = -1;
    m_current_root_being_drawn.ClearAll();
    
    //mode: DRAWINGNEWROOT
    RPoint *rpt;
    RDiameter *rdm;
    const std::vector<RPoint> *line; 
    
    for(i = 0; i < detected_roots.size(); i++)    // number of detected roots
    {
      //count the total number of valid points
      int total_points = 0;
      for(j = 0; j < detected_roots.at(i).points.size(); j++)
      {
        if(detected_roots.at(i).points.at(j).x < 0 || detected_roots.at(i).points.at(j).y < 0) continue;
        total_points++;
      }
      // this vector keeps track of whether each detected point has been found to be within the radial measure of an established root
      std::vector<bool> pointInsideRadius;
      pointInsideRadius.resize(total_points);
      for (j=0; j<total_points; j++)
      {
        pointInsideRadius.at(j) = false;
      }
      
      //Diameter discard option
      if (m_rfy_data->GetSettings().GetAutoDetectDiameterCheck() && ((detected_roots.at(i).diameter*2) > m_rfy_data->GetSettings().GetAutoDetectDiameterValue())) continue;
      if (detected_roots.at(i).diameter < 0) continue;
      
      //Length discard option
      templength = 0;
      const std::vector<RootPoint> tempVec = (std::vector<RootPoint>) (detected_roots[i]).points;
      for (l = 1; l < tempVec.size(); l++)
      {
        if (tempVec[l].x < 0 || tempVec[l].y < 0) continue;
        dx = (tempVec[l].x-tempVec[l-1].x) / m_rfy_data->GetExperiment().GetMMConversion();
        dy = (tempVec[l].y-tempVec[l-1].y) / m_rfy_data->GetExperiment().GetMMConversion();
        templength += sqrt(pow(dx,2)+pow(dy,2));
      }
      if (m_rfy_data->GetSettings().GetAutoDetectLengthCheck() && (m_rfy_data->GetSettings().GetAutoDetectLengthValue() > templength)) continue;
      
      //Vertical discard option
      vector<RootPoint>::iterator pt1 = detected_roots.at(i).points.begin();
      vector<RootPoint>::iterator pt2 = detected_roots.at(i).points.begin() + (total_points-1);
      if (pt1->x > pt2->x)
      {
        vector<RootPoint>::iterator pt_temp = pt1;
        pt1 = pt2;
        pt2 = pt_temp;
      }
      double angle = fabs( atan2( fabs((float)pt1->x - pt2->x), fabs((float)pt1->y - pt2->y) ) * 180 / (4.0*atan(1.0)) );
      if (m_rfy_data->GetSettings().GetAutoDetectVerticalCheck() && (angle < m_rfy_data->GetSettings().GetAutoDetectDegreeValue())) continue;
      
      //Duplicate root check (only checks for exact match of points)
      //discards root if > 75% of points are within an established root's radial distance
      
      const RDiameter *curEstablishedDiameter;
      double curEstablishedRadius = 0.0;
      double distToEstRoot = 0.0;
      std::vector<RPoint>::const_iterator estRootPt1;
      std::vector<RPoint>::const_iterator estRootPt2;
      pt2d endpt1, endpt2;
      pt2d ptOnDetectedRoot;
      pt2d ptClosest;
      int shared_points = 0;
      
      const vector<Root> *current_roots =  tube? tube->GetAllRoots() : NULL;  // vector of current established roots
      if(current_roots != NULL)
      {
        for (l = 0; l < current_roots->size(); l++)
        {
          if(current_roots->at(l).GetPrimaryWindow() == window)
          {
            const RootInstance *inst = current_roots->at(l).GetRootInstance(session);  // the current established root instance
            if(!inst) continue;
            curEstablishedDiameter = inst->GetDiameter(0);
            curEstablishedRadius = curEstablishedDiameter->radius;
            estRootPt1 = inst->GetAllPoints()->begin();
            estRootPt2 = inst->GetAllPoints()->begin()+1;
            
            for ( ; estRootPt2<inst->GetAllPoints()->end(); estRootPt2++)
            {
              endpt1.x = estRootPt1->x;
              endpt1.y = estRootPt1->y;
              endpt2.x = estRootPt2->x;
              endpt2.y = estRootPt2->y;
              for (j=0; j<total_points; j++)
              {
                ptOnDetectedRoot.x = detected_roots.at(i).points.at(j).x;
                ptOnDetectedRoot.y = detected_roots.at(i).points.at(j).y;
                
                distToEstRoot = DistPt2Seg(endpt1, endpt2, ptOnDetectedRoot, &ptClosest);
                
                // if the distance from the jth detected point is inside the radial distance of and established root 
                // and that point has not yet been determined as inside the radial distance
                if (distToEstRoot < curEstablishedRadius*curEstablishedRadius && !pointInsideRadius.at(j))
                {
                  shared_points++;
                  // the jth point has been determined to be inside the radius, so set this true so it won't be looked at again.
                  pointInsideRadius.at(j) = true;
                }
              }
              estRootPt1 = estRootPt2;
            }
          }
        }
      }
      // look in RootflyDlg.h for a description of SHARED_POINT_PERCENTAGE and an explanation of how the autodetect function was changed
      if(shared_points > 0 && shared_points >= (total_points * SHARED_POINT_PERCENTAGE)) continue;
      
      //Create Root Points
      for(j = 0; j < detected_roots.at(i).points.size(); j++)
      {
        if(detected_roots.at(i).points.at(j).x < 0 || detected_roots.at(i).points.at(j).y < 0) continue;
        rpt = m_current_root_being_drawn.CreatePoint();
        assert(rpt); if(!rpt) return;
        rpt->x = detected_roots.at(i).points.at(j).x;
        rpt->y = detected_roots.at(i).points.at(j).y;
        rpt->window = window;
      }
      
      m_current_root_being_drawn.SetSessionNumber(session);
      m_current_root_being_drawn.SetAlive();
      
      //Create Root Diameter
      rdm = m_current_root_being_drawn.CreateDiameter();
      rdm->r_point.x = detected_roots.at(i).points.at(total_points/2).x;
      rdm->r_point.y = detected_roots.at(i).points.at(total_points/2).y;
      rdm->radius = detected_roots.at(i).diameter / 2;
      rdm->r_point.window = window;
      
      if(m_rfy_data->GetSettings().GetAutoDetectPostProcessCheck())
      {
        FixAutoDetectResolution(m_current_root_being_drawn, rdm->radius);
      }
      
      line = m_current_root_being_drawn.GetAllPoints();    
      if (line->size() == 0)  return;
      m_controller.C_CreateRoot( line->at(0).window, m_current_root_being_drawn, GetConditionDefault() );
      m_current_root_being_drawn.ClearAll();
      m_selected_root_id = -1;
      m_selected_root_session = -1;
      SetMode(MODE_IMAGEDISPLAYED);
      ReDraw(1,0);
    }
    
    //free memory reserved for roots
    while (detected_roots.size() > 0)
    {
      while (detected_roots.at(0).points.size() > 0)  detected_roots.at(0).points.erase( detected_roots.at(0).points.begin() );
      while (detected_roots.at(0).extremity.size() > 0)  detected_roots.at(0).extremity.erase( detected_roots.at(0).extremity.begin() );
      detected_roots.erase( detected_roots.begin() );
    }
    
    m_controller.C_SetImageScanned(true);
  }
  SetMode(MODE_IMAGEDISPLAYED);
}

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                              MOUSE BUTTON FUNCTIONS                                 //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

class ModeHandlerFactory
{
public:
  ModeHandlerFactory(CRootflyDlg* dlg) { m_mh = CreateModeHandler(dlg); }
  ~ModeHandlerFactory() { delete m_mh; }
  ModeHandler* GetModeHandler() { return m_mh; }
protected:
  ModeHandler* CreateModeHandler(CRootflyDlg* dlg)
  {
    switch( dlg->GetMode() )  
    {
    case MODE_NOFILELOADED:
      return new ModeHandlerMODE_NOFILELOADED(dlg);
    case MODE_STARTINGMOVEALL:
      return new ModeHandlerMODE_STARTINGMOVEALL(dlg);
    case MODE_MOVINGSINGLEROOT:
      return new ModeHandlerMODE_MOVINGSINGLEROOT(dlg);
    case MODE_MOVINGALLROOTS:
      return new ModeHandlerMODE_MOVINGALLROOTS(dlg);
    case MODE_SIZINGDIAMETER:
      return new ModeHandlerMODE_SIZINGDIAMETER(dlg);
    case MODE_DRAWINGNEWROOT:       
      return new ModeHandlerMODE_DRAWINGNEWROOT(dlg);
    case MODE_ROOTSELECTED:    
      return new ModeHandlerMODE_ROOTSELECTED(dlg);
    case MODE_NOIMAGEDISPLAYED:
      return new ModeHandlerMODE_NOIMAGEDISPLAYED(dlg);
    case MODE_EXTENDINGENDPOINT:
      return new ModeHandlerMODE_EXTENDINGENDPOINT(dlg);
    case MODE_IMAGEDISPLAYED: 
      return new ModeHandlerMODE_IMAGEDISPLAYED(dlg);
    case MODE_RESIZINGDIAMETER:
      return new ModeHandlerMODE_RESIZINGDIAMETER(dlg);
    case MODE_MOVINGRPOINT:
      return new ModeHandlerMODE_MOVINGRPOINT(dlg);
    case MODE_LOCATINGDIAMETER: 
      return new ModeHandlerMODE_LOCATINGDIAMETER(dlg);
    case MODE_REPOSITIONDIAMETER:
      return new ModeHandlerMODE_REPOSITIONDIAMETER(dlg);
    case MODE_ADDPOINT:
      return new ModeHandlerMODE_ADDPOINT(dlg);
    case MODE_PANNINGIMAGE:
      return new ModeHandlerMODE_PANNINGIMAGE(dlg);
    default:
      assert(0);
      AfxMessageBox("Program unstable.  Click Debug->Reset Mode");
      return new ModeHandlerMODE_NOFILELOADED(dlg);  // just pick the one that seems least harmful
    }
  }
private:
  ModeHandler* m_mh;
};

void CRootflyDlg::OnLButtonDown(UINT nFlags, CPoint point)
{  
  //  TRACE("mode = %d\n", GetMode());
  //  {
  //    CString foo;
  //    RPoint p = m_converter.ClientToImage( point );
  //    foo.Format("(%d,%d;%d)", p.x, p.y, p.window);
  //    AfxMessageBox(foo);
  ////    return;
  //  }
  CheckForThumbnailClick(point);   
  
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnLButtonDown(nFlags, point);
  }
  
  EnableDisable();
  ReDraw(1, 0);
}

void CRootflyDlg::OnLButtonUp(UINT nFlags, CPoint point)
{
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnLButtonUp(nFlags, point);
  }
}

void CRootflyDlg::OnLButtonDblClk(UINT nFlags, CPoint point)
{  
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnLButtonDblClk(nFlags, point);
  }
}

void CRootflyDlg::OnRButtonDown(UINT nFlags, CPoint point)
{
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnRButtonDown(nFlags, point);
  }
}

//void CRootflyDlg::LeftClickBeforeRight(UINT nFlags, CPoint point)
//{
//  this->OnLButtonDown(nFlags, point);
//}
//
//void CRootflyDlg::MouseMoveBeforeRight(UINT nFlags, CPoint point)
//{
//  this->OnMouseMove(nFlags, point);
//}

void CRootflyDlg::OnRButtonUp(UINT nFlags, CPoint point) 
{
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnRButtonUp(nFlags, point);
  }
}

void CRootflyDlg::OnMouseMove(UINT nFlags, CPoint point)
{     
  m_last_mouse_point = point;
  if(PointDetection(&point))
  {
    ModeHandlerFactory mhf(this);
    mhf.GetModeHandler()->OnMouseMove(nFlags, point);
  }
} 

// Converts points for 3x3 Grid.  The 3x3 Grid consists of three adjacent rects, one for each of three
//   sessions.  Each rect displays three images, one for each of three windows.
// This function is needed because coords that are passed to mode handler need to be offset with respect to the middle rect.
// This function does 3 things:
//     - finds in which rectangle a point is (the point comes from mouse move, mouse click, etc.)
//     - offsets point with respect to middle rect
//       For example, if the image width is 100, and the user clicks at x=80 in client coordinates, then
//           this function will change the 80 to 180 and set the session to current_session - 1.
//           (This is a legacy of the fact that the program used to show only 1 session with 3 windows.
//            So this function changes the x coordinate to be the one that would have been obtained if the
//            user had clicked at the same relative location in the image in the current session.)
//     - if no points have yet been added to m_current_root_being_drawn, then
//       sets the session number of m_current_root_being_drawn depending upon rect
//Returns: true if point is in valid rectangle
//         (i.e., if drawing a new root, it is in the appropriate rect; else in any of the three rects)
// (??? Will this logic work for other modes, e.g., extendendpoint?  Why or why not?)
//Parameters:
//		CPoint* point   => Pointer to the point being detected.  Will change this to the correct offset.
//Potential Problems:
//		Assumes the left and right rect are same size and directly next to main rect
//Written Spring 2010 -MTB
bool CRootflyDlg::PointDetection(CPoint* point)
{
  CRect main,left,right;
  const CPoint unchanged_point = *point;
  int curr_session;
  
  curr_session = m_rfy_data->GetApplicationData().GetCurrentSession(); // Get sesion of middle window
  if(true)  //in 3x3 grid mode
  {
    //Find offset by comparing to main window
    main = m_converter.GetExtendedClientRect(m_rfy_data->GetApplicationData().GetCurrentWindow()-1, m_rfy_data->GetApplicationData().GetCurrentWindow()+1);
    int offset = main.right - main.left; //Here is where to add border constant
    
    left = main;
    left.left -= offset;
    left.right -= offset;
    
    right = main;
    right.left += offset;
    right.right += offset;
    
    
    
    bool in_left = left.PtInRect( *point ) != 0;
    bool in_middle = main.PtInRect( *point ) != 0;
    bool in_right = right.PtInRect( *point ) != 0;
    bool in_any = in_left || in_middle || in_right ;
    if ( in_any )
    {
      int val;
      if (in_right)  val = -1;
      if (in_left)  val = 1;
      if (in_middle)  val = 0;
      point->x += (offset * val);
      
      m_point_session = curr_session + -1*val;
      if(m_current_root_being_drawn.GetAllPoints()->size() == 0)
      {
        m_current_root_being_drawn.SetSessionNumber(m_point_session);
      }
      //If session clicked is different from the session the selected root is in and in a mode
      //that modifies a selected root then return false.
      if(m_point_session != m_selected_root_session && (GetMode() == MODE_ADDPOINT || GetMode() == MODE_MOVINGRPOINT || GetMode() == MODE_RESIZINGDIAMETER || GetMode() == MODE_EXTENDINGENDPOINT || GetMode() == MODE_MOVINGSINGLEROOT))
      {
        return false;
      }
      if(m_point_session == m_current_root_being_drawn.GetSessionNumber())
        return true;
    }
    return false;
  }
  else  //not 3x3 grid mode 
  {
    main = m_converter.GetClientRect( m_rfy_data->GetApplicationData().GetCurrentWindow() );
    
    if(main.PtInRect( *point ) ) 
    {
      if(m_current_root_being_drawn.GetAllPoints()->size() == 0)
      {
        m_current_root_being_drawn.SetSessionNumber(curr_session);
      }
      m_point_session = curr_session;
      return true;
    }
    else
    {
      return false;
    }
  }
  
}

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                            FILE RELATED FUNCTIONS                                   //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

/**

  Action: Whenever user selects: File->Exit  
  Should close main window         
*/
void CRootflyDlg::OnFileExit() 
{
  // Send message to close window
  PostMessage(WM_CLOSE);
}

// Scan directory for max/min window/session, and set ranges of UI accordingly.
void CRootflyDlg::SetNavigation(const CString& file_directory)
{
  // scan all files and set max/min window/session
  int max_window = 0, min_window = 10000, max_session = 0, min_session = 10000;    
  
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  //iff.FindNavigationLimits(file_directory,&max_window,&min_window,&max_session,&min_session, this);
  iff.CatalogueImages(file_directory,&max_window,&min_window,&max_session,&min_session, this, m_controller);
  // save the map built in catalogue images
  m_controller.C_SetImageFileFormat(iff);
  
  //had to add these because CatalogueImages changes the current tube to the last one in the file. This may be a recent change, 
  //because it doesn't work like this in the old rootfly version I had
  ImageInfo ii = iff.GetImageInfoOfFirstFile(file_directory);
  m_controller.C_SetTube(ii.tube);
  
  m_controller.C_SetMinWindows(min_window);
  m_controller.C_SetMinSessions(min_session);
  m_controller.C_SetMaxWindows(max_window);
  m_controller.C_SetMaxSessions(max_session);
  
  // set values of Navigation pane
  // displays tube without padded zeros
  m_edit_tube.SetWindowText(StripPaddedZeros(m_rfy_data->GetApplicationData().GetCurrentTubeName()));
  m_spin_window.SetPos(m_rfy_data->GetApplicationData().GetCurrentWindow());
  m_spin_session.SetPos(m_rfy_data->GetApplicationData().GetCurrentSession());
  
  // sets the ranges of Navigation pane
  //m_exp_data = m_rfy_data->GetExperiment();
  m_spin_window.SetRange(1,m_rfy_data->GetExperiment().GetMaxWindows());
  m_spin_session.SetRange(1,m_rfy_data->GetExperiment().GetMaxSessions());
  m_scrollbar_vertical.SetScrollRange(1,m_rfy_data->GetExperiment().GetMaxWindows());
  m_scrollbar_horizontal.SetScrollRange(1,m_rfy_data->GetExperiment().GetMaxSessions());
  
  // set scroll bars 
  m_scrollbar_vertical.SetScrollPos(m_rfy_data->GetApplicationData().GetCurrentWindow());
  m_scrollbar_horizontal.SetScrollPos(m_rfy_data->GetApplicationData().GetCurrentSession());
}

// Scan directory for valid files.  If none found, reprompt user for valid image directory
// 'directory' is both input and output
bool CRootflyDlg::ValidateDirectory(CString* directory)
{
  CString& file_directory = *directory;
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  while(!iff.ImageFileExistsInDirectory(file_directory))
  {
    UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
    CFolderDialog dlg(_T("No Valid Images Found. Choose Folder where Images are Located" ), file_directory, this, uFlags);
    if (dlg.DoModal() == IDOK)
    {
      file_directory = dlg.GetFolderPath();
      file_directory += "\\";
      m_controller.C_SetImageDir(file_directory);
      m_controller.C_SetModify(true);
      
      //        filename = file_directory + m_rfy_data->GetApplicationData().GetFileName();
    }
    else return false;
  }   
  //    m_file_directory = file_directory;
  return true;
}

/**
/  
/    @Action: Whenever user selects: File->New
/             Creates a Browse Folder Dialog and 
/             allows user to select location of 
/             new experiment.
/
*/
void CRootflyDlg::OnFileNew() 
{
  CWaitCursor wait;
  UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
  
  m_first_save = false;
  
  // Let user choose folder that contains images for the new project
  CString file_directory;
  
  // Customize string in file dialog
  CString str = "Choose Folder where Images are Located";  // need _T?
  bool valid_directory = false;
  while (!valid_directory)
  {
    // gets the default directory
    file_directory = m_rfy_data->GetSettings().GetDefaultFileDir();
    
    CFolderDialog dlg(str, file_directory, this, uFlags);
    int fdlgid = dlg.DoModal();
    CWaitCursor wait;
    if (fdlgid == IDOK)
    {          
      // Look for images in chosen folder
      CFileFind ff;
      CString tmp = dlg.GetFolderPath();
      // sets the new default directory
      m_controller.C_SetDefaultFileDirectory(tmp);
      
      if (ff.FindFile(tmp+"\\*.jpg") || ff.FindFile(tmp+"\\*.bmp") || ff.FindFile(tmp+"\\*.jpeg"))
      {  
        CString image_directory = dlg.GetFolderPath();
        image_directory += "\\";
        
        // Let user select the image file name format
        ImageFileFormat iff = m_rfy_data->GetExperiment().GetImageFileFormat();
        ImageFileFormatDlg optionsdlg(&iff,image_directory,this);
        int id = optionsdlg.DoModal();
        CWaitCursor wait;
        if (id == IDOK)
        {
          //if (ValidateDirectory(&image_directory) == false) return;
          if (!iff.ImageFileExistsInDirectory(image_directory))
          {
            // do nothing have to find valid file first
            valid_directory = false;
          } else
          {
            // valid file found. start new project
            valid_directory = true;
            
            m_controller.C_ClearAllExperiment();
            m_controller.C_SetImageFileFormat(iff);
            m_controller.C_SetImageDir(image_directory);
            
            CString file_name = image_directory + DEFAULT_FILENAME;
            
            m_controller.C_CreateNewProject(_T(file_name),_T(image_directory));
            SetNavigation(image_directory);
            iff = m_rfy_data->GetExperiment().GetImageFileFormat();
            
            //had to add this to remap the indices, since this is broken before the SetNavigation call.
            ImageInfo temp_image_info = iff.GetImageInfoOfFirstFile(image_directory);
            m_controller.C_SetWindow(temp_image_info.window);		  
            
            if (!iff.DateIsIncluded())
            {
              SessionDatesDlg dlg;
              std::vector<Date> dates;
              dlg.SetMaxSessions(m_rfy_data->GetExperiment().GetMaxSessions());
              dlg.DoModal();
              dates = dlg.GetSessionDates();
              m_controller.C_SetSessionDates(dates);
            } else
            {
              // dates included in file, get dates from file name to store
              ImageInfo temp_info;
              std::vector<Date> dates;
              dates = iff.GetDatesFromDir(image_directory,m_rfy_data->GetExperiment().GetMaxSessions(),this);                
              m_controller.C_SetSessionDates(dates);
            }
            
            // set auto detect options
            m_controller.C_SetAutoDetectOptions(FALSE, FALSE, 0., FALSE, 0., FALSE, 0., TRUE);
            
            // get main image file name
            file_name = iff.GetImageFileName(image_directory,m_rfy_data->GetApplicationData().GetCurrentTubeName(),m_rfy_data->GetApplicationData().GetCurrentWindow(),m_rfy_data->GetApplicationData().GetCurrentSession());
            
            // clear undo/redo list
            m_controller.ClearUndoRedo();
            
            SetMode(MODE_NOIMAGEDISPLAYED);              
            
            m_controller.C_SetCurrentImage(file_name);
            m_controller.C_SetModify(false);  // unnecessary, but just in case
            
            ClearSelection();
            ReDraw(1,1);
          }
        } // ImageFileFormatDlg OK
        else return; // ImageFileFormatDlg Cancel
      } // If images in directory
      else str = "No Images Found. Choose Folder where Images are Located"; //AfxMessageBox("No image files found in directory.");
    } // ChooseImageDirectoryDlg OK
    else return; // ChooseImageDirectory Cancel
  } // while not valid directory
} 


/**
/    Creates autosave name from filename
/    CWK Spring10
/
*/
CString GetAutoSaveName(const CString& filename)
{
  // Add '~' to beginning of filename, after the last backslash, to signify autosaved file
  int last_slash = filename.ReverseFind('\\');
  
  CString autosave_name = filename;
  autosave_name.Insert(last_slash + 1, '~');
  
  return autosave_name;  
}

/**
/    Finds the next available archive_name and returns it
/    CWK Spring10
/
*/
CString GetArchiveName(const CString& filename)
{
  CString archive_name;
  CString fmt;
  
  FILE* archive = NULL;
  int filenum = 0;
  int index = 0;
  
  fmt = filename;
  
  // insert newly created archive substring into archive_name
  int last_slash = filename.ReverseFind('\\');
  fmt.Insert(last_slash + 1, ".#%03d-");
  
  // create archive file name starting at '.#0-' 
  // loops until the next available archive file is found
  do{	
    // create archive substring to insert into archive_name
    
    archive_name.Format(fmt, filenum);
    
    // try to open archive file to see if it exists
    // if exists, must increment and loop again
    archive = fopen(archive_name, "r");
    
    filenum++;
    
  } while(archive != NULL);
  
  return archive_name;
}


/**
/    Checks whether autosave file exists.
/    If yes, then user is prompted whether to use the autosave file instead.
/	       If yes, then a backup (archive) is made of the current file, and the current file is clobbered with the autosave file.
/        Either way, the autosave file is removed.
/    CWK Spring10
/
*/
void DoAutoSaveRecovery(const CString& filename)
{
  //check for autosave, ask if user would like to open autosave file to recover work
  CString autosave_name = GetAutoSaveName(filename);
  
  //try to open the autosaved file for reading to see if it exists
  FILE* autosave = NULL;
  autosave = fopen(autosave_name, "r");
  
  //if an autosave exists, ask the user if they would like to use it.  If so, delete old file, take '~' off
  //autosaved file, and open it.
  if(autosave)
  {
    fclose(autosave);
    
    CString archive_name = GetArchiveName(filename);
    
    DWORD ret = MessageBox(NULL,"A version of this file has been recovered by autosave.  Load autosaved file instead?", "Autosave", MB_YESNO);
    if(ret == IDYES)
    {
      // archive the current file just in case the user made a mistake;
      // this allows the user to manually recover the '.#' archive file
      rename(filename, archive_name);			// STDIO call
      
      // make the autosave file the current file
      rename(autosave_name, filename);
    }
    else
    {
      // archive the backup file just in case
      rename(autosave_name, archive_name);
    }
  }
}


/**  
/    @Action: Whenever user selects: File->Open
/             Creates an open file dialog
/     
*/
void CRootflyDlg::OnFileOpen() 
{
  CWaitCursor wait;
  g_debug_logfile.Print("=========================================\n");
  g_debug_logfile.Print("CRootflyDlg::OnFileOpen() 1\n");
  int i, j, k;
  
  //check to see if any modifications have been made to the file
  if (m_rfy_data->GetApplicationData().GetModify())
  {
    int return_value = AfxMessageBox("Project data has changed. Save changes?", MB_YESNOCANCEL);
    if (return_value == IDCANCEL)  return;
    else if (return_value == IDYES)
    {
      OnFileSave();  //call the save function if things have changed
    }
  }
  
  CFileDialog fdlg(TRUE,NULL,NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Rootfly File(*.rfy)|*.rfy|All Files(*.*)|*.*||",this);
  
  if (fdlg.DoModal() == IDOK)
  {
    CWaitCursor wait;
    
    CString filename = fdlg.GetPathName();
    
    // Check for valid .rfy extension
    if (filename.Right(4) != ".rfy")
    {
      filename += ".rfy";
      //AfxMessageBox("Error: File should be of extension RFY");
      //return;
    }
    
#ifdef USE_AUTOSAVE
    {
      DoAutoSaveRecovery(filename);
    }
#endif
    
    // Load file
    int status = m_controller.C_LoadFile(filename);
    if (status != ERRORS::RFY_LOADERROR_SUCCESS)
    {
      if (status == ERRORS::RFY_LOADERROR_VERSION)
        AfxMessageBox("Error! Invalid file version");
      else if (status == ERRORS::RFY_LOADERROR_FILECORRUPT)
        AfxMessageBox("Error! Loading corrupt file");
      else if (status == ERRORS::RFY_LOADERROR_NOFILEFOUND)
        AfxMessageBox("Error! Unable to open file");
      else if (status == ERRORS::RFY_LOADERROR_USERCANCELED)
      {}
      //SetMode(MODE_NOFILELOADED); // KDS (See bug 2962833)
      return;
    }   
    
    // clear undo/redo list
    m_controller.ClearUndoRedo();
    
    SetMode(MODE_IMAGEDISPLAYED);
    
    const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
    
    CString file_directory = m_rfy_data->GetExperiment().GetImageDir();
    
    // while no valid images were found, keeps asking user to specify new directory
    if (ValidateDirectory(&file_directory) == false) return;
    m_controller.C_SetDefaultFileDirectory( file_directory );
    
    // Set initial tube, window, and session info
    ImageInfo ii = iff.GetImageInfoOfFirstFile(file_directory);
    m_controller.C_SetExperiment(ii.expt);
    m_controller.C_SetTube(ii.tube);
    m_controller.C_SetWindow(ii.window);
    m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );
    m_controller.C_SetSession(ii.session);
    
    // If the current experiment is a converted old file, then
    // convert to the new format (i.e., image coordinates instead of client coordinates)
    if (m_rfy_data->GetExperiment().IsOldFile())
    {      
      CWaitCursor wait;
      m_controller.C_ConvertOldRoots(file_directory, iff, this);
    }      
    
    // Get the min/max window/sesion
    SetNavigation(file_directory);
    m_controller.C_SetSession(ii.session);
    
    // make sure to have dates from file
    bool included = false;
    vector<Date> temp_dates = m_rfy_data->GetExperiment().GetSessionDates();
    for (i = 0; i < temp_dates.size(); i++)
    {
      if (temp_dates.at(i).month != -1) included = true;
    }
    if (!included && iff.DateIsIncluded())
    {
      
      temp_dates = iff.GetDatesFromDir(m_rfy_data->GetExperiment().GetImageDir(), m_rfy_data->GetExperiment().GetMaxSessions(),this);
      m_controller.C_SetSessionDates(temp_dates);
    }
    
    // get main image file name
    filename = iff.GetImageFileName(file_directory,m_rfy_data->GetApplicationData().GetCurrentTubeName(),m_rfy_data->GetApplicationData().GetCurrentWindow(),m_rfy_data->GetApplicationData().GetCurrentSession());
    
    m_first_save = true;
    
    ///////////////////////////////////////////////
    // loads custom codes from file and adds them to the list
    const vector<Tube> *t1;   
    const vector<Root> *r1;
    const vector<RootInstance> *ri;
    CString temp;
    
    t1 = m_rfy_data->GetExperiment().GetAllTubes();
    if (!t1) return;
    for (i = 0; i < t1->size(); i++)
    {
      r1 = t1->at(0).GetAllRoots();
      if (!r1) return;
      for (j = 0; j < r1->size(); j++)
      {
        ri = r1->at(j).GetAllRootInstances();
        if (!ri) return;
        for (k = 0; k < ri->size(); k++)
        {
          
          SetCondition(ri->at(k).GetConditionCode());
        }
      }
    }
    ///////////////////////////////////////////////
    
    SetMode(MODE_IMAGEDISPLAYED);
    m_controller.C_SetCurrentImage(filename); 
    ClearSelection();
    ReDraw(1, 1);
  } 
}  
  
/**
/  
/    @Action: Whenever user selects: File->SaveAs
/             Creates new Save As dialog
/     
*/
void CRootflyDlg::OnFileSaveAs() 
{
  g_debug_logfile.Print("=========================================\n");
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 1\n");
  //Create new save as dialog that only allows *.rfy files to be saved
  CFileDialog fdlg(FALSE,"rfy",NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Rootfly Files(*.rfy)|*.rfy|All Files(*.*)|*.*||",this);
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 2\n");
  
  if (fdlg.DoModal() == IDOK)
  {
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 3\n");
    m_first_save = true;
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 4\n");
    CString filename = fdlg.GetPathName();
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 5\n");
    m_controller.C_SetFilename(filename);
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 6\n");
    m_controller.C_SaveFile(filename, true);      
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 7\n");
  } 
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 8\n");
}

/**
/  
/    @Action: Whenever user selects: File->Save    
/             Sends request for a save action.      
/        
*/
void CRootflyDlg::OnFileSave() 
{
  g_debug_logfile.Print("=========================================\n");
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 1\n");
  CString filename = m_rfy_data->GetApplicationData().GetFileName();
  
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 2\n");
  if (filename == "")    AfxMessageBox("Filename not specified");
  else if (!m_first_save)  OnFileSaveAs();
  else
  {
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 3\n");
    m_first_save = true;
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 4\n");
    m_controller.C_SaveFile(filename, true);
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 5\n");
  }
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 6\n");
  g_debug_logfile.Close();
}

void CRootflyDlg::OnDebugShowdebuglogfilelocation() 
{
  char a[1000];
  _getcwd(a, 1000);
  CString s = "Log file should be \r\n";
  s += a;
  s += "\\rootfly_logfile.txt";
  AfxMessageBox(s);
}

/**
* This is the export function for the entire program - NOTE: The condition codes are enabled if DISPLAY_CONDITION_CODE is defined below.
*/

void CRootflyDlg::OnFileExport() 
{
  g_debug_logfile.Print("=========================================\n");
  g_debug_logfile.Print("CRootflyDlg::OnFileExport\n");
  //This controls whether or not the export function will include a condition code for each root session.
  //Comment it out to disable the condition code.
  CFileDialog fdlg(FALSE,"csv",NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"CSV(*.csv)|*.csv|All Files(*.*)|*.*||",this);
  fdlg.m_ofn.lpstrTitle = "Export";
  if (fdlg.DoModal() == IDOK)
  {
    
    CString filename = fdlg.GetPathName();  
    m_controller.C_Export(filename);
  }
  g_debug_logfile.Close();
}
/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                               HELPER FUNCTIONS                                      //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

double CRootflyDlg::DotProduct(pt2d a, pt2d b)
{ 
  return a.x * b.x + a.y * b.y; 
} 

/**

  @brief This method calculates the minimum distance from a
  point to a line segment.
  INPUT:
  'a' and 'b':  the endpoints of the line segment
  'p':  the point
  OUTPUT:
  'c':  the point on the line segment closest to 'p'
  (either the intersection of the perpendicular line through 'p' with the line 
  segment, or one of the endpoints of the line segment if the intersection is
  outside the line segment.)
  RETURN:
  Returns the square of the distance from 'p' to the line segment.
  (
*/
double CRootflyDlg::DistPt2Seg(const pt2d& a, const pt2d& b, const pt2d& p, pt2d *c)
{                           
  pt2d d = b - a;
  pt2d m = p - a;
  double r = DotProduct(m,d) / DotProduct(d,d);   
  
  if      (r < 0.0)     *c = a;    
  else if (r > 1.0)     *c = b;    
  else                  *c = a + d * r;
  
  d = p - (*c);
  return DotProduct(d, d); 
}

// returns whether point (x,y) is inside circle defined by 'diam'
// radius is expanded by 'tolerance' 
bool CRootflyDlg::IsPtInsideDiameter(int x, int y, const RDiameter* diam, int tolerance)
{
  double dx = diam->r_point.x - x;
  double dy = diam->r_point.y - y;
  return sqrt( dx * dx + dy * dy ) < diam->radius + tolerance;
}

// This function searches through every root in the current window
// and finds the closest polyline measured by the perpendicular distance
// from the point (x,y) to each polyline segment.
// INPUT:
//   (x,y) point
// OUTPUT:
//   (temp->r_point.x, temp->r_point.y) is the point along the polyline (may be between two vertices)
//                      that is closest to (x,y)
//   temp->radius is the minimum distance
//   temp->r_point.window is the ID of the root
// NOTE:  This function does *NOT* return the distance to the polyline vertex!!!
void CRootflyDlg::FindNearestPoint(int x, int y, RDiameter *temp ) const
{
  const int threshold = 18;  // how close point has to be to hit
  pt2d a,b,p,c;  
  double dist, min_dist = 999999.9;   
  std::vector<Root>::const_iterator root_ptr;
  const RootInstance *ri;
  std::vector<RPoint>::const_iterator p1;
  std::vector<RPoint>::const_iterator p2;
  temp->radius = -1;
  
  const int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if (tube == NULL)   return;
  const std::vector<Root> *root = tube->GetAllRoots();
  if (root == NULL) return;
  for ( root_ptr = root->begin(); root_ptr != root->end(); root_ptr++ )
  {
    ri = root_ptr->GetRootInstance(m_point_session);
    if (!ri)  continue;  // no root instance, try next root
    p1 = ri->GetAllPoints()->begin();
    p2 = ri->GetAllPoints()->begin() + 1;   
    while( p2 != ri->GetAllPoints()->end())
    {     
      a.x = p1->x;
      a.y = p1->y;
      b.x = p2->x;
      b.y = p2->y;
      p.x = x;
      p.y = y + m_converter.GetImageSize().cy * (- root_ptr->GetPrimaryWindow() + window);
      dist = DistPt2Seg(a,b,p,&c);   
      if ( dist < min_dist )
      {
        min_dist = dist;
        temp->r_point.x = (int)(c.x);
        temp->r_point.y = (int)(c.y);
        if ( min_dist < threshold ) 
        {
          temp->radius = min_dist;
          temp->r_point.window = root_ptr->GetID();  // window contains ID !!!  
        }
      } 
      p1++;
      p2++;          
    }
  } 
}

// This function returns the root ID of the smallest root diameter that encircles (x,y).  
// Returns -1 if (x,y) is outside all the diameters.
int CRootflyDlg::FindNearestDiameter(int x, int y) const
{ 
  int selected = -1;
  if ( m_rfy_data->GetSettings().GetShowDiameters() )
  {
    const int threshold = 4;  // how close point has to be to hit
    double min_radius = 9999999.9;   
    std::vector<Root>::const_iterator root_ptr;
    const RootInstance *ri;
    int window = m_rfy_data->GetApplicationData().GetCurrentWindow();   
    
    const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());  
    if (tube == NULL)  return selected;
    const std::vector<Root> *root = tube->GetAllRoots();   
    
    if (root == NULL) return selected;
    for ( root_ptr = root->begin(); root_ptr != root->end(); root_ptr++ )
    {
      ri = root_ptr->GetRootInstance(m_point_session);
      if (ri == NULL)      continue;
      const RDiameter* rdia = ri->GetDiameter(0);
      // Adjusting y coordinate by number of windows the root's primary windows differs from the current selected window.
      bool inside = IsPtInsideDiameter(x, (y + m_converter.GetImageSize().cy * (- root_ptr->GetPrimaryWindow() + window)), rdia, threshold);
      if ( inside && rdia->radius < min_radius ) 
      { 
        min_radius = rdia->radius;
        selected = root_ptr->GetID();
      }  
    }
  }
  return selected;
}

// Given a point 'rpoint' (in image coordinates), returns the closest point on the
// The int* segment_start will be set to the starting point of the segment containing the diameter
// root instance 'r'.
// Note:  The function first converts to client coordinates to handle roots that span
// multiple windows, then it converts back to image coordinates before returning.
// used to be // RPoint CRootflyDlg::FindNearestPointOnRoot(const RPoint& rpoint, const RootInstance& r ) const
RPoint CRootflyDlg::FindNearestPointOnRootSeg(const RPoint& rpoint, const RootInstance& r, int* segment_start ) const
{
  RPoint out;
  double dist, min_dist = 99999.9;     
  const vector<RPoint> *line = r.GetAllPoints();
  assert( line->size() >= 2 );
  if (line->size() < 2)  return RPoint(-1, -1, -1);
  vector<RPoint>::const_iterator p1, p2;
  CPoint temp;
  
  p1 = line->begin();
  p2 = line->begin() + 1;
  *segment_start = 0;
  int count = 0;
  // for each line segment of the root instance, ...
  while ( p2 != line->end() )
  {
    // convert to client coordinates
    CPoint pp1 = m_converter.ImageToClient(*p1);
    CPoint pp2 = m_converter.ImageToClient(*p2);
    CPoint ppp = m_converter.ImageToClient(rpoint);
    
    // compute the distance to the segment, and store the minimum distance
    pt2d a,b,p,c; 
    a.x = pp1.x;
    a.y = pp1.y;
    b.x = pp2.x;
    b.y = pp2.y;
    p.x = ppp.x;
    p.y = ppp.y;
    dist = DistPt2Seg(a, b, p, &c);   
    if ( dist < min_dist )
    {
      min_dist = dist;
      temp.x = (int)(c.x);
      temp.y = (int)(c.y);
      *segment_start = count;
    }
    count++;
    p1++;
    p2++;
  } 
  
  // convert back to image coordinates
  return m_converter.ClientToImage(temp, rpoint.window);
}

//Wrapper for FindNearestPointOnRootSeg for people not interested in the segment
RPoint CRootflyDlg::FindNearestPointOnRoot(const RPoint& rpoint, const RootInstance& r ) const
{
  int dummy;
  RPoint point = FindNearestPointOnRootSeg( rpoint, r, &dummy); //call with NULL pointer
  point.window = rpoint.window;
  return point;
}

// Note:  This is the only function that actually computes whether the mouse is near a particular
// vertex of a root polyline. -- STB
int CRootflyDlg::GetSelectedIndex( CPoint pt ) const
{
  if (m_selected_root_id == -1)  return NOTHING_SELECTED;
  const RPoint *rpt;
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return NOTHING_SELECTED;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return NOTHING_SELECTED;
  const RootInstance *inst = root->GetRootInstance(m_point_session);
  //assert(inst); do not assert because point could be outside current selected session due to 3x3 -MTB
  if (!inst)  return NOTHING_SELECTED;
  const int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  // Adjusting y coordinate by number of windows the root's primary windows differs from the current selected window.
  pt.y += m_converter.GetImageSize().cy * (- root->GetPrimaryWindow() + window);
  CRect rect( pt.x-20, pt.y-20, pt.x+20, pt.y+20 );
  
  if ( m_rfy_data->GetSettings().GetShowDiameters() )
  { 
    CRect diameter_top;
    diameter_top.left = inst->GetDiameter(0)->r_point.x - 7;
    diameter_top.right =  inst->GetDiameter(0)->r_point.x + 7;
    diameter_top.top =  inst->GetDiameter(0)->r_point.y - inst->GetDiameter(0)->radius - 7;
    diameter_top.bottom =  inst->GetDiameter(0)->r_point.y - inst->GetDiameter(0)->radius + 7;
    if ( diameter_top.PtInRect(pt))
      return RESIZING_DIAMETER_SELECTED;
    
    const int threshold = 4;
    if ( IsPtInsideDiameter(pt.x, pt.y, inst->GetDiameter(0), threshold) )
    {
      return DIAMETER_SELECTED;
    }
  }  
  int size = inst->GetAllPoints()->size();
  for ( int i = 0; i < size; i++ )
  {
    rpt = inst->GetPoint(i);
    if ( rect.PtInRect(CPoint(rpt->x,rpt->y)))
      return i;
  }  
  return NOTHING_SELECTED;
}

// This function updates the starting point of the root segment the diameter is located in
// and the ratio of the distance from that point to the diameter over the total distance of that segment
// the data is stored in the variable section_start_point_index and diameter_position_ratio    
void CRootflyDlg::UpdateDiameterPositionRatio()
{
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_point_session);
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  
  //get first point of section of root with diameter
  FindNearestPointOnRootSeg( diam->r_point, *inst, &section_start_point_index);
  //get points of section
  const RPoint* start_point = inst->GetPoint(section_start_point_index);
  const RPoint* other_point = inst->GetPoint(section_start_point_index + 1);
  
  double dist_1_d = sqrt(pow(((float)start_point->x - diam->r_point.x),2) + pow(((float)start_point->y - diam->r_point.y),2));
  double dist_1_2 = sqrt(pow(((float)start_point->x - other_point->x),2) + pow(((float)start_point->y - other_point->y),2));
  if(dist_1_d != 0 && dist_1_2 != 0)  diameter_position_ratio = dist_1_d / dist_1_2;
  else diameter_position_ratio = .5;
}

/**
/        Enables/disable menu/buttons depending on mode set.         
/        User: Thai Nguyen (TVN)
*/
void CRootflyDlg::EnableDisable()
{
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  CMenu *menu = GetMenu();
  bool file_loaded     = (GetMode() != MODE_NOFILELOADED);
  bool root_selected   = (GetMode() == MODE_ROOTSELECTED);
  bool image_displayed = ((GetMode() != MODE_NOIMAGEDISPLAYED) && file_loaded);
  bool file_loaded_e   = file_loaded ? MF_ENABLED : MF_GRAYED;
  bool image_displayed_e =image_displayed ? MF_ENABLED : MF_GRAYED;
  bool root_selected_e = root_selected ? MF_ENABLED : MF_GRAYED;
  bool date_included   = iff.DateIsIncluded();
  bool tube_created = (m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName()) != NULL) ? true: false;
  bool show_roots_enabled = (m_checkbox_show_roots.GetCheck() == BST_CHECKED) ;
  
  // get tube, window, and session of current image
  int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int session = m_rfy_data->GetApplicationData().GetCurrentSession();
  const Tube *tube = (m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName()));
  
  //bool annotated = m_progress_data.GetDone(tubeNumber,window,session);
  bool annotated;
  if(tube == NULL)  annotated = false;  //SetMode(MODE_IMAGEDISPLAYED);
  else              annotated = tube->GetImageAnnotated(window, session);
  bool annotated_e = file_loaded && annotated ? MF_ENABLED : MF_GRAYED;
  bool not_annotated_e = image_displayed && !annotated ? MF_ENABLED : MF_GRAYED;
  
  
  // handle message commands
  // disable only when no file loaded
  menu->EnableMenuItem(ID_FILE_SAVEAS         , file_loaded_e);
  menu->EnableMenuItem(ID_FILE_SAVE           , file_loaded_e);
  menu->EnableMenuItem(ID_FILE_EXPORT         , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_CHANGE_IMG_DIR , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_IMAGEFILEFORMAT, file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_CONVERSION     , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_SESSION_DATES  , date_included);
  menu->EnableMenuItem(ID_EDIT_ZOOM_OUT       , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_ZOOM_IN        , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_UNDO , m_controller.isUndoVectorEmpty());
  menu->EnableMenuItem(ID_EDIT_REDO , m_controller.isRedoVectorEmpty());
  menu->EnableMenuItem(IDC_BUTTON_Annotate,     not_annotated_e);  
  menu->EnableMenuItem(ID_EDIT_COPYSESSION,     annotated_e);  
  menu->EnableMenuItem(ID_EDIT_AUTODETECTROOTS, annotated_e);  
  
  // Undo should not be allowed when drawing a new root - CWK Spring10
  if( GetMode() == MODE_DRAWINGNEWROOT )
     menu->EnableMenuItem(ID_EDIT_UNDO , true);
  
  // handle spin edit boxes
  // disable only when no file loaded
  m_edit_window.EnableWindow  (file_loaded);
  m_edit_tube.EnableWindow    (file_loaded);
  m_edit_session.EnableWindow (file_loaded);
  m_spin_window.EnableWindow  (file_loaded);
  m_spin_tube.EnableWindow    (file_loaded);
  m_spin_session.EnableWindow (file_loaded);
  m_spin_diameter.EnableWindow ((root_selected || GetMode() == MODE_RESIZINGDIAMETER));
  
  // set spin boxes to read-only only when no file loaded
  m_edit_window.SetReadOnly   (!file_loaded);
  m_edit_tube.SetReadOnly     (!file_loaded);
  m_edit_session.SetReadOnly  (!file_loaded); 
  
  // handle properties boxes
  // enable only when root is selected
  m_combo_color.EnableWindow  (root_selected);
  m_combo_state.EnableWindow  (root_selected);
  m_combo_custom.EnableWindow (root_selected); 
  m_edit_diameter.SetReadOnly (!(root_selected || GetMode() == MODE_RESIZINGDIAMETER));
  
  // handle buttons
  // disable all only when no file loaded
  // enable extend endpoint, delete root, add diameter
  //     only when root is selected  
  m_button_new_root.EnableWindow        (image_displayed && annotated);
  m_button_extend_endpoints.EnableWindow(root_selected && annotated);
  m_button_delete_root.EnableWindow     (root_selected && annotated);
  m_button_move_all_roots.EnableWindow  (image_displayed && annotated);
  //  m_button_copy.EnableWindow            (file_loaded && annotated);
  //  m_check_zoom_out.EnableWindow         (file_loaded);
  //  m_check_zoom_in.EnableWindow         (file_loaded);
  EnableDisableBitmaps();
  //  m_button_auto_detect.EnableWindow        (image_displayed);
  m_button_annotate.EnableWindow( image_displayed && !annotated);
  
  // menu items corresponding to buttons
  menu->EnableMenuItem(IDC_BUTTON_New_Root,         annotated_e);
  menu->EnableMenuItem(IDC_BUTTON_Extend_Endpoints, root_selected_e);
  menu->EnableMenuItem(IDC_BUTTON_Delete_Root,      root_selected_e);
  menu->EnableMenuItem(IDC_BUTTON_Move_All_Roots,   annotated_e);
  //  menu->EnableMenuItem(IDC_BUTTON_Copy_Session,     file_loaded_e);  
  
  // menu items (navigate)
  menu->EnableMenuItem(ID_KEYBOARD_UPARROW,         file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_DOWNARROW,       file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_LEFTARROW,       file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_RIGHTARROW,      file_loaded_e);  
  menu->EnableMenuItem(ID_NAVIGATE_PROGRESSWINDOW,  file_loaded_e); 
  
  //ANTHONY
  //menu->CheckMenuItem(ID_VIEW_RIGHTPANE,			TRUE);
  
  
  // check boxes
  
  bool checkbox_status;
  
  switch( GetMode() )
  {
  case MODE_IMAGEDISPLAYED:
  case MODE_ROOTSELECTED:
  case MODE_PANNINGIMAGE:   checkbox_status = true;
    break;
  default:                  checkbox_status = false;
    break;
  }
  
  m_checkbox_show_roots.EnableWindow( checkbox_status );
  m_checkbox_show_dead_roots.EnableWindow( checkbox_status && show_roots_enabled);
  m_checkbox_show_diameters.EnableWindow( checkbox_status && show_roots_enabled);
  m_checkbox_show_root_ids.EnableWindow( checkbox_status && show_roots_enabled);
}

// returns true if in zoomed state, false otherwise
bool CRootflyDlg::IsZoomOutMode() const
{
  if (m_hWnd)
  {
    // old:  using Edit->Zoom
    unsigned isCheckedOut = GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND); 
    if (isCheckedOut == MF_CHECKED)
      return 1;
    else
      return (m_check_zoom_out.GetCheck() != 0);
    
    //   return ((ret & MF_CHECKED) != 0);
    // new:  using check box on main dialog    
  }
  else
  {
    return 0;
  }
}

void CRootflyDlg::OnClose() 
{
  //check to see if any modifications have been made to the file
  int return_value;
  bool cancelled_saveas = FALSE;
  
  if (m_rfy_data->GetApplicationData().GetModify())
  {
    return_value = AfxMessageBox("Project data has changed. Save changes?", MB_YESNOCANCEL);
    if (return_value == IDCANCEL)  return;
    else if (return_value == IDYES)
    {
      OnFileSave();  //call the save function if things have changed
      
      if(m_rfy_data->GetApplicationData().GetModify()) //The user must have cancelled a SaveAs on new.rfy*
      {
        cancelled_saveas = TRUE;
      }
    }
  }
  
  if(!cancelled_saveas)
  {
    // save settings to registry
    m_controller.C_SaveUserPreferences();
    
    
#ifdef USE_AUTOSAVE
    // delete autosave file
    //remove(m_rfy_data->GetApplicationData().GetAutosaveName());
    remove(GetAutoSaveName(m_rfy_data->GetApplicationData().GetFileName()));
#endif
    
    CDialog::OnClose();
  }
}

// brings up the dialog for mm conversion
void CRootflyDlg::OnEditCalibrate() 
{
  MMConversionDlg convert;
  CString str;
  convert.SetConversion(m_rfy_data->GetExperiment().GetMMConversion());
  convert.SetLineColor(m_rfy_data->GetSettings().GetRootDrawingColorLiveSelected());
  
  int ret = convert.DoModal();
  if (ret == IDOK)
  {    
    float conversion = convert.GetConversion();
    m_controller.C_SetMMConversion(conversion);
    ReDraw(1, 0);
  }
}

// Move all roots (called 'Register' in version 1.0.3)
void CRootflyDlg::OnButtonMoveAllRoots() 
{
  //plan
  //change to mode picking start point
		//lbutton down, hold start point
		//change mode to moving all roots
  
  //on mouse move update all roots in window and redraw
  //on left click, change mode back
  
  SetMode(MODE_STARTINGMOVEALL);
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  
  ReDraw(1,0);
}

void CRootflyDlg::MyZoomOut() 
{
  if (GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND) == MF_CHECKED)
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
  else {
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_CHECKED);
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_IN, MF_UNCHECKED);
    m_check_zoom_in.SetCheck(BST_UNCHECKED);
  }
 
  CClientDC dc(this);
  m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
  SetZoomWindows();
  ReDraw(1, 0);	
}


// test function for zoom in checkbox
void CRootflyDlg::MyZoomIn() 
{
  if (GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_IN, MF_BYCOMMAND) == MF_CHECKED)
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_IN, MF_UNCHECKED);
  else {
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_IN, MF_CHECKED);
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
    m_check_zoom_out.SetCheck(BST_UNCHECKED);
  }

  CClientDC dc(this);
  m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
  SetZoomWindows();
  ReDraw(1, 0);
}



void CRootflyDlg::OnEditChangeImgDir() 
{
  // Let user choose folder that contains images for the new project
  UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
  CString image_directory;
  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  
  bool valid_directory = false;
  while(!valid_directory)
  {
    CFolderDialog dlg(_T("Choose Folder where Images are Located" ), image_directory, this, uFlags);
    int fdlgid = dlg.DoModal();
    if (fdlgid == IDOK)
    {   
      image_directory = dlg.GetFolderPath();
      image_directory += "\\";
      if (iff.ImageFileExistsInDirectory(image_directory))
        valid_directory = true;
      else
      {
        AfxMessageBox("No valid images were found in the directory");
      }
    }
    else return;
  }
  
  // We have valid directory. Let's save it.
  m_controller.C_SetImageDir(image_directory);
  m_controller.C_SetModify(true);
  
  // Set initial tube, window, and session info
  ImageInfo ii = iff.GetImageInfoOfFirstFile(image_directory);
  m_controller.C_SetExperiment(ii.expt);
  m_controller.C_SetTube(ii.tube);
  m_controller.C_SetWindow(ii.window);
  m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );
  m_controller.C_SetSession(ii.session);
  
  SetNavigation(image_directory);
  if (!iff.DateIsIncluded())
  {
    SessionDatesDlg dlg;
    std::vector<Date> dates;
    dlg.SetSessionDates(m_rfy_data->GetExperiment().GetSessionDates());
    int ret = dlg.DoModal();
    if (ret == IDOK)
    {
      dates = dlg.GetSessionDates();
      m_controller.C_SetSessionDates(dates);      
      m_controller.C_SetModify(true);
    }
  }
  
  // get main image file name
  CString file_name = iff.GetImageFileName(image_directory,m_rfy_data->GetApplicationData().GetCurrentTubeName(),m_rfy_data->GetApplicationData().GetCurrentWindow(),m_rfy_data->GetApplicationData().GetCurrentSession());
  
  SetMode(MODE_IMAGEDISPLAYED);
  
  m_controller.C_SetCurrentImage(file_name);
  
  ReDraw(1, 1);
}

void CRootflyDlg::OnEditColors() 
{
  CColorDlg colordlg(&m_controller, &m_rfy_data->GetSettings(), this);
  int id = colordlg.DoModal();
  if (id == IDOK)
  {
    ReDraw(1, 0);
  }
}


// display the help menu when F1 is pressed 
BOOL CRootflyDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
  OnHelpContents();
  return true;
}


//Will move up one window when up arrow is pressed
void CRootflyDlg::OnUpArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(-1,0);
  }
}

//Will move down one window when down arrow is pressed
void CRootflyDlg::OnDownArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(1,0);
  }
}

//Will move right one session when right arrow is pressed
void CRootflyDlg::OnRightArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(1,1);
  }
}

//Will move left one session when left arrow is pressed
void CRootflyDlg::OnLeftArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(-1,1);
  }
}

void CRootflyDlg::OnDeletePoint()
{ 
  m_controller.C_DeletePoint(m_selected_index, m_selected_root_id, m_selected_root_session);
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if ( !tube ) {AfxMessageBox("No tube");return;}
  const Root *root = tube->GetRoot(m_selected_root_id);
  if ( !root ) {AfxMessageBox("No root");return;}
  const RootInstance *inst = root->GetRootInstance(m_selected_root_session);
  if ( !inst ) {AfxMessageBox("No inst");return;}
  const RDiameter *diam = inst->GetDiameter(0);
  if ( !diam ) {AfxMessageBox("No diam");return;}
  
  RPoint temp = FindNearestPointOnRoot( diam->r_point, *inst );
  if ( temp.x < 0 || temp.y < 0) return;
  double dist = sqrt(pow(((float)temp.x - diam->r_point.x),2) + pow(((float)temp.y - diam->r_point.y),2));
  if ( dist > 1.0 )  //move the diameter if it is not attached to the root anymore
  {
    m_controller.C_MoveDiameter(0, m_selected_root_id, m_selected_root_session, temp);
  }
  
  SetMode(MODE_ROOTSELECTED);  
  ReDraw(1,0);
}

void CRootflyDlg::OnAddPoint()
{ 
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if ( !tube ) {AfxMessageBox("No tube");return;}
  const Root *root = tube->GetRoot(m_selected_root_id);
  if ( !root ) {AfxMessageBox("No root");return;}
  const RootInstance *inst = root->GetRootInstance(m_selected_root_session);
  if ( !inst ) {AfxMessageBox("No inst");return;}
  const RDiameter *diam = inst->GetDiameter(0);
  if ( !diam ) {AfxMessageBox("No diam");return;}
  
  RPoint *rpt = m_controller.C_CreateMidPoint(m_selected_root_id, m_selected_root_session, section_start_point_index);
  
  rpt->x = temporary_point_to_add.x;
  rpt->y = temporary_point_to_add.y;
  rpt->window = diam->r_point.window;
  
  //Set Mode to move new point
  m_selected_index = section_start_point_index + 1;
  
  // earlier, MODE_MOVINGRPOINT was called. this has been changed
  // to MODE_ADDPOINT to make it possible for the Undo functionality
  // to distinguish between the operations 'Add Point' and 'Move Point'
  SetMode(MODE_ADDPOINT); 
  
  UpdateDiameterPositionRatio();
  
  
  ReDraw(1,0);
}

void CRootflyDlg::OnDebugResetmode() 
{
  CString str;
  str.Format("Current mode: %d.  Please notify developer.", GetMode());
  AfxMessageBox(str);
  SetMode(MODE_IMAGEDISPLAYED);
  ClearSelection();
}

// This status text provides two purposes:  It informs the user about possible actions, and it provides
// information about the current 'mode' to aid debugging.  In order for the second purpose to be accomplished,
// be sure that the text is unique for each mode.
void CRootflyDlg::UpdateStatusText()
{
  CString str; 
  
  //STATUSBAR
  Status_bar.SetPaneText(0,"Started");
  
  ModeHandlerFactory mhf(this);
  CString str2 = mhf.GetModeHandler()->GetStatusText();
  str += _T( str2 );
  
  //m_status_text.SetWindowText(str);
  //STATUSBAR
  Status_bar.SetPaneText(0,str);
  CWnd* pApplyButton = GetDlgItem (IDC_STATIC_status);
  ASSERT (pApplyButton);
  pApplyButton->ShowWindow (SW_HIDE);
}

CDebugDlg* g_debug_dlg = NULL;
void CRootflyDlg::OnDebugDisplayrootdata() 
{
  // format data into a string
  CString str;
  str = "***** Roots in this tube *****\r\n";
  str += "i (Root ID) (displayable ID): [First session, last session] primary window\r\n";
  {
    const Experiment& exp = m_rfy_data->GetExperiment();   
    const Tube *tube = exp.GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
    if (tube == NULL) { AfxMessageBox("No root data to display");  return; }
    const vector<Root> *root = tube->GetAllRoots();
    if (root == NULL) { AfxMessageBox("No roots in this tube");  return; }
    CRect main; 
    CPoint conv_point;
    CString tmpstr;
    
    int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
    for ( int i = 0; i < root->size(); i++ )
    {
      const Root& rr = root->at(i);
      if ( 1 ) // rr.GetPrimaryWindow() == window)
      {
        tmpstr.Format("Root %3d (%3d) (%3d): [%3d,%3d] w=%3d\r\n", i, rr.GetID(), rr.GetDisplayableID(), rr.GetFirstSessionNumber(), rr.GetLastSessionNumber(), rr.GetPrimaryWindow());
        str += tmpstr;
        
        const std::vector<RootInstance>* rii = rr.GetAllRootInstances();
        for ( int j = 0 ; j < rii->size() ; j++)
        {
          const RootInstance& ri = rii->at(j);
          
          tmpstr.Format("  Session %3d:  %s ", ri.GetSessionNumber(), ri.IsAlive() ? "LIVE" : "DEAD");
          str += tmpstr;
          const std::vector<RPoint>* pp = ri.GetAllPoints();
          for ( int k = 0 ; k < pp->size() ; k++)
          {
            const RPoint& rp = pp->at(k);
            tmpstr.Format("(%3d,%3d,%3d) ", rp.x, rp.y, rp.window);
            str += tmpstr;
          }
          const RDiameter* rd = ri.GetDiameter(0);
          if (rd)  
          {
            tmpstr.Format(" dia=(%3d,%3d,%3d;%3.0f)", rd->r_point.x, rd->r_point.y, rd->r_point.window, rd->radius);
            str += tmpstr;
          }
          str += "\r\n";
        }
      }
    }
  }
  
  // display data
  if (g_debug_dlg == NULL)
  {
    g_debug_dlg = new CDebugDlg;
    g_debug_dlg->Create(CDebugDlg::IDD, this);
  }
  g_debug_dlg->SetText(str);
  g_debug_dlg->ShowWindow( SW_SHOW );
}

void CRootflyDlg::OnDebugRepairdiameter() 
{
  if (m_selected_root_id < 0)  return;
  
  // get selected root	
  CRect main;
  CPoint conv_point;
  vector<RPoint> extended_line;
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(m_rfy_data->GetApplicationData().GetCurrentTubeName());
  if (!tube)  return;
  const Root *root = tube->GetRoot( m_selected_root_id );
  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_rfy_data->GetApplicationData().GetCurrentSession());
  if (!inst)  return;
  const std::vector<RPoint> *line = inst->GetAllPoints();
  if (!line)  return;
  conv_point = m_converter.ImageToClient( CPoint( line->at(0).x, line->at(0).y), m_rfy_data->GetApplicationData().GetCurrentWindow() );
  const RDiameter *diam = inst->GetDiameter(0);
  if (!diam)  return;
  
  RDiameter* ddiam = const_cast<RDiameter*>( diam );
  ddiam->r_point.window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  
  m_controller.C_SetModify(true);
  ReDraw(1, 0);
}

void CRootflyDlg::OnNavigateProgresswindow() 
{
  FindAnnotatedRoots();
  
  ProgressDlg dlg(m_rfy_data->GetExperiment(), m_rfy_data->GetApplicationData().GetCurrentTubeName(), this);
  dlg.DoModal();  
  if(dlg.m_selected_window !=-1 && dlg.m_selected_tube != -1)       
    ProgressImageClicked(dlg.m_selected_window, dlg.m_selected_tube, dlg.m_selected_session);
}

void CRootflyDlg::OnEditAutodetect() 
{
  CAutoDetectDlg autodlg(&m_controller, &m_rfy_data->GetSettings(), this);
  autodlg.DoModal();
}

//this is helper method for the ProgressWindow Class
//it will find if a file exists and if it does it will set it as existing in the class
void CRootflyDlg::FindAnnotatedRoots()
{
//  const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
  int window_count = m_rfy_data->GetExperiment().GetMaxWindows();
  int session_count = m_rfy_data->GetExperiment().GetMaxSessions();
  int tube_count = m_rfy_data->GetExperiment().GetNTubes();
  CString fileName;
  
  int w, s;
  
  if (m_mode == MODE_NOFILELOADED)
    return;
  
  const std::vector<Tube> *tubes;
  const std::vector<Root> *roots;
  //  const Tube *tube;
  //  const Root *root;
  
  tubes = m_rfy_data->GetExperiment().GetAllTubes();
  /*
  for (tube = tubes->begin(); tube < tubes->end(); tube++)
  {
  roots = tube->GetAllRoots();
  for (root = roots->begin(); root < roots->end(); root++)
  {
  w = root->GetPrimaryWindow();
  s = root->GetFirstSessionNumber();
  
    if ((w > window_count) || (s > session_count))
    continue;
    
      m_progress_data.SetAnnotated(atoi(tube->GetTag()), w, s);
      }
}*/
  
  for(int i = 0; i < tubes->size(); i++)
  {
    roots = tubes->at(i).GetAllRoots();
    for(int j = 0; j < roots->size(); j++)
    {
      w = roots->at(j).GetPrimaryWindow();
      s = roots->at(j).GetFirstSessionNumber();
      
      if((w > window_count) || (s > session_count))
      {
        continue;
      }
      //m_progress_data.SetAnnotated(atoi(tubes->at(i).GetTag()), w, s);
    }
  }
  
}

// change the bitmaps on the zoom buttons to indicate different behavior
// (whether the button navigates to the next session, etc., or pans within the current zoomed-in image)
void CRootflyDlg::SetZoomBitmaps(bool which_bitmaps)
{
  if (which_bitmaps)
  { // set to zoomed in bitmaps
    m_button_upzoom.LoadBitmaps(IDB_UP_ZOOM, IDB_UP_ZOOM_DN, IDB_UP_ZOOM, IDB_UP_ZOOM_DIS);
    m_button_rightzoom.LoadBitmaps(IDB_RIGHT_ZOOM, IDB_RIGHT_ZOOM_DN, IDB_RIGHT_ZOOM, IDB_RIGHT_ZOOM_DIS);
    m_button_leftzoom.LoadBitmaps(IDB_LEFT_ZOOM, IDB_LEFT_ZOOM_DN, IDB_LEFT_ZOOM, IDB_LEFT_ZOOM_DIS);
    m_button_downzoom.LoadBitmaps(IDB_DOWN_ZOOM, IDB_DOWN_ZOOM_DN, IDB_DOWN_ZOOM, IDB_DOWN_ZOOM_DIS);
  }
  else
  { // set to regular bitmaps
    m_button_upzoom.LoadBitmaps(IDB_UP, IDB_UP_DN, IDB_UP, IDB_UP_DIS);
    m_button_rightzoom.LoadBitmaps(IDB_RIGHT, IDB_RIGHT_DN, IDB_RIGHT, IDB_RIGHT_DIS);
    m_button_leftzoom.LoadBitmaps(IDB_LEFT, IDB_LEFT_DN, IDB_LEFT, IDB_LEFT_DIS);
    m_button_downzoom.LoadBitmaps(IDB_DOWN, IDB_DOWN_DN, IDB_DOWN, IDB_DOWN_DIS);
  }
  m_button_upzoom.Invalidate();
  m_button_rightzoom.Invalidate();
  m_button_leftzoom.Invalidate();
  m_button_downzoom.Invalidate();
}

void CRootflyDlg::EnableDisableBitmaps()
{
  bool disable_right;
  bool disable_left;
  bool disable_up;
  bool disable_down;
  bool file_loaded     = (GetMode() != MODE_NOFILELOADED);
  //bool image_displayed = ((GetMode() != MODE_NOIMAGEDISPLAYED) && file_loaded);

  if(m_check_zoom_out.GetCheck() == BST_CHECKED) m_button_zoomout.EnableWindow(false);
  else m_button_zoomout.EnableWindow(true);

  if(m_zoomin > 4) m_button_zoomin.EnableWindow(false);
  else m_button_zoomin.EnableWindow(true);

  if(m_zoomin == 1)
  {
    if(m_rfy_data->GetApplicationData().GetCurrentSession() == m_rfy_data->GetExperiment().GetMaxSessions()) disable_right = true;
    else disable_right = false;
    if(m_rfy_data->GetApplicationData().GetCurrentSession() == 1) disable_left = true;
    else disable_left = false;
    if(m_rfy_data->GetApplicationData().GetCurrentWindow() == 1) disable_up = true;
    else disable_up = false;
    if(m_rfy_data->GetApplicationData().GetCurrentWindow() == m_rfy_data->GetExperiment().GetMaxWindows()) disable_down = true;
    else disable_down = false;
  }  
  else
  {
    disable_right = false;
    disable_up = false;
    disable_down = false;
    disable_left = false;
  }

  if(file_loaded)
  {
    if(disable_right) m_button_rightzoom.EnableWindow(false);
    else m_button_rightzoom.EnableWindow(true);
    if(disable_up) m_button_upzoom.EnableWindow(false);
    else m_button_upzoom.EnableWindow(true);
    if(disable_left) m_button_leftzoom.EnableWindow(false);
    else m_button_leftzoom.EnableWindow(true);
    if(disable_down) m_button_downzoom.EnableWindow(false);
    else m_button_downzoom.EnableWindow(true);
  }
  else
  {
    m_button_rightzoom.EnableWindow(false);
    m_button_leftzoom.EnableWindow(false);
    m_button_downzoom.EnableWindow(false);
    m_button_upzoom.EnableWindow(false);
  }
}

void CRootflyDlg::OnEditZoomOut() 
{
/*// TODO: Add your command handler code here

  if(GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND) == MF_CHECKED)
  {
		GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
    m_check_zoom_out.SetCheck(BST_UNCHECKED);
    }// if
    else
    {
    GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_CHECKED);
    m_check_zoom_out.SetCheck(BST_CHECKED);
    m_check_zoom_in.SetCheck(BST_UNCHECKED);
    }// else
    
      CClientDC dc(this);
      m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
      SetZoomWindows();
  ReDraw(1, 0);*/	
  if (m_zoomin - m_zoomin * .2 >= 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // decrease zoom level of image
    m_zoomin -= m_zoomin * .2;
    m_shiftx -= m_shiftx * .2;
    m_shifty -= m_shifty * .2;
    ReDraw(1, 0);
  }
  else if (m_zoomin != 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // display image at regular resolution
    m_zoomin = 1;
    m_shiftx = 0;
    m_shifty = 0;
    ReDraw(1, 0);
    SetZoomBitmaps(false);
  }
  else if (m_zoomin == 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // entering 3x3 display mode
    m_check_zoom_out.SetCheck(BST_CHECKED);
    MyZoomOut();
    ReDraw(1, 0);
  }
}

void CRootflyDlg::OnEditZoomIn()
{
/*if (GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_IN, MF_BYCOMMAND) == MF_CHECKED)
GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_IN, MF_UNCHECKED);
else {
GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_IN, MF_CHECKED);
GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
m_check_zoom_out.SetCheck(BST_UNCHECKED);
}
CClientDC dc(this);
m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
SetZoomWindows();
  ReDraw(1, 0);*/
  
  if (m_zoomin <= 4 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // zoom in on image, or increase zoom level of image
    double old_zoomin = m_zoomin; 
    m_zoomin += m_zoomin * .2;
    m_shiftx += m_shiftx * .2;
    m_shifty += m_shifty * .2;
    if (old_zoomin == 1.0 && m_zoomin > 1.0)  SetZoomBitmaps(true);
    ReDraw(1, 0);
  }
  else if (m_zoomin == 1) 
  { // display image at regular resolution
    m_check_zoom_in.SetCheck(BST_CHECKED);
    MyZoomIn();
    ReDraw(1, 0);
  }
}

void CRootflyDlg::CheckForCommandLine() 
{
  
  LPWSTR *szArglist;
  int nArgs, i, j, k;
  CString fileToOpen;
  
  // Parse Command Line Arguments Into An Array
  
  szArglist = CommandLineToArgvW(GetCommandLineW(), &nArgs);
  
  if (nArgs >= 2) 
  {
    fileToOpen.Format("%ws",szArglist[1]);  // the 'w' causes an ASSERT to fire, even though it works -- STB 4/20/10
  }
  
  // Check The Second Argument For Valid File Extension
  
  if (strnicmp(fileToOpen.Right(4), ".rfy", 4) == 0) 
  {
    
    // If Extension = .rfy Then Attempt To Load File
    int status = m_controller.C_LoadFile(fileToOpen);
    if (status != ERRORS::RFY_LOADERROR_SUCCESS)
    {
      CString str;
      if (status == ERRORS::RFY_LOADERROR_VERSION)
      {
        str.Format("Error! Invalid file version in '%s'", (const char*) fileToOpen);
        AfxMessageBox(str);
      }
      else if (status == ERRORS::RFY_LOADERROR_FILECORRUPT)
      {
        str.Format("Error!  File '%s' is corrupt and cannot be loaded", (const char*) fileToOpen);
        AfxMessageBox(str);
      }
      else if (status == ERRORS::RFY_LOADERROR_NOFILEFOUND)
      {
        str.Format("Error!  Unable to open file '%s'", (const char*) fileToOpen);
        AfxMessageBox(str);
      }
      else if (status == ERRORS::RFY_LOADERROR_USERCANCELED)
      {
      }
      SetMode(MODE_NOFILELOADED);
      return;
    }    
    
    // for batch conversion of files before 1.8.25
    //      {
    //        if (nArgs == 3)
    //        {
    //          AfxMessageBox("3");
    //          CString newname = fileToOpen;
    ////          newname.Format("%ws",szArglist[2]);
    //          newname.Insert( newname.GetLength()-4, "_new");
    //          m_controller.C_SaveFile(newname);
    //          exit(1);
    //        }
    //      }
    
    SetMode(MODE_IMAGEDISPLAYED);
    
    const ImageFileFormat& iff = m_rfy_data->GetExperiment().GetImageFileFormat();
    
    CString file_directory = m_rfy_data->GetExperiment().GetImageDir();
    
    // while no valid images were found, keeps asking user to specify new directory
    if (ValidateDirectory(&file_directory) == false) return;
    
    // Set initial tube, window, and session info
    ImageInfo ii = iff.GetImageInfoOfFirstFile(file_directory);
    m_controller.C_SetExperiment(ii.expt);
    m_controller.C_SetTube(ii.tube);
    m_controller.C_SetWindow(ii.window);
    m_converter.SetWindow( m_rfy_data->GetApplicationData().GetCurrentWindow() );
    m_controller.C_SetSession(ii.session);
    
    // If the current experiment is a converted old file, then
    // convert to the new format (i.e., image coordinates instead of client coordinates)
    if (m_rfy_data->GetExperiment().IsOldFile())
    {      
      CWaitCursor wait;
      m_controller.C_ConvertOldRoots(file_directory, iff, this);
    }      
    
    // Get the min/max window/sesion
    SetNavigation(file_directory);
    
    // make sure to have dates from file
    bool included = false;
    vector<Date> temp_dates = m_rfy_data->GetExperiment().GetSessionDates();
    for (i = 0; i < temp_dates.size(); i++)
    {
      if (temp_dates.at(i).month != -1) included = true;
    }
    if (!included && iff.DateIsIncluded())
    {
      
      temp_dates = iff.GetDatesFromDir(m_rfy_data->GetExperiment().GetImageDir(), m_rfy_data->GetExperiment().GetMaxSessions(),this);
      m_controller.C_SetSessionDates(temp_dates);
    }
    
    // get main image file name
    fileToOpen = iff.GetImageFileName(file_directory,m_rfy_data->GetApplicationData().GetCurrentTubeName(),m_rfy_data->GetApplicationData().GetCurrentWindow(),m_rfy_data->GetApplicationData().GetCurrentSession());
    
    m_first_save = true;
    
    ///////////////////////////////////////////////
    // loads custom codes from file and adds them to the list
    const vector<Tube> *t1;   
    const vector<Root> *r1;
    const vector<RootInstance> *ri;
    CString temp;
    
    t1 = m_rfy_data->GetExperiment().GetAllTubes();
    if (!t1) return;
    for (i = 0; i < t1->size(); i++)
    {
      r1 = t1->at(0).GetAllRoots();
      if (!r1) return;
      for (j = 0; j < r1->size(); j++)
      {
        ri = r1->at(j).GetAllRootInstances();
        if (!ri) return;
        for (k = 0; k < ri->size(); k++)
        {
          
          SetCondition(ri->at(k).GetConditionCode());
        }
      }
    }
    ///////////////////////////////////////////////
    
    SetMode(MODE_IMAGEDISPLAYED);
    m_controller.C_SetCurrentImage(fileToOpen); 
    ClearSelection();
    ReDraw(1, 1);
    
  }
  
  LocalFree(szArglist);
}

void CRootflyDlg::OnEditUndo() 
{
  m_controller.Undo();
  ReDraw(1, 0);
  EnableDisable();
  
  // Because the MODE_IMAGEDISPLAYED now has two different Status Texts it can 
  // return, this line was added in order to print the correct status text depending
  // on whether displayed image is annotated or not.  In the future, perhaps consider
  // adding another mode in order to stay consistent with the precedent that
  // only one status text exists for each mode (ie: MODE_IMAGEDISPLAYEDNOTANNOTATED and
  // MODE_IMAGEDISPLAYEDANNOTATED
  
  // This was added to enable undo creating a new root while it was selected. It could possibly
  // lead to other errors; sufficient testing has not been conducted.
  if (GetMode() == MODE_ROOTSELECTED)
    SetMode(MODE_IMAGEDISPLAYED);
  else
    
    UpdateStatusText();
}

void CRootflyDlg::OnEditRedo() 
{
  m_controller.Redo();
  ReDraw(1, 0);	
  EnableDisable();
  UpdateStatusText();
}

//This function toggles the check mark next to the Right Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewRightpane(CCmdUI* pCmdUI) 
{
  CMenu* mmenu = GetMenu();
  UINT state = mmenu->GetMenuState(ID_VIEW_RIGHTPANE, MF_UNCHECKED);
  
  if(state == MF_UNCHECKED){
    mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_CHECKED);
    
  }
  else{
    mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_UNCHECKED);
  }
  
  RECT m_windowSize;
  GetClientRect(&m_windowSize);
  OnSizeNew(7,m_windowSize.right, m_windowSize.bottom);
  
}

//when the window is redrawn, this fucntion will be called if the 
// Right Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the right pane
void CRootflyDlg::Rootfly_Right_Objects_Hide()
{
  //AfxMessageBox("HIDE OBJECTS");
  //CMenu* mmenu = GetMenu();
  //UINT state = mmenu->GetMenuState(ID_VIEW_RIGHTPANE, MF_UNCHECKED);
  
  CWnd* pApplyButton = GetDlgItem (IDC_STATIC_Properties_Section);
  ASSERT (pApplyButton);
  
  
		//mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_UNCHECKED);
		//AfxMessageBox("overview status now unchecked:HIDE");
  pApplyButton->ShowWindow (SW_HIDE);
  
		pApplyButton = GetDlgItem (IDC_STATIC_Tube);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Tube);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_TUBE);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_SPIN_TUBE);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Window);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_WINDOW);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_SPIN_WINDOW);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Session);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_SESSION);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    ///
    pApplyButton = GetDlgItem (IDC_SPIN_SESSION);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Root_ID);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_ROOTID);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);

    pApplyButton = GetDlgItem (IDC_SPIN_DIAMETER);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Diameter);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_DIAMETER);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Length);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_EDIT_LENGTH);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Color);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    ///
    
    pApplyButton = GetDlgItem (IDC_COMBO_COLOR);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_State);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_COMBO_STATE);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_STATIC_Custom);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_COMBO_CUSTOM);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_CHECK_Show_Root_IDs);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_CHECK_Show_Dead_Roots);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_CHECK_Show_Roots);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_CHECK_Show_Diameters);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_New_Root);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    ////
    
    pApplyButton = GetDlgItem (IDC_BUTTON_Extend_Endpoints);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_Delete_Root);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_Move_All_Roots);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    //		pApplyButton = GetDlgItem (IDC_BUTTON_Copy_Session);
    //		ASSERT (pApplyButton);
    //		pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_UPZOOM);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_DOWNZOOM);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_RIGHTZOOM);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_LEFTZOOM);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_ZoomOut);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    pApplyButton = GetDlgItem (IDC_BUTTON_ZoomIn);
    ASSERT (pApplyButton);
    pApplyButton->ShowWindow (SW_HIDE);
    
    
}


/*void CRootflyDlg::OnBUTTONAutoDetect() 
{
RunAutomaticRootDetector(1);	
ReDraw(1,0);	
}*/

//This function toggles the check mark next to the Left Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewLeftpane(CCmdUI* pCmdUI) 
{
  if(GetMenu()->GetMenuState(ID_VIEW_LEFTPANE, MF_BYCOMMAND) == MF_CHECKED){
    GetMenu()->CheckMenuItem(ID_VIEW_LEFTPANE, MF_UNCHECKED);
  }
  else{
    GetMenu()->CheckMenuItem(ID_VIEW_LEFTPANE, MF_CHECKED);
  }
  
  RECT m_windowSize;
  GetClientRect(&m_windowSize);
  OnSizeNew(8,m_windowSize.right, m_windowSize.bottom);
  
}

//This function toggles the check mark next to the Bottom Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewBottompane(CCmdUI* pCmdUI) 
{
  if(GetMenu()->GetMenuState(ID_VIEW_BOTTOMPANE, MF_BYCOMMAND) == MF_CHECKED){
    GetMenu()->CheckMenuItem(ID_VIEW_BOTTOMPANE, MF_UNCHECKED);
  }
  else{
    GetMenu()->CheckMenuItem(ID_VIEW_BOTTOMPANE, MF_CHECKED);
  }
  
  RECT m_windowSize;
  GetClientRect(&m_windowSize);
  OnSizeNew(9,m_windowSize.right, m_windowSize.bottom);
}


//when the window is redrawn, this fucntion will be called if the 
// Left Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the left pane
void CRootflyDlg::Rootfly_Left_Objects_Hide(){
  GetDlgItem(IDC_SCROLLBAR2)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_WIND_PIC_1)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_WIND_PIC_2)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_WIND_PIC_3)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_WIND_PIC_4)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_WIND_PIC_5)->ShowWindow (SW_HIDE);
  //	GetDlgItem(IDC_BUTTON_Auto_Detect)->ShowWindow (SW_HIDE);
  
  //TODO: figure out why the annotate button is always hidden if next line is uncommented
  //	GetDlgItem(IDC_BUTTON_Annotate)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_STATIC_Window_Section)->ShowWindow (SW_HIDE);
  //  GetDlgItem(IDC_CHECK_Done_Annotating)->ShowWindow (SW_HIDE);
}


//when the window is redrawn, this fucntion will be called if the 
// Bottom Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the bottom pane
void CRootflyDlg::Rootfly_Bottom_Objects_Hide(){
  GetDlgItem(IDC_SCROLLBAR3)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESS_PIC_1)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESS_PIC_2)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESS_PIC_3)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESS_PIC_4)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESS_PIC_5)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESSION_DATE_1)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESSION_DATE_2)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESSION_DATE_3)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESSION_DATE_4)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_SESSION_DATE_5)->ShowWindow (SW_HIDE);
  GetDlgItem(IDC_STATIC_Session_Section)->ShowWindow (SW_HIDE);
  
}


//this the check button to say that an image is done being annotated.
/*void CRootflyDlg::OnCHECKDoneAnnotating() 
{
std::string tubeString (m_rfy_data->GetApplicationData().GetCurrentTubeName());     
int tubeNumber;	  
std::stringstream ss(tubeString);
ss >> tubeNumber;
//const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName( m_rfy_data->GetApplicationData().GetCurrentTubeName() );  // current tube

  int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int session = m_rfy_data->GetApplicationData().GetCurrentSession();
  if(m_done_annotating.GetCheck()==BST_CHECKED)
  {
  /*
  std::string tubeString (m_rfy_data->GetApplicationData().GetCurrentTubeName());     
  int tubeNumber;	  
  std::stringstream ss(tubeString);
  ss >> tubeNumber;
  *
  
    m_progress_data.SetDone(tubeNumber,window,session);      
    m_controller.C_SetImageAnnotated(true);
    }
    else{
    m_progress_data.SetUnDone(tubeNumber,window,session);          
    m_controller.C_SetImageAnnotated(false);
    }
    
}*/

// Keybindings for navigation while zoomed in
void CRootflyDlg::OnKeyboardLeftzoom() 
{
 	m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_session = m_rfy_data->GetApplicationData().GetCurrentSession();
  int min_session = m_rfy_data->GetExperiment().GetMinSessions();
  int max_session = m_rfy_data->GetExperiment().GetMaxSessions();
  int new_session = curr_session - 1;
  
  if (m_zoomin != 1) 
  {
    m_shiftx -= 50 * (int)(m_zoomin + 1);
    SetZoomWindows();
    ReDraw(1, 0);
  }
  else
  {
    if (new_session < 1) 
    {
      temp.Format("%d", curr_session);
      m_edit_session.SetWindowText(temp);
      return;
    }
    if (new_session != curr_session)
    {
      // user enters value bigger than range. set new range
      if (new_session > max_session) 
      {
        m_controller.C_SetMaxSessions(new_session); 
        m_spin_session.SetRange(min_session, new_session);
        m_scrollbar_horizontal.SetScrollRange(min_session, new_session);
      }
      // user enters value smaller than range. set new range
      if (new_session < min_session) 
      {
        m_controller.C_SetMinSessions(new_session); 
        m_spin_session.SetRange(new_session, max_session);
        m_scrollbar_horizontal.SetScrollRange(new_session, max_session);
      }
      m_scrollbar_horizontal.SetScrollPos(new_session);  
      m_controller.C_SetSession(new_session);
      temp.Format("%d", new_session);
      m_edit_session.SetWindowText(temp);
      ReDraw(1, 1);   
    }
    
    // reset the Mode so the text in the status bar updates correctly
    if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  }
}

void CRootflyDlg::OnKeyboardDownzoom() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int min_window = m_rfy_data->GetExperiment().GetMinWindows();
  int max_window = m_rfy_data->GetExperiment().GetMaxWindows();
  int new_window = curr_window + 1;	
  
  if (m_zoomin != 1) 
  {
    m_shifty += 50 * (int)(m_zoomin + 1);
    SetZoomWindows();
    ReDraw(1, 0);
  }
  else
  {
   	if (new_window < 1) 
    {
      temp.Format("%d", curr_window);
      m_edit_window.SetWindowText(temp);
      return;  
    }
    if (new_window != curr_window)
    {
      // user enters value bigger than range. set new range
      if (new_window > max_window) 
      {
        m_controller.C_SetMaxWindows(new_window); 
        m_spin_window.SetRange(min_window, new_window);
        m_scrollbar_vertical.SetScrollRange(min_window, new_window);
      }
      // user enters value bigger than range. set new range
      if (new_window < min_window) 
      {
        m_controller.C_SetMinWindows(new_window); 
        m_spin_window.SetRange(new_window, max_window);
        m_scrollbar_vertical.SetScrollRange(new_window, max_window);
      }
      m_scrollbar_vertical.SetScrollPos(new_window);
      m_controller.C_SetWindow(new_window);
      m_converter.SetWindow( new_window );
      temp.Format("%d", new_window);
      m_edit_window.SetWindowText(temp);
      ReDraw(1, 1);
    }  
    
    // reset the Mode so the text in the status bar updates correctly
    if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  }
}

void CRootflyDlg::OnKeyboardUpzoom() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int min_window = m_rfy_data->GetExperiment().GetMinWindows();
  int max_window = m_rfy_data->GetExperiment().GetMaxWindows();
  int new_window = curr_window - 1;	
  
  if (m_zoomin != 1) 
  {
    m_shifty -= 50 * (int)(m_zoomin + 1);
    SetZoomWindows();
    ReDraw(1, 0);
  }
  else
  {
   	if (new_window < 1) 
    {
      temp.Format("%d", curr_window);
      m_edit_window.SetWindowText(temp);
      return;  
    }
    if (new_window != curr_window)
    {
      // user enters value bigger than range. set new range
      if (new_window > max_window) 
      {
        m_controller.C_SetMaxWindows(new_window); 
        m_spin_window.SetRange(min_window, new_window);
        m_scrollbar_vertical.SetScrollRange(min_window, new_window);
      }
      // user enters value bigger than range. set new range
      if (new_window < min_window) 
      {
        m_controller.C_SetMinWindows(new_window); 
        m_spin_window.SetRange(new_window, max_window);
        m_scrollbar_vertical.SetScrollRange(new_window, max_window);
      }
      m_scrollbar_vertical.SetScrollPos(new_window);
      m_controller.C_SetWindow(new_window);
      m_converter.SetWindow( new_window );
      temp.Format("%d", new_window);
      m_edit_window.SetWindowText(temp);
      ReDraw(1, 1);
    }  
    
    // reset the Mode so the text in the status bar updates correctly
    if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  }
}

void CRootflyDlg::OnKeyboardRightzoom() 
{
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  CString temp;
  int curr_session = m_rfy_data->GetApplicationData().GetCurrentSession();
  int min_session = m_rfy_data->GetExperiment().GetMinSessions();
  int max_session = m_rfy_data->GetExperiment().GetMaxSessions();
  int new_session = curr_session + 1;
  
  if (m_zoomin != 1) 
  {
    m_shiftx += 50 * (int)(m_zoomin + 1);
    SetZoomWindows();
    ReDraw(1, 0);
  }
  else
  {
    if (new_session < 1) 
    {
      temp.Format("%d", curr_session);
      m_edit_session.SetWindowText(temp);
      return;
    }
    if (new_session != curr_session)
    {
      // user enters value bigger than range. set new range
      if (new_session > max_session) 
      {
        m_controller.C_SetMaxSessions(new_session); 
        m_spin_session.SetRange(min_session, new_session);
        m_scrollbar_horizontal.SetScrollRange(min_session, new_session);
      }
      // user enters value smaller than range. set new range
      if (new_session < min_session) 
      {
        m_controller.C_SetMinSessions(new_session); 
        m_spin_session.SetRange(new_session, max_session);
        m_scrollbar_horizontal.SetScrollRange(new_session, max_session);
      }
      m_scrollbar_horizontal.SetScrollPos(new_session);  
      m_controller.C_SetSession(new_session);
      temp.Format("%d", new_session);
      m_edit_session.SetWindowText(temp);
      ReDraw(1, 1);   
    }
    
    // reset the Mode so the text in the status bar updates correctly
    if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  }
}

// If zoomed in, will zoom in/out
BOOL CRootflyDlg::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
  if (zDelta > 0 && m_zoomin <= 4 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // zoom in on image, or zoom in further on image
    double old_zoomin = m_zoomin; 
    m_zoomin += m_zoomin * .2;
    m_shiftx += m_shiftx * .2;
    m_shifty += m_shifty * .2;
    if (old_zoomin == 1.0 && m_zoomin > 1.0)  SetZoomBitmaps(true);
  }
  else if (zDelta < 0 && m_zoomin - m_zoomin * .2 >= 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // reduce zoom level of image
    m_zoomin -= m_zoomin * .2;
    m_shiftx -= m_shiftx * .2;
    m_shifty -= m_shifty * .2;
  }
  else if (zDelta < 0 && m_zoomin != 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // display image at regular resolution (instead of being zoomed in on image)
    m_zoomin = 1;
    m_shiftx = 0;
    m_shifty = 0;
    SetZoomBitmaps(false);
  }
  else if (zDelta < 0 && m_zoomin == 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // entering 3x3 display mode
    m_check_zoom_out.SetCheck(BST_CHECKED);
    MyZoomOut();
  }
  else if (zDelta > 0 && m_zoomin == 1)
  { // display image at regular resolution (instead of 3x3 display)
    m_check_zoom_in.SetCheck(BST_CHECKED);
    MyZoomIn();
  }
  SetZoomWindows();
  ReDraw(1,0);
  
  return CDialog::OnMouseWheel(nFlags, zDelta, pt);
}

// Bind the buttons to the key commands (instead of redoing the code)
void CRootflyDlg::OnButtonUpzoom() 
{
  OnKeyboardUpzoom();
}

void CRootflyDlg::OnButtonLeftzoom() 
{
  OnKeyboardLeftzoom();
}

void CRootflyDlg::OnButtonRightzoom() 
{
  OnKeyboardRightzoom();
}

void CRootflyDlg::OnButtonDownzoom() 
{
  OnKeyboardDownzoom();
}

// Variable ZoomIn and ZoomOut buttons
void CRootflyDlg::OnBUTTONZoomOut() 
{
  if (m_zoomin - m_zoomin * .2 >= 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // decrease zoom level of image
    m_zoomin -= m_zoomin * .2;
    m_shiftx -= m_shiftx * .2;
    m_shifty -= m_shifty * .2;
    ReDraw(1, 0);
  }
  else if (m_zoomin != 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // display image at regular resolution
    m_zoomin = 1;
    m_shiftx = 0;
    m_shifty = 0;
    ReDraw(1, 0);
    SetZoomBitmaps(false);
  }
  else if (m_zoomin == 1 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // entering 3x3 display mode
    m_check_zoom_out.SetCheck(BST_CHECKED);
    MyZoomOut();
    ReDraw(1, 0);
  }
  SetZoomWindows();
  ReDraw(1,0);
}

void CRootflyDlg::OnBUTTONZoomIn() 
{
  if (m_zoomin <= 4 && m_check_zoom_in.GetCheck() == BST_CHECKED) 
  { // zoom in on image, or increase zoom level of image
    double old_zoomin = m_zoomin; 
    m_zoomin += m_zoomin * .2;
    m_shiftx += m_shiftx * .2;
    m_shifty += m_shifty * .2;
    if (old_zoomin == 1.0 && m_zoomin > 1.0)  SetZoomBitmaps(true);
    //ReDraw(1, 0);
  }
  else if (m_zoomin == 1) 
  { // display image at regular resolution
    m_check_zoom_in.SetCheck(BST_CHECKED);
    MyZoomIn();
    //ReDraw(1, 0);
  }
  SetZoomWindows();
  ReDraw(1,0);
}

// Open annotate dialog so user can choose to autocopy and autodetect.  Then mark
// image as annotated if the dialog was close by clicking "Ok".
void CRootflyDlg::OnBUTTONAnnotate() 
{
  int i;
  
  // These static variables are used to save the user's last selections in the
  // Annotate dialog box and are initialized below with their default values
  static BOOL blnAutoCopy = TRUE;
  static BOOL blnAutoDetect = FALSE;
  static BOOL blnRememberSession = FALSE;
  
  std::string tubeString (m_rfy_data->GetApplicationData().GetCurrentTubeName());
  std::stringstream ss(tubeString);
  
  // get tube, window, and session of current image
  int result;
  static int remember_session = 0;
  int tubeNumber;
  int window = m_rfy_data->GetApplicationData().GetCurrentWindow();
  int session = m_rfy_data->GetApplicationData().GetCurrentSession();
  ss >> tubeNumber;
  
  //Create annotate dialog
  CAnnotateDlg dlgAnnotate(NULL, blnAutoCopy, blnAutoDetect, blnRememberSession);
  
  //Check to see if user has told the program to remember the current session
  if(blnRememberSession == FALSE) {
    // Create Annotate dialog with default values
    result = dlgAnnotate.DoModal();
  }
  else {
    result = IDOK; //tell the system that 'OK' was clicked in the dialog
    remember_session = 1;
  }
  
  // image was closed by clicking "Ok"
  if(result == IDOK)
  {
    if(!remember_session) 
    {
      // retrieve the user's checkbox selections in the Annotate dialog
      blnAutoCopy = dlgAnnotate.Get_autocopy_Value();
      blnAutoDetect = dlgAnnotate.Get_autodetect_Value();
      blnRememberSession = dlgAnnotate.Get_remember_session_Value();
    }
    
    // set the image as done in the progress data     
    m_controller.C_SetImageAnnotated_U(true, window, session);
    
    UpdateStatusText();
    
    // copy if box was checked
    if(blnAutoCopy == TRUE)
    {
      CString tubename = m_rfy_data->GetApplicationData().GetCurrentTubeName();
      //session number must be >= 1
      if(session >= 2 /*and less than max sessions?*/)
      {
        for(i = session-1; i >= 1; i--) 
        {
          const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName(tubename);
          if(tube->GetImageExists(window, i) && tube->GetImageAnnotated(window, i))
          {
            m_controller.C_CopySession(i, session, tubename, window, -1);
            break;
          }
        }
      }
      ReDraw(1,0);
    }
    
    // autodetect if box was checked
    if(dlgAnnotate.m_check_autodetect)
    {
      RunAutomaticRootDetector(1);
      ReDraw(1, 0);
    }
    
    EnableDisable();
  }
}

void CRootflyDlg::OnEditAutodetectroots() 
{
  RunAutomaticRootDetector(1);
  ReDraw(1, 0);	
}


// FixAutoDetectResolution removes unnecessary artifacts generated by checking
// the distance between a point and the line segment created by the two points
// surrounding it.  Essentially this function approximates a root instance using
// a reduced number of polylines.  Ideally this would have been done in the root
// detection function itself, but Guang wrote the code so that a large number
// of points are defined for each detected root.
void CRootflyDlg::FixAutoDetectResolution(RootInstance &root, double radius)
{
  int distance;
  vector<pt2d> points(4);
  RPoint *middle_point;
  vector<RPoint> &root_points = *(root.GetAllPoints());
  
  for(int i = 0; i < root_points.size() - 2; i++)
  {
    // Convert points
    RPointToPt2d(root_points.at(i), points.at(0));
    RPointToPt2d(root_points.at(i + 1), points.at(1));
    RPointToPt2d(root_points.at(i + 2), points.at(2));
    
    distance = DistPt2Seg(points.at(0), points.at(2), points.at(1), &(points.at(3)));
    printf("%lf, %lf\n", sqrt(distance), radius);
    
    if(sqrt(distance) < radius / 4)
    {
      middle_point = &(root_points.at(i+1));
      root_points.erase(middle_point);
      i--;
    }
  }
}

// Helper function for FixAutoDetectResolution.  Converts RPoints to pt2d types.
void CRootflyDlg::RPointToPt2d(RPoint pt1, pt2d &pt2)
{
  pt2.x = pt1.x;
  pt2.y = pt1.y;
}

void CRootflyDlg::OnTimer(UINT nIDEvent) 
{
#ifdef USE_AUTOSAVE
  // Autosave was implemented in 2009 but was not reliable (caused crashing).  
  // It was modified in 2010, but there is still no guarantee that it works, b/c there
  // is no mutex.  In our lab testing, it never causes a problem.  But when John McIntyre ran
  // the code, he was experiencing some crashing which may be due to this feature.  
  // -- STB 5/3/10
  
  bool file_loaded = (GetMode() != MODE_NOFILELOADED);
  CString filename = m_rfy_data->GetApplicationData().GetFileName();
  CString autosave_name = GetAutoSaveName(filename);
  CString autosave_tempname = GetAutoSaveName(autosave_name);
  
  // Save the temporary autosave file, delete old autosave, rename temp to autosave
  //if (file_loaded && m_rfy_data->GetApplicationData().GetModify()) CWK
  if (file_loaded)
  {
    m_controller.C_SaveFile(autosave_tempname, false);
    remove(autosave_name);
    rename(autosave_tempname, autosave_name);
  }
#endif
  
  CDialog::OnTimer(nIDEvent);
}

// Set minimum size of window
void CRootflyDlg::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
  lpMMI->ptMinTrackSize.x = 1000;
  lpMMI->ptMinTrackSize.y = 720;
}

// Deselects the selected root if it is no longer being displayed
void CRootflyDlg::CheckWhetherToDeselectRoot()
{
  const Tube *tube = m_rfy_data->GetExperiment().GetTubeByName( m_rfy_data->GetApplicationData().GetCurrentTubeName() );
  if (!tube)  return;
  const Root *root = tube->GetRoot( m_selected_root_id );
  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_rfy_data->GetApplicationData().GetCurrentSession());
  if (!inst)  return;
  
  ///////
  // Deselect Cases
  ///////
  
  bool show_roots = (m_checkbox_show_roots.GetCheck() == BST_CHECKED);
  bool show_dead_roots = (m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED);
  
  if( !show_roots )
  { // if show roots is not checked, deselect root
    DeselectRoot();
  }
  else if( !inst->IsAlive() && !show_dead_roots )
  { // if root is dead and show dead roots is not checked, deselect root
    DeselectRoot();
  }
  
}

void CRootflyDlg::DeselectRoot()
{
  SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  m_selected_root_session = -1;
  // be sure to redraw after calling this function
}
